/****************************************************************************
** Meta object code from reading C++ file 'lc_lineoptions.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/forms/lc_lineoptions.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_lineoptions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LC_LineOptions_t {
    QByteArrayData data[20];
    char stringdata0[249];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LC_LineOptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LC_LineOptions_t qt_meta_stringdata_LC_LineOptions = {
    {
QT_MOC_LITERAL(0, 0, 14), // "LC_LineOptions"
QT_MOC_LITERAL(1, 15, 9), // "closeLine"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 4), // "undo"
QT_MOC_LITERAL(4, 31, 4), // "redo"
QT_MOC_LITERAL(5, 36, 8), // "polyline"
QT_MOC_LITERAL(6, 45, 5), // "start"
QT_MOC_LITERAL(7, 51, 14), // "onAngleClicked"
QT_MOC_LITERAL(8, 66, 5), // "value"
QT_MOC_LITERAL(9, 72, 10), // "onXClicked"
QT_MOC_LITERAL(10, 83, 10), // "onYClicked"
QT_MOC_LITERAL(11, 94, 14), // "onPointClicked"
QT_MOC_LITERAL(12, 109, 10), // "onSetAngle"
QT_MOC_LITERAL(13, 120, 22), // "onAngleRelativeClicked"
QT_MOC_LITERAL(14, 143, 14), // "languageChange"
QT_MOC_LITERAL(15, 158, 20), // "checkActionRttiValid"
QT_MOC_LITERAL(16, 179, 15), // "RS2::ActionType"
QT_MOC_LITERAL(17, 195, 10), // "actionType"
QT_MOC_LITERAL(18, 206, 27), // "getSettingsOptionNamePrefix"
QT_MOC_LITERAL(19, 234, 14) // "doSaveSettings"

    },
    "LC_LineOptions\0closeLine\0\0undo\0redo\0"
    "polyline\0start\0onAngleClicked\0value\0"
    "onXClicked\0onYClicked\0onPointClicked\0"
    "onSetAngle\0onAngleRelativeClicked\0"
    "languageChange\0checkActionRttiValid\0"
    "RS2::ActionType\0actionType\0"
    "getSettingsOptionNamePrefix\0doSaveSettings"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LC_LineOptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   89,    2, 0x0a /* Public */,
       3,    0,   90,    2, 0x0a /* Public */,
       4,    0,   91,    2, 0x0a /* Public */,
       5,    0,   92,    2, 0x0a /* Public */,
       6,    0,   93,    2, 0x0a /* Public */,
       7,    1,   94,    2, 0x09 /* Protected */,
       9,    1,   97,    2, 0x09 /* Protected */,
      10,    1,  100,    2, 0x09 /* Protected */,
      11,    1,  103,    2, 0x09 /* Protected */,
      12,    0,  106,    2, 0x09 /* Protected */,
      13,    1,  107,    2, 0x09 /* Protected */,
      14,    0,  110,    2, 0x09 /* Protected */,
      15,    1,  111,    2, 0x09 /* Protected */,
      18,    0,  114,    2, 0x09 /* Protected */,
      19,    0,  115,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void,
    QMetaType::Bool, 0x80000000 | 16,   17,
    QMetaType::QString,
    QMetaType::Void,

       0        // eod
};

void LC_LineOptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LC_LineOptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closeLine(); break;
        case 1: _t->undo(); break;
        case 2: _t->redo(); break;
        case 3: _t->polyline(); break;
        case 4: _t->start(); break;
        case 5: _t->onAngleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->onXClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->onYClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->onPointClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->onSetAngle(); break;
        case 10: _t->onAngleRelativeClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->languageChange(); break;
        case 12: { bool _r = _t->checkActionRttiValid((*reinterpret_cast< RS2::ActionType(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->getSettingsOptionNamePrefix();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->doSaveSettings(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LC_LineOptions::staticMetaObject = { {
    QMetaObject::SuperData::link<LC_ActionOptionsWidget::staticMetaObject>(),
    qt_meta_stringdata_LC_LineOptions.data,
    qt_meta_data_LC_LineOptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LC_LineOptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LC_LineOptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LC_LineOptions.stringdata0))
        return static_cast<void*>(this);
    return LC_ActionOptionsWidget::qt_metacast(_clname);
}

int LC_LineOptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LC_ActionOptionsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
