/********************************************************************************
** Form generated from reading UI file 'lc_linejoinoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_LINEJOINOPTIONS_H
#define UI_LC_LINEJOINOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_LineJoinOptions
{
public:
    QGridLayout *gridLayout;
    QComboBox *cbLine1EdgeMode;
    QCheckBox *cbPolyline;
    QLabel *label_3;
    QComboBox *cbAttributesSource;
    QComboBox *cbLine2EdgeMode;
    QCheckBox *cbRemoveOriginals;
    QLabel *label_2;
    QLabel *label;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *LC_LineJoinOptions)
    {
        if (LC_LineJoinOptions->objectName().isEmpty())
            LC_LineJoinOptions->setObjectName(QString::fromUtf8("LC_LineJoinOptions"));
        LC_LineJoinOptions->resize(660, 24);
        LC_LineJoinOptions->setMinimumSize(QSize(80, 0));
        gridLayout = new QGridLayout(LC_LineJoinOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        cbLine1EdgeMode = new QComboBox(LC_LineJoinOptions);
        cbLine1EdgeMode->addItem(QString());
        cbLine1EdgeMode->addItem(QString());
        cbLine1EdgeMode->addItem(QString());
        cbLine1EdgeMode->setObjectName(QString::fromUtf8("cbLine1EdgeMode"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(cbLine1EdgeMode->sizePolicy().hasHeightForWidth());
        cbLine1EdgeMode->setSizePolicy(sizePolicy);
        cbLine1EdgeMode->setMinimumSize(QSize(95, 0));
        cbLine1EdgeMode->setMaximumSize(QSize(120, 16777215));

        gridLayout->addWidget(cbLine1EdgeMode, 0, 3, 1, 1);

        cbPolyline = new QCheckBox(LC_LineJoinOptions);
        cbPolyline->setObjectName(QString::fromUtf8("cbPolyline"));

        gridLayout->addWidget(cbPolyline, 0, 8, 1, 1);

        label_3 = new QLabel(LC_LineJoinOptions);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 0, 6, 1, 1);

        cbAttributesSource = new QComboBox(LC_LineJoinOptions);
        cbAttributesSource->addItem(QString());
        cbAttributesSource->addItem(QString());
        cbAttributesSource->addItem(QString());
        cbAttributesSource->addItem(QString());
        cbAttributesSource->setObjectName(QString::fromUtf8("cbAttributesSource"));
        sizePolicy.setHeightForWidth(cbAttributesSource->sizePolicy().hasHeightForWidth());
        cbAttributesSource->setSizePolicy(sizePolicy);
        cbAttributesSource->setMinimumSize(QSize(80, 0));
        cbAttributesSource->setMaximumSize(QSize(90, 16777215));

        gridLayout->addWidget(cbAttributesSource, 0, 7, 1, 1);

        cbLine2EdgeMode = new QComboBox(LC_LineJoinOptions);
        cbLine2EdgeMode->addItem(QString());
        cbLine2EdgeMode->addItem(QString());
        cbLine2EdgeMode->addItem(QString());
        cbLine2EdgeMode->setObjectName(QString::fromUtf8("cbLine2EdgeMode"));
        sizePolicy.setHeightForWidth(cbLine2EdgeMode->sizePolicy().hasHeightForWidth());
        cbLine2EdgeMode->setSizePolicy(sizePolicy);
        cbLine2EdgeMode->setMinimumSize(QSize(95, 0));
        cbLine2EdgeMode->setMaximumSize(QSize(120, 16777215));

        gridLayout->addWidget(cbLine2EdgeMode, 0, 5, 1, 1);

        cbRemoveOriginals = new QCheckBox(LC_LineJoinOptions);
        cbRemoveOriginals->setObjectName(QString::fromUtf8("cbRemoveOriginals"));

        gridLayout->addWidget(cbRemoveOriginals, 0, 9, 1, 1);

        label_2 = new QLabel(LC_LineJoinOptions);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 4, 1, 1);

        label = new QLabel(LC_LineJoinOptions);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 10, 1, 1);


        retranslateUi(LC_LineJoinOptions);

        QMetaObject::connectSlotsByName(LC_LineJoinOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_LineJoinOptions)
    {
        LC_LineJoinOptions->setWindowTitle(QCoreApplication::translate("LC_LineJoinOptions", "Form", nullptr));
        cbLine1EdgeMode->setItemText(0, QCoreApplication::translate("LC_LineJoinOptions", "Extend/Trim", nullptr));
        cbLine1EdgeMode->setItemText(1, QCoreApplication::translate("LC_LineJoinOptions", "Add segment", nullptr));
        cbLine1EdgeMode->setItemText(2, QCoreApplication::translate("LC_LineJoinOptions", "No change", nullptr));

        cbPolyline->setText(QCoreApplication::translate("LC_LineJoinOptions", "Create Polyline", nullptr));
        label_3->setText(QCoreApplication::translate("LC_LineJoinOptions", "Attributes:", nullptr));
        cbAttributesSource->setItemText(0, QCoreApplication::translate("LC_LineJoinOptions", "Active Pen", nullptr));
        cbAttributesSource->setItemText(1, QCoreApplication::translate("LC_LineJoinOptions", "Line 1", nullptr));
        cbAttributesSource->setItemText(2, QCoreApplication::translate("LC_LineJoinOptions", "Line 2", nullptr));
        cbAttributesSource->setItemText(3, QCoreApplication::translate("LC_LineJoinOptions", "Both lines", nullptr));

        cbLine2EdgeMode->setItemText(0, QCoreApplication::translate("LC_LineJoinOptions", "Extend/Trim", nullptr));
        cbLine2EdgeMode->setItemText(1, QCoreApplication::translate("LC_LineJoinOptions", "Add segment", nullptr));
        cbLine2EdgeMode->setItemText(2, QCoreApplication::translate("LC_LineJoinOptions", "No change", nullptr));

        cbRemoveOriginals->setText(QCoreApplication::translate("LC_LineJoinOptions", "Remove originals", nullptr));
        label_2->setText(QCoreApplication::translate("LC_LineJoinOptions", "Line 2:", nullptr));
        label->setText(QCoreApplication::translate("LC_LineJoinOptions", "Line 1:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_LineJoinOptions: public Ui_LC_LineJoinOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_LINEJOINOPTIONS_H
