/****************************************************************************
** Meta object code from reading C++ file 'qc_mdiwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/main/qc_mdiwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qc_mdiwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QC_MDIWindow_t {
    QByteArrayData data[18];
    char stringdata0[195];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QC_MDIWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QC_MDIWindow_t qt_meta_stringdata_QC_MDIWindow = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QC_MDIWindow"
QT_MOC_LITERAL(1, 13, 14), // "slotPenChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 6), // "RS_Pen"
QT_MOC_LITERAL(4, 36, 1), // "p"
QT_MOC_LITERAL(5, 38, 11), // "slotFileNew"
QT_MOC_LITERAL(6, 50, 19), // "slotFileNewTemplate"
QT_MOC_LITERAL(7, 70, 8), // "fileName"
QT_MOC_LITERAL(8, 79, 15), // "RS2::FormatType"
QT_MOC_LITERAL(9, 95, 4), // "type"
QT_MOC_LITERAL(10, 100, 12), // "slotFileOpen"
QT_MOC_LITERAL(11, 113, 12), // "slotFileSave"
QT_MOC_LITERAL(12, 126, 5), // "bool&"
QT_MOC_LITERAL(13, 132, 9), // "cancelled"
QT_MOC_LITERAL(14, 142, 10), // "isAutoSave"
QT_MOC_LITERAL(15, 153, 14), // "slotFileSaveAs"
QT_MOC_LITERAL(16, 168, 13), // "slotFilePrint"
QT_MOC_LITERAL(17, 182, 12) // "slotZoomAuto"

    },
    "QC_MDIWindow\0slotPenChanged\0\0RS_Pen\0"
    "p\0slotFileNew\0slotFileNewTemplate\0"
    "fileName\0RS2::FormatType\0type\0"
    "slotFileOpen\0slotFileSave\0bool&\0"
    "cancelled\0isAutoSave\0slotFileSaveAs\0"
    "slotFilePrint\0slotZoomAuto"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QC_MDIWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x0a /* Public */,
       5,    0,   62,    2, 0x0a /* Public */,
       6,    2,   63,    2, 0x0a /* Public */,
      10,    2,   68,    2, 0x0a /* Public */,
      11,    2,   73,    2, 0x0a /* Public */,
      11,    1,   78,    2, 0x2a /* Public | MethodCloned */,
      15,    1,   81,    2, 0x0a /* Public */,
      16,    0,   84,    2, 0x0a /* Public */,
      17,    0,   85,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 8,    7,    9,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 8,    7,    9,
    QMetaType::Bool, 0x80000000 | 12, QMetaType::Bool,   13,   14,
    QMetaType::Bool, 0x80000000 | 12,   13,
    QMetaType::Bool, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QC_MDIWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QC_MDIWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotPenChanged((*reinterpret_cast< const RS_Pen(*)>(_a[1]))); break;
        case 1: _t->slotFileNew(); break;
        case 2: { bool _r = _t->slotFileNewTemplate((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< RS2::FormatType(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->slotFileOpen((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< RS2::FormatType(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->slotFileSave((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->slotFileSave((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->slotFileSaveAs((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->slotFilePrint(); break;
        case 8: _t->slotZoomAuto(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QC_MDIWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMdiSubWindow::staticMetaObject>(),
    qt_meta_stringdata_QC_MDIWindow.data,
    qt_meta_data_QC_MDIWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QC_MDIWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QC_MDIWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QC_MDIWindow.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "RS_LayerListListener"))
        return static_cast< RS_LayerListListener*>(this);
    if (!strcmp(_clname, "RS_BlockListListener"))
        return static_cast< RS_BlockListListener*>(this);
    return QMdiSubWindow::qt_metacast(_clname);
}

int QC_MDIWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMdiSubWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
