/********************************************************************************
** Form generated from reading UI file 'lc_lineoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_LINEOPTIONS_H
#define UI_LC_LINEOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Ui_LineOptionsRel
{
public:
    QGridLayout *gridLayout_2;
    QLabel *label;
    QSpacerItem *horizontalSpacer;
    QFrame *frame;
    QGridLayout *gridLayout;
    QToolButton *bPolyline;
    QRadioButton *rbY;
    QRadioButton *rbAngle;
    QRadioButton *rbX;
    QFrame *sep2;
    QCheckBox *cbRelAngle;
    QFrame *sep1;
    QToolButton *bUndo;
    QToolButton *bClose;
    QToolButton *bRedo;
    QLineEdit *leAngle;
    QRadioButton *rbPoint;

    void setupUi(QWidget *Ui_LineOptionsRel)
    {
        if (Ui_LineOptionsRel->objectName().isEmpty())
            Ui_LineOptionsRel->setObjectName(QString::fromUtf8("Ui_LineOptionsRel"));
        Ui_LineOptionsRel->resize(556, 24);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Ui_LineOptionsRel->sizePolicy().hasHeightForWidth());
        Ui_LineOptionsRel->setSizePolicy(sizePolicy);
        Ui_LineOptionsRel->setMinimumSize(QSize(0, 0));
        Ui_LineOptionsRel->setMaximumSize(QSize(700, 39));
        gridLayout_2 = new QGridLayout(Ui_LineOptionsRel);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(-1, 1, -1, 1);
        label = new QLabel(Ui_LineOptionsRel);
        label->setObjectName(QString::fromUtf8("label"));
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setMinimumSize(QSize(45, 0));

        gridLayout_2->addWidget(label, 0, 0, 2, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 0, 16, 2, 1);

        frame = new QFrame(Ui_LineOptionsRel);
        frame->setObjectName(QString::fromUtf8("frame"));
        frame->setFrameShape(QFrame::NoFrame);
        frame->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(frame);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setSizeConstraint(QLayout::SetMinimumSize);
        gridLayout->setHorizontalSpacing(3);
        gridLayout->setVerticalSpacing(4);
        gridLayout->setContentsMargins(3, 0, -1, 0);
        bPolyline = new QToolButton(frame);
        bPolyline->setObjectName(QString::fromUtf8("bPolyline"));

        gridLayout->addWidget(bPolyline, 0, 8, 1, 1);

        rbY = new QRadioButton(frame);
        rbY->setObjectName(QString::fromUtf8("rbY"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(rbY->sizePolicy().hasHeightForWidth());
        rbY->setSizePolicy(sizePolicy1);
        rbY->setMinimumSize(QSize(30, 0));

        gridLayout->addWidget(rbY, 0, 1, 1, 1);

        rbAngle = new QRadioButton(frame);
        rbAngle->setObjectName(QString::fromUtf8("rbAngle"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(rbAngle->sizePolicy().hasHeightForWidth());
        rbAngle->setSizePolicy(sizePolicy2);
        rbAngle->setMinimumSize(QSize(50, 0));

        gridLayout->addWidget(rbAngle, 0, 3, 1, 1);

        rbX = new QRadioButton(frame);
        rbX->setObjectName(QString::fromUtf8("rbX"));
        sizePolicy2.setHeightForWidth(rbX->sizePolicy().hasHeightForWidth());
        rbX->setSizePolicy(sizePolicy2);
        rbX->setMinimumSize(QSize(30, 0));

        gridLayout->addWidget(rbX, 0, 0, 1, 1);

        sep2 = new QFrame(frame);
        sep2->setObjectName(QString::fromUtf8("sep2"));
        QSizePolicy sizePolicy3(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(sep2->sizePolicy().hasHeightForWidth());
        sep2->setSizePolicy(sizePolicy3);
        sep2->setMinimumSize(QSize(0, 0));
        sep2->setFrameShape(QFrame::VLine);
        sep2->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(sep2, 0, 6, 1, 1);

        cbRelAngle = new QCheckBox(frame);
        cbRelAngle->setObjectName(QString::fromUtf8("cbRelAngle"));
        sizePolicy2.setHeightForWidth(cbRelAngle->sizePolicy().hasHeightForWidth());
        cbRelAngle->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(cbRelAngle, 0, 5, 1, 1);

        sep1 = new QFrame(frame);
        sep1->setObjectName(QString::fromUtf8("sep1"));
        sizePolicy3.setHeightForWidth(sep1->sizePolicy().hasHeightForWidth());
        sep1->setSizePolicy(sizePolicy3);
        sep1->setMinimumSize(QSize(0, 0));
        sep1->setFrameShape(QFrame::VLine);
        sep1->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(sep1, 0, 9, 1, 1);

        bUndo = new QToolButton(frame);
        bUndo->setObjectName(QString::fromUtf8("bUndo"));

        gridLayout->addWidget(bUndo, 0, 11, 1, 1);

        bClose = new QToolButton(frame);
        bClose->setObjectName(QString::fromUtf8("bClose"));
        bClose->setIconSize(QSize(24, 22));
        bClose->setCheckable(false);

        gridLayout->addWidget(bClose, 0, 7, 1, 1);

        bRedo = new QToolButton(frame);
        bRedo->setObjectName(QString::fromUtf8("bRedo"));

        gridLayout->addWidget(bRedo, 0, 10, 1, 1);

        leAngle = new QLineEdit(frame);
        leAngle->setObjectName(QString::fromUtf8("leAngle"));
        sizePolicy2.setHeightForWidth(leAngle->sizePolicy().hasHeightForWidth());
        leAngle->setSizePolicy(sizePolicy2);
        leAngle->setMinimumSize(QSize(40, 0));
        leAngle->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leAngle, 0, 4, 1, 1);

        rbPoint = new QRadioButton(frame);
        rbPoint->setObjectName(QString::fromUtf8("rbPoint"));
        sizePolicy2.setHeightForWidth(rbPoint->sizePolicy().hasHeightForWidth());
        rbPoint->setSizePolicy(sizePolicy2);
        rbPoint->setMinimumSize(QSize(50, 0));

        gridLayout->addWidget(rbPoint, 0, 2, 1, 1);


        gridLayout_2->addWidget(frame, 0, 1, 2, 1);


        retranslateUi(Ui_LineOptionsRel);
        QObject::connect(bClose, SIGNAL(clicked()), Ui_LineOptionsRel, SLOT(closeLine()));
        QObject::connect(bUndo, SIGNAL(clicked()), Ui_LineOptionsRel, SLOT(undo()));
        QObject::connect(bRedo, SIGNAL(clicked()), Ui_LineOptionsRel, SLOT(redo()));
        QObject::connect(bPolyline, SIGNAL(clicked()), Ui_LineOptionsRel, SLOT(polyline()));

        QMetaObject::connectSlotsByName(Ui_LineOptionsRel);
    } // setupUi

    void retranslateUi(QWidget *Ui_LineOptionsRel)
    {
        Ui_LineOptionsRel->setWindowTitle(QCoreApplication::translate("Ui_LineOptionsRel", "Line Options", nullptr));
        label->setText(QCoreApplication::translate("Ui_LineOptionsRel", "Direction:", nullptr));
        bPolyline->setText(QCoreApplication::translate("Ui_LineOptionsRel", "Polyline", nullptr));
        rbY->setText(QCoreApplication::translate("Ui_LineOptionsRel", "Y", nullptr));
        rbAngle->setText(QCoreApplication::translate("Ui_LineOptionsRel", "Angle", nullptr));
        rbX->setText(QCoreApplication::translate("Ui_LineOptionsRel", "X", nullptr));
#if QT_CONFIG(tooltip)
        cbRelAngle->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        cbRelAngle->setText(QCoreApplication::translate("Ui_LineOptionsRel", "Rel", nullptr));
#if QT_CONFIG(tooltip)
        bUndo->setToolTip(QCoreApplication::translate("Ui_LineOptionsRel", "Undo the last line drawing", nullptr));
#endif // QT_CONFIG(tooltip)
        bUndo->setText(QCoreApplication::translate("Ui_LineOptionsRel", "Undo", nullptr));
#if QT_CONFIG(tooltip)
        bClose->setToolTip(QCoreApplication::translate("Ui_LineOptionsRel", "Form a closed contour from lines drawn", nullptr));
#endif // QT_CONFIG(tooltip)
        bClose->setText(QCoreApplication::translate("Ui_LineOptionsRel", "Close", nullptr));
#if QT_CONFIG(tooltip)
        bRedo->setToolTip(QCoreApplication::translate("Ui_LineOptionsRel", "Redo the last line drawing", nullptr));
#endif // QT_CONFIG(tooltip)
        bRedo->setText(QCoreApplication::translate("Ui_LineOptionsRel", "Redo", nullptr));
        rbPoint->setText(QCoreApplication::translate("Ui_LineOptionsRel", "Point", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Ui_LineOptionsRel: public Ui_Ui_LineOptionsRel {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_LINEOPTIONS_H
