/********************************************************************************
** Form generated from reading UI file 'lc_rectangle3pointsoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_RECTANGLE3POINTSOPTIONS_H
#define UI_LC_RECTANGLE3POINTSOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_Rectangle3PointsOptions
{
public:
    QGridLayout *gridLayout;
    QLineEdit *leAngle;
    QFrame *line;
    QCheckBox *cbPolyline;
    QComboBox *cbEdges;
    QLabel *lblEdges;
    QCheckBox *cbQuadrangle;
    QCheckBox *chkFixedBaseAngle;
    QFrame *line_2;
    QFrame *frmRectSettings;
    QGridLayout *gridLayout_2;
    QLabel *lblX;
    QLineEdit *leX;
    QComboBox *cbCorners;
    QLabel *label;
    QCheckBox *cbSnapRadiusCenter;
    QLabel *lblLenY;
    QLineEdit *leLenY;
    QLineEdit *leRadius;
    QLabel *lblRadius;
    QFrame *frmQuad;
    QGridLayout *gridLayout_3;
    QLineEdit *leInnerAngle;
    QFrame *line_3;
    QCheckBox *cbFixedInnerAngle;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *LC_Rectangle3PointsOptions)
    {
        if (LC_Rectangle3PointsOptions->objectName().isEmpty())
            LC_Rectangle3PointsOptions->setObjectName(QString::fromUtf8("LC_Rectangle3PointsOptions"));
        LC_Rectangle3PointsOptions->resize(1228, 24);
        gridLayout = new QGridLayout(LC_Rectangle3PointsOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        leAngle = new QLineEdit(LC_Rectangle3PointsOptions);
        leAngle->setObjectName(QString::fromUtf8("leAngle"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(leAngle->sizePolicy().hasHeightForWidth());
        leAngle->setSizePolicy(sizePolicy);
        leAngle->setMinimumSize(QSize(30, 0));
        leAngle->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leAngle, 1, 5, 1, 1);

        line = new QFrame(LC_Rectangle3PointsOptions);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::VLine);
        line->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line, 1, 3, 1, 1);

        cbPolyline = new QCheckBox(LC_Rectangle3PointsOptions);
        cbPolyline->setObjectName(QString::fromUtf8("cbPolyline"));

        gridLayout->addWidget(cbPolyline, 1, 6, 1, 1);

        cbEdges = new QComboBox(LC_Rectangle3PointsOptions);
        cbEdges->addItem(QString());
        cbEdges->addItem(QString());
        cbEdges->addItem(QString());
        cbEdges->setObjectName(QString::fromUtf8("cbEdges"));

        gridLayout->addWidget(cbEdges, 1, 10, 1, 1);

        lblEdges = new QLabel(LC_Rectangle3PointsOptions);
        lblEdges->setObjectName(QString::fromUtf8("lblEdges"));

        gridLayout->addWidget(lblEdges, 1, 9, 1, 1);

        cbQuadrangle = new QCheckBox(LC_Rectangle3PointsOptions);
        cbQuadrangle->setObjectName(QString::fromUtf8("cbQuadrangle"));

        gridLayout->addWidget(cbQuadrangle, 1, 0, 1, 1);

        chkFixedBaseAngle = new QCheckBox(LC_Rectangle3PointsOptions);
        chkFixedBaseAngle->setObjectName(QString::fromUtf8("chkFixedBaseAngle"));

        gridLayout->addWidget(chkFixedBaseAngle, 1, 4, 1, 1);

        line_2 = new QFrame(LC_Rectangle3PointsOptions);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::VLine);
        line_2->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line_2, 1, 7, 1, 1);

        frmRectSettings = new QFrame(LC_Rectangle3PointsOptions);
        frmRectSettings->setObjectName(QString::fromUtf8("frmRectSettings"));
        frmRectSettings->setFrameShape(QFrame::NoFrame);
        frmRectSettings->setFrameShadow(QFrame::Raised);
        gridLayout_2 = new QGridLayout(frmRectSettings);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(3, 0, 3, 0);
        lblX = new QLabel(frmRectSettings);
        lblX->setObjectName(QString::fromUtf8("lblX"));

        gridLayout_2->addWidget(lblX, 0, 5, 1, 1);

        leX = new QLineEdit(frmRectSettings);
        leX->setObjectName(QString::fromUtf8("leX"));
        sizePolicy.setHeightForWidth(leX->sizePolicy().hasHeightForWidth());
        leX->setSizePolicy(sizePolicy);
        leX->setMinimumSize(QSize(30, 0));
        leX->setMaximumSize(QSize(60, 16777215));

        gridLayout_2->addWidget(leX, 0, 6, 1, 1);

        cbCorners = new QComboBox(frmRectSettings);
        cbCorners->addItem(QString());
        cbCorners->addItem(QString());
        cbCorners->addItem(QString());
        cbCorners->setObjectName(QString::fromUtf8("cbCorners"));

        gridLayout_2->addWidget(cbCorners, 0, 1, 1, 1);

        label = new QLabel(frmRectSettings);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout_2->addWidget(label, 0, 0, 1, 1);

        cbSnapRadiusCenter = new QCheckBox(frmRectSettings);
        cbSnapRadiusCenter->setObjectName(QString::fromUtf8("cbSnapRadiusCenter"));

        gridLayout_2->addWidget(cbSnapRadiusCenter, 0, 4, 1, 1);

        lblLenY = new QLabel(frmRectSettings);
        lblLenY->setObjectName(QString::fromUtf8("lblLenY"));

        gridLayout_2->addWidget(lblLenY, 0, 7, 1, 1);

        leLenY = new QLineEdit(frmRectSettings);
        leLenY->setObjectName(QString::fromUtf8("leLenY"));
        sizePolicy.setHeightForWidth(leLenY->sizePolicy().hasHeightForWidth());
        leLenY->setSizePolicy(sizePolicy);
        leLenY->setMinimumSize(QSize(30, 0));
        leLenY->setMaximumSize(QSize(60, 16777215));

        gridLayout_2->addWidget(leLenY, 0, 8, 1, 1);

        leRadius = new QLineEdit(frmRectSettings);
        leRadius->setObjectName(QString::fromUtf8("leRadius"));
        sizePolicy.setHeightForWidth(leRadius->sizePolicy().hasHeightForWidth());
        leRadius->setSizePolicy(sizePolicy);
        leRadius->setMinimumSize(QSize(30, 0));
        leRadius->setMaximumSize(QSize(60, 16777215));

        gridLayout_2->addWidget(leRadius, 0, 3, 1, 1);

        lblRadius = new QLabel(frmRectSettings);
        lblRadius->setObjectName(QString::fromUtf8("lblRadius"));

        gridLayout_2->addWidget(lblRadius, 0, 2, 1, 1);


        gridLayout->addWidget(frmRectSettings, 1, 8, 1, 1);

        frmQuad = new QFrame(LC_Rectangle3PointsOptions);
        frmQuad->setObjectName(QString::fromUtf8("frmQuad"));
        frmQuad->setFrameShape(QFrame::NoFrame);
        frmQuad->setFrameShadow(QFrame::Raised);
        gridLayout_3 = new QGridLayout(frmQuad);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setHorizontalSpacing(3);
        gridLayout_3->setContentsMargins(2, 0, 3, 0);
        leInnerAngle = new QLineEdit(frmQuad);
        leInnerAngle->setObjectName(QString::fromUtf8("leInnerAngle"));
        sizePolicy.setHeightForWidth(leInnerAngle->sizePolicy().hasHeightForWidth());
        leInnerAngle->setSizePolicy(sizePolicy);
        leInnerAngle->setMinimumSize(QSize(30, 0));
        leInnerAngle->setMaximumSize(QSize(40, 16777215));

        gridLayout_3->addWidget(leInnerAngle, 0, 3, 1, 1);

        line_3 = new QFrame(frmQuad);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::VLine);
        line_3->setFrameShadow(QFrame::Sunken);

        gridLayout_3->addWidget(line_3, 0, 1, 1, 1);

        cbFixedInnerAngle = new QCheckBox(frmQuad);
        cbFixedInnerAngle->setObjectName(QString::fromUtf8("cbFixedInnerAngle"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(cbFixedInnerAngle->sizePolicy().hasHeightForWidth());
        cbFixedInnerAngle->setSizePolicy(sizePolicy1);

        gridLayout_3->addWidget(cbFixedInnerAngle, 0, 2, 1, 1);


        gridLayout->addWidget(frmQuad, 1, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 11, 1, 1);


        retranslateUi(LC_Rectangle3PointsOptions);

        QMetaObject::connectSlotsByName(LC_Rectangle3PointsOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_Rectangle3PointsOptions)
    {
        LC_Rectangle3PointsOptions->setWindowTitle(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Form", nullptr));
        cbPolyline->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Polyline", nullptr));
        cbEdges->setItemText(0, QCoreApplication::translate("LC_Rectangle3PointsOptions", "Both", nullptr));
        cbEdges->setItemText(1, QCoreApplication::translate("LC_Rectangle3PointsOptions", "Vertical", nullptr));
        cbEdges->setItemText(2, QCoreApplication::translate("LC_Rectangle3PointsOptions", "Horizontal", nullptr));

        lblEdges->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Edges:", nullptr));
        cbQuadrangle->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Quadrangle", nullptr));
        chkFixedBaseAngle->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Fixed Base Angle:", nullptr));
        lblX->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Length X:", nullptr));
        cbCorners->setItemText(0, QCoreApplication::translate("LC_Rectangle3PointsOptions", "Straight", nullptr));
        cbCorners->setItemText(1, QCoreApplication::translate("LC_Rectangle3PointsOptions", "Round", nullptr));
        cbCorners->setItemText(2, QCoreApplication::translate("LC_Rectangle3PointsOptions", "Bevel", nullptr));

        label->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Corners:", nullptr));
        cbSnapRadiusCenter->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Snap shift", nullptr));
        lblLenY->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Length Y:", nullptr));
        lblRadius->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Radius:", nullptr));
        cbFixedInnerAngle->setText(QCoreApplication::translate("LC_Rectangle3PointsOptions", "Fixed Inner Angle:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_Rectangle3PointsOptions: public Ui_LC_Rectangle3PointsOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_RECTANGLE3POINTSOPTIONS_H
