/********************************************************************************
** Form generated from reading UI file 'lc_modifygapoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_MODIFYGAPOPTIONS_H
#define UI_LC_MODIFYGAPOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_ModifyGapOptions
{
public:
    QGridLayout *gridLayout;
    QLineEdit *leSize;
    QLabel *label_4;
    QLabel *label_3;
    QCheckBox *cbFree;
    QLineEdit *leDistance;
    QLabel *label;
    QComboBox *cbLineSnap;
    QComboBox *cbGapSnap;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *LC_ModifyGapOptions)
    {
        if (LC_ModifyGapOptions->objectName().isEmpty())
            LC_ModifyGapOptions->setObjectName(QString::fromUtf8("LC_ModifyGapOptions"));
        LC_ModifyGapOptions->resize(591, 24);
        gridLayout = new QGridLayout(LC_ModifyGapOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        leSize = new QLineEdit(LC_ModifyGapOptions);
        leSize->setObjectName(QString::fromUtf8("leSize"));
        leSize->setMinimumSize(QSize(40, 0));
        leSize->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leSize, 0, 1, 1, 1);

        label_4 = new QLabel(LC_ModifyGapOptions);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout->addWidget(label_4, 0, 7, 1, 1);

        label_3 = new QLabel(LC_ModifyGapOptions);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 0, 5, 1, 1);

        cbFree = new QCheckBox(LC_ModifyGapOptions);
        cbFree->setObjectName(QString::fromUtf8("cbFree"));

        gridLayout->addWidget(cbFree, 0, 2, 1, 1);

        leDistance = new QLineEdit(LC_ModifyGapOptions);
        leDistance->setObjectName(QString::fromUtf8("leDistance"));
        leDistance->setMinimumSize(QSize(40, 0));
        leDistance->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leDistance, 0, 6, 1, 1);

        label = new QLabel(LC_ModifyGapOptions);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        cbLineSnap = new QComboBox(LC_ModifyGapOptions);
        cbLineSnap->addItem(QString());
        cbLineSnap->addItem(QString());
        cbLineSnap->addItem(QString());
        cbLineSnap->addItem(QString());
        cbLineSnap->setObjectName(QString::fromUtf8("cbLineSnap"));

        gridLayout->addWidget(cbLineSnap, 0, 4, 1, 1);

        cbGapSnap = new QComboBox(LC_ModifyGapOptions);
        cbGapSnap->addItem(QString());
        cbGapSnap->addItem(QString());
        cbGapSnap->addItem(QString());
        cbGapSnap->setObjectName(QString::fromUtf8("cbGapSnap"));

        gridLayout->addWidget(cbGapSnap, 0, 9, 1, 1);

        label_2 = new QLabel(LC_ModifyGapOptions);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 3, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 10, 1, 1);


        retranslateUi(LC_ModifyGapOptions);

        QMetaObject::connectSlotsByName(LC_ModifyGapOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_ModifyGapOptions)
    {
        LC_ModifyGapOptions->setWindowTitle(QCoreApplication::translate("LC_ModifyGapOptions", "Form", nullptr));
        label_4->setText(QCoreApplication::translate("LC_ModifyGapOptions", "Gap Snap:", nullptr));
        label_3->setText(QCoreApplication::translate("LC_ModifyGapOptions", "Snap Distance:", nullptr));
        cbFree->setText(QCoreApplication::translate("LC_ModifyGapOptions", "Free", nullptr));
        label->setText(QCoreApplication::translate("LC_ModifyGapOptions", "Gap Size:", nullptr));
        cbLineSnap->setItemText(0, QCoreApplication::translate("LC_ModifyGapOptions", "Free", nullptr));
        cbLineSnap->setItemText(1, QCoreApplication::translate("LC_ModifyGapOptions", "Start", nullptr));
        cbLineSnap->setItemText(2, QCoreApplication::translate("LC_ModifyGapOptions", "Middle", nullptr));
        cbLineSnap->setItemText(3, QCoreApplication::translate("LC_ModifyGapOptions", "End", nullptr));

        cbGapSnap->setItemText(0, QCoreApplication::translate("LC_ModifyGapOptions", "Start", nullptr));
        cbGapSnap->setItemText(1, QCoreApplication::translate("LC_ModifyGapOptions", "Middle", nullptr));
        cbGapSnap->setItemText(2, QCoreApplication::translate("LC_ModifyGapOptions", "End", nullptr));

        label_2->setText(QCoreApplication::translate("LC_ModifyGapOptions", "Line Snap:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_ModifyGapOptions: public Ui_LC_ModifyGapOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_MODIFYGAPOPTIONS_H
