/****************************************************************************
** Meta object code from reading C++ file 'lc_penpalettewidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/lc_penpalettewidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_penpalettewidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LC_PenPaletteWidget_t {
    QByteArrayData data[45];
    char stringdata0[995];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LC_PenPaletteWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LC_PenPaletteWidget_t qt_meta_stringdata_LC_PenPaletteWidget = {
    {
QT_MOC_LITERAL(0, 0, 19), // "LC_PenPaletteWidget"
QT_MOC_LITERAL(1, 20, 6), // "escape"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 14), // "onTableClicked"
QT_MOC_LITERAL(4, 43, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 55, 10), // "modelIndex"
QT_MOC_LITERAL(6, 66, 23), // "onTableSelectionChanged"
QT_MOC_LITERAL(7, 90, 14), // "QItemSelection"
QT_MOC_LITERAL(8, 105, 8), // "selected"
QT_MOC_LITERAL(9, 114, 10), // "deselected"
QT_MOC_LITERAL(10, 125, 18), // "onPenEditorChanged"
QT_MOC_LITERAL(11, 144, 13), // "keyPressEvent"
QT_MOC_LITERAL(12, 158, 10), // "QKeyEvent*"
QT_MOC_LITERAL(13, 169, 1), // "e"
QT_MOC_LITERAL(14, 171, 42), // "fillPenEditorBySelectedEntity..."
QT_MOC_LITERAL(15, 214, 39), // "fillPenEditorBySelectedEntity..."
QT_MOC_LITERAL(16, 254, 28), // "fillPenEditorByPenToolBarPen"
QT_MOC_LITERAL(17, 283, 19), // "invokeOptionsDialog"
QT_MOC_LITERAL(18, 303, 11), // "focusOnFile"
QT_MOC_LITERAL(19, 315, 21), // "createOrUpdatePenItem"
QT_MOC_LITERAL(20, 337, 25), // "applyEditorPenToSelection"
QT_MOC_LITERAL(21, 363, 42), // "selectEntitiesWithAttributesP..."
QT_MOC_LITERAL(22, 406, 39), // "selectEntitiesWithDrawingPenB..."
QT_MOC_LITERAL(23, 446, 26), // "applyEditorPenToPenToolBar"
QT_MOC_LITERAL(24, 473, 19), // "removeActivePenItem"
QT_MOC_LITERAL(25, 493, 27), // "applySelectedPenToSelection"
QT_MOC_LITERAL(26, 521, 48), // "selectEntitiesWithAttributesP..."
QT_MOC_LITERAL(27, 570, 45), // "selectEntitiesWithDrawingPenB..."
QT_MOC_LITERAL(28, 616, 32), // "applySelectedPenItemToPenToolBar"
QT_MOC_LITERAL(29, 649, 21), // "removeSelectedPenItem"
QT_MOC_LITERAL(30, 671, 22), // "removeSelectedPenItems"
QT_MOC_LITERAL(31, 694, 19), // "editSelectedPenItem"
QT_MOC_LITERAL(32, 714, 17), // "filterMaskChanged"
QT_MOC_LITERAL(33, 732, 29), // "onTableViewContextMenuInvoked"
QT_MOC_LITERAL(34, 762, 3), // "pos"
QT_MOC_LITERAL(35, 766, 23), // "onPenEditorColorChanged"
QT_MOC_LITERAL(36, 790, 5), // "index"
QT_MOC_LITERAL(37, 796, 23), // "onPenEditorWidthChanged"
QT_MOC_LITERAL(38, 820, 26), // "onPenEditorLineTypeChanged"
QT_MOC_LITERAL(39, 847, 33), // "applySelectedPenItemToActiveL..."
QT_MOC_LITERAL(40, 881, 26), // "fillPenEditorByActiveLayer"
QT_MOC_LITERAL(41, 908, 27), // "applyEditorPenToActiveLayer"
QT_MOC_LITERAL(42, 936, 14), // "onModelChanged"
QT_MOC_LITERAL(43, 951, 13), // "doDoubleClick"
QT_MOC_LITERAL(44, 965, 29) // "updatePenToolbarByActiveLayer"

    },
    "LC_PenPaletteWidget\0escape\0\0onTableClicked\0"
    "QModelIndex\0modelIndex\0onTableSelectionChanged\0"
    "QItemSelection\0selected\0deselected\0"
    "onPenEditorChanged\0keyPressEvent\0"
    "QKeyEvent*\0e\0fillPenEditorBySelectedEntityAttributesPen\0"
    "fillPenEditorBySelectedEntityDrawingPen\0"
    "fillPenEditorByPenToolBarPen\0"
    "invokeOptionsDialog\0focusOnFile\0"
    "createOrUpdatePenItem\0applyEditorPenToSelection\0"
    "selectEntitiesWithAttributesPenByPenEditor\0"
    "selectEntitiesWithDrawingPenByPenEditor\0"
    "applyEditorPenToPenToolBar\0"
    "removeActivePenItem\0applySelectedPenToSelection\0"
    "selectEntitiesWithAttributesPenBySelectedPenItem\0"
    "selectEntitiesWithDrawingPenBySelectedPenItem\0"
    "applySelectedPenItemToPenToolBar\0"
    "removeSelectedPenItem\0removeSelectedPenItems\0"
    "editSelectedPenItem\0filterMaskChanged\0"
    "onTableViewContextMenuInvoked\0pos\0"
    "onPenEditorColorChanged\0index\0"
    "onPenEditorWidthChanged\0"
    "onPenEditorLineTypeChanged\0"
    "applySelectedPenItemToActiveLayer\0"
    "fillPenEditorByActiveLayer\0"
    "applyEditorPenToActiveLayer\0onModelChanged\0"
    "doDoubleClick\0updatePenToolbarByActiveLayer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LC_PenPaletteWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      34,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  184,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  185,    2, 0x0a /* Public */,
       6,    2,  188,    2, 0x0a /* Public */,
      10,    0,  193,    2, 0x0a /* Public */,
      11,    1,  194,    2, 0x0a /* Public */,
      14,    0,  197,    2, 0x0a /* Public */,
      15,    0,  198,    2, 0x0a /* Public */,
      16,    0,  199,    2, 0x0a /* Public */,
      17,    1,  200,    2, 0x0a /* Public */,
      17,    0,  203,    2, 0x2a /* Public | MethodCloned */,
      19,    0,  204,    2, 0x0a /* Public */,
      20,    0,  205,    2, 0x0a /* Public */,
      21,    0,  206,    2, 0x0a /* Public */,
      22,    0,  207,    2, 0x0a /* Public */,
      23,    0,  208,    2, 0x0a /* Public */,
      24,    0,  209,    2, 0x0a /* Public */,
      25,    0,  210,    2, 0x0a /* Public */,
      26,    0,  211,    2, 0x0a /* Public */,
      27,    0,  212,    2, 0x0a /* Public */,
      28,    0,  213,    2, 0x0a /* Public */,
      29,    0,  214,    2, 0x0a /* Public */,
      30,    0,  215,    2, 0x0a /* Public */,
      31,    0,  216,    2, 0x0a /* Public */,
      32,    0,  217,    2, 0x0a /* Public */,
      33,    1,  218,    2, 0x0a /* Public */,
      35,    1,  221,    2, 0x0a /* Public */,
      37,    1,  224,    2, 0x0a /* Public */,
      38,    1,  227,    2, 0x0a /* Public */,
      39,    0,  230,    2, 0x0a /* Public */,
      40,    0,  231,    2, 0x0a /* Public */,
      41,    0,  232,    2, 0x0a /* Public */,
      42,    0,  233,    2, 0x0a /* Public */,
      43,    0,  234,    2, 0x0a /* Public */,
      44,    0,  235,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 7,    8,    9,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   34,
    QMetaType::Void, QMetaType::Int,   36,
    QMetaType::Void, QMetaType::Int,   36,
    QMetaType::Void, QMetaType::Int,   36,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void LC_PenPaletteWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LC_PenPaletteWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->escape(); break;
        case 1: _t->onTableClicked((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 2: _t->onTableSelectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 3: _t->onPenEditorChanged(); break;
        case 4: _t->keyPressEvent((*reinterpret_cast< QKeyEvent*(*)>(_a[1]))); break;
        case 5: _t->fillPenEditorBySelectedEntityAttributesPen(); break;
        case 6: _t->fillPenEditorBySelectedEntityDrawingPen(); break;
        case 7: _t->fillPenEditorByPenToolBarPen(); break;
        case 8: _t->invokeOptionsDialog((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->invokeOptionsDialog(); break;
        case 10: _t->createOrUpdatePenItem(); break;
        case 11: _t->applyEditorPenToSelection(); break;
        case 12: _t->selectEntitiesWithAttributesPenByPenEditor(); break;
        case 13: _t->selectEntitiesWithDrawingPenByPenEditor(); break;
        case 14: _t->applyEditorPenToPenToolBar(); break;
        case 15: _t->removeActivePenItem(); break;
        case 16: _t->applySelectedPenToSelection(); break;
        case 17: _t->selectEntitiesWithAttributesPenBySelectedPenItem(); break;
        case 18: _t->selectEntitiesWithDrawingPenBySelectedPenItem(); break;
        case 19: _t->applySelectedPenItemToPenToolBar(); break;
        case 20: _t->removeSelectedPenItem(); break;
        case 21: _t->removeSelectedPenItems(); break;
        case 22: _t->editSelectedPenItem(); break;
        case 23: _t->filterMaskChanged(); break;
        case 24: _t->onTableViewContextMenuInvoked((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 25: _t->onPenEditorColorChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 26: _t->onPenEditorWidthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 27: _t->onPenEditorLineTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 28: _t->applySelectedPenItemToActiveLayer(); break;
        case 29: _t->fillPenEditorByActiveLayer(); break;
        case 30: _t->applyEditorPenToActiveLayer(); break;
        case 31: _t->onModelChanged(); break;
        case 32: _t->doDoubleClick(); break;
        case 33: _t->updatePenToolbarByActiveLayer(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (LC_PenPaletteWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LC_PenPaletteWidget::escape)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LC_PenPaletteWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_LC_PenPaletteWidget.data,
    qt_meta_data_LC_PenPaletteWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LC_PenPaletteWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LC_PenPaletteWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LC_PenPaletteWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::LC_PenPaletteWidget"))
        return static_cast< Ui::LC_PenPaletteWidget*>(this);
    return QWidget::qt_metacast(_clname);
}

int LC_PenPaletteWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 34;
    }
    return _id;
}

// SIGNAL 0
void LC_PenPaletteWidget::escape()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
