/****************************************************************************
** Meta object code from reading C++ file 'qg_dlgtext.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/forms/qg_dlgtext.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qg_dlgtext.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QG_DlgText_t {
    QByteArrayData data[37];
    char stringdata0[425];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QG_DlgText_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QG_DlgText_t qt_meta_stringdata_QG_DlgText = {
    {
QT_MOC_LITERAL(0, 0, 10), // "QG_DlgText"
QT_MOC_LITERAL(1, 11, 21), // "updateUniCharComboBox"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 7), // "setText"
QT_MOC_LITERAL(4, 42, 8), // "RS_Text&"
QT_MOC_LITERAL(5, 51, 1), // "t"
QT_MOC_LITERAL(6, 53, 5), // "isNew"
QT_MOC_LITERAL(7, 59, 10), // "updateText"
QT_MOC_LITERAL(8, 70, 14), // "setAlignmentTL"
QT_MOC_LITERAL(9, 85, 14), // "setAlignmentTC"
QT_MOC_LITERAL(10, 100, 14), // "setAlignmentTR"
QT_MOC_LITERAL(11, 115, 14), // "setAlignmentML"
QT_MOC_LITERAL(12, 130, 14), // "setAlignmentMC"
QT_MOC_LITERAL(13, 145, 14), // "setAlignmentMR"
QT_MOC_LITERAL(14, 160, 14), // "setAlignmentLL"
QT_MOC_LITERAL(15, 175, 14), // "setAlignmentLC"
QT_MOC_LITERAL(16, 190, 14), // "setAlignmentLR"
QT_MOC_LITERAL(17, 205, 14), // "setAlignmentBL"
QT_MOC_LITERAL(18, 220, 14), // "setAlignmentBC"
QT_MOC_LITERAL(19, 235, 14), // "setAlignmentBR"
QT_MOC_LITERAL(20, 250, 15), // "setAlignmentFit"
QT_MOC_LITERAL(21, 266, 17), // "setAlignmentAlign"
QT_MOC_LITERAL(22, 284, 18), // "setAlignmentMiddle"
QT_MOC_LITERAL(23, 303, 12), // "setAlignment"
QT_MOC_LITERAL(24, 316, 1), // "a"
QT_MOC_LITERAL(25, 318, 7), // "setFont"
QT_MOC_LITERAL(26, 326, 1), // "f"
QT_MOC_LITERAL(27, 328, 8), // "loadText"
QT_MOC_LITERAL(28, 337, 4), // "load"
QT_MOC_LITERAL(29, 342, 2), // "fn"
QT_MOC_LITERAL(30, 345, 8), // "saveText"
QT_MOC_LITERAL(31, 354, 4), // "save"
QT_MOC_LITERAL(32, 359, 12), // "insertSymbol"
QT_MOC_LITERAL(33, 372, 19), // "updateUniCharButton"
QT_MOC_LITERAL(34, 392, 10), // "insertChar"
QT_MOC_LITERAL(35, 403, 6), // "reject"
QT_MOC_LITERAL(36, 410, 14) // "languageChange"

    },
    "QG_DlgText\0updateUniCharComboBox\0\0"
    "setText\0RS_Text&\0t\0isNew\0updateText\0"
    "setAlignmentTL\0setAlignmentTC\0"
    "setAlignmentTR\0setAlignmentML\0"
    "setAlignmentMC\0setAlignmentMR\0"
    "setAlignmentLL\0setAlignmentLC\0"
    "setAlignmentLR\0setAlignmentBL\0"
    "setAlignmentBC\0setAlignmentBR\0"
    "setAlignmentFit\0setAlignmentAlign\0"
    "setAlignmentMiddle\0setAlignment\0a\0"
    "setFont\0f\0loadText\0load\0fn\0saveText\0"
    "save\0insertSymbol\0updateUniCharButton\0"
    "insertChar\0reject\0languageChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QG_DlgText[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      29,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  159,    2, 0x0a /* Public */,
       3,    2,  162,    2, 0x0a /* Public */,
       7,    0,  167,    2, 0x0a /* Public */,
       8,    0,  168,    2, 0x0a /* Public */,
       9,    0,  169,    2, 0x0a /* Public */,
      10,    0,  170,    2, 0x0a /* Public */,
      11,    0,  171,    2, 0x0a /* Public */,
      12,    0,  172,    2, 0x0a /* Public */,
      13,    0,  173,    2, 0x0a /* Public */,
      14,    0,  174,    2, 0x0a /* Public */,
      15,    0,  175,    2, 0x0a /* Public */,
      16,    0,  176,    2, 0x0a /* Public */,
      17,    0,  177,    2, 0x0a /* Public */,
      18,    0,  178,    2, 0x0a /* Public */,
      19,    0,  179,    2, 0x0a /* Public */,
      20,    0,  180,    2, 0x0a /* Public */,
      21,    0,  181,    2, 0x0a /* Public */,
      22,    0,  182,    2, 0x0a /* Public */,
      23,    1,  183,    2, 0x0a /* Public */,
      25,    1,  186,    2, 0x0a /* Public */,
      27,    0,  189,    2, 0x0a /* Public */,
      28,    1,  190,    2, 0x0a /* Public */,
      30,    0,  193,    2, 0x0a /* Public */,
      31,    1,  194,    2, 0x0a /* Public */,
      32,    1,  197,    2, 0x0a /* Public */,
      33,    1,  200,    2, 0x0a /* Public */,
      34,    0,  203,    2, 0x0a /* Public */,
      35,    0,  204,    2, 0x0a /* Public */,
      36,    0,  205,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Bool,    5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void, QMetaType::QString,   26,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   29,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   29,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QG_DlgText::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QG_DlgText *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateUniCharComboBox((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setText((*reinterpret_cast< RS_Text(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->updateText(); break;
        case 3: _t->setAlignmentTL(); break;
        case 4: _t->setAlignmentTC(); break;
        case 5: _t->setAlignmentTR(); break;
        case 6: _t->setAlignmentML(); break;
        case 7: _t->setAlignmentMC(); break;
        case 8: _t->setAlignmentMR(); break;
        case 9: _t->setAlignmentLL(); break;
        case 10: _t->setAlignmentLC(); break;
        case 11: _t->setAlignmentLR(); break;
        case 12: _t->setAlignmentBL(); break;
        case 13: _t->setAlignmentBC(); break;
        case 14: _t->setAlignmentBR(); break;
        case 15: _t->setAlignmentFit(); break;
        case 16: _t->setAlignmentAlign(); break;
        case 17: _t->setAlignmentMiddle(); break;
        case 18: _t->setAlignment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->setFont((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->loadText(); break;
        case 21: _t->load((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: _t->saveText(); break;
        case 23: _t->save((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 24: _t->insertSymbol((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 25: _t->updateUniCharButton((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 26: _t->insertChar(); break;
        case 27: _t->reject(); break;
        case 28: _t->languageChange(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QG_DlgText::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_QG_DlgText.data,
    qt_meta_data_QG_DlgText,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QG_DlgText::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QG_DlgText::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QG_DlgText.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::QG_DlgText"))
        return static_cast< Ui::QG_DlgText*>(this);
    return QDialog::qt_metacast(_clname);
}

int QG_DlgText::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 29;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
