/****************************************************************************
** Meta object code from reading C++ file 'lc_slicedivideoptions.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/forms/lc_slicedivideoptions.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_slicedivideoptions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LC_SliceDivideOptions_t {
    QByteArrayData data[25];
    char stringdata0[434];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LC_SliceDivideOptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LC_SliceDivideOptions_t qt_meta_stringdata_LC_SliceDivideOptions = {
    {
QT_MOC_LITERAL(0, 0, 21), // "LC_SliceDivideOptions"
QT_MOC_LITERAL(1, 22, 14), // "languageChange"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 20), // "checkActionRttiValid"
QT_MOC_LITERAL(4, 59, 15), // "RS2::ActionType"
QT_MOC_LITERAL(5, 75, 10), // "actionType"
QT_MOC_LITERAL(6, 86, 11), // "doSetAction"
QT_MOC_LITERAL(7, 98, 19), // "RS_ActionInterface*"
QT_MOC_LITERAL(8, 118, 1), // "a"
QT_MOC_LITERAL(9, 120, 6), // "update"
QT_MOC_LITERAL(10, 127, 27), // "getSettingsOptionNamePrefix"
QT_MOC_LITERAL(11, 155, 14), // "onCountChanged"
QT_MOC_LITERAL(12, 170, 5), // "value"
QT_MOC_LITERAL(13, 176, 25), // "onDistanceEditingFinished"
QT_MOC_LITERAL(14, 202, 27), // "onTickLengthEditingFinished"
QT_MOC_LITERAL(15, 230, 26), // "onTickAngleEditingFinished"
QT_MOC_LITERAL(16, 257, 27), // "onTickOffsetEditingFinished"
QT_MOC_LITERAL(17, 285, 33), // "onCircleStartAngleEditingFini..."
QT_MOC_LITERAL(18, 319, 29), // "onDrawTickOnEdgesIndexChanged"
QT_MOC_LITERAL(19, 349, 5), // "index"
QT_MOC_LITERAL(20, 355, 22), // "onTickSnapIndexChanged"
QT_MOC_LITERAL(21, 378, 17), // "onRelAngleClicked"
QT_MOC_LITERAL(22, 396, 7), // "checked"
QT_MOC_LITERAL(23, 404, 15), // "onDivideClicked"
QT_MOC_LITERAL(24, 420, 13) // "onModeClicked"

    },
    "LC_SliceDivideOptions\0languageChange\0"
    "\0checkActionRttiValid\0RS2::ActionType\0"
    "actionType\0doSetAction\0RS_ActionInterface*\0"
    "a\0update\0getSettingsOptionNamePrefix\0"
    "onCountChanged\0value\0onDistanceEditingFinished\0"
    "onTickLengthEditingFinished\0"
    "onTickAngleEditingFinished\0"
    "onTickOffsetEditingFinished\0"
    "onCircleStartAngleEditingFinished\0"
    "onDrawTickOnEdgesIndexChanged\0index\0"
    "onTickSnapIndexChanged\0onRelAngleClicked\0"
    "checked\0onDivideClicked\0onModeClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LC_SliceDivideOptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   89,    2, 0x09 /* Protected */,
       3,    1,   90,    2, 0x09 /* Protected */,
       6,    2,   93,    2, 0x09 /* Protected */,
      10,    0,   98,    2, 0x09 /* Protected */,
      11,    1,   99,    2, 0x09 /* Protected */,
      13,    0,  102,    2, 0x09 /* Protected */,
      14,    0,  103,    2, 0x09 /* Protected */,
      15,    0,  104,    2, 0x09 /* Protected */,
      16,    0,  105,    2, 0x09 /* Protected */,
      17,    0,  106,    2, 0x09 /* Protected */,
      18,    1,  107,    2, 0x09 /* Protected */,
      20,    1,  110,    2, 0x09 /* Protected */,
      21,    1,  113,    2, 0x09 /* Protected */,
      23,    1,  116,    2, 0x09 /* Protected */,
      24,    1,  119,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Bool, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Bool,    8,    9,
    QMetaType::QString,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void, QMetaType::Bool,   22,

       0        // eod
};

void LC_SliceDivideOptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LC_SliceDivideOptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->languageChange(); break;
        case 1: { bool _r = _t->checkActionRttiValid((*reinterpret_cast< RS2::ActionType(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->doSetAction((*reinterpret_cast< RS_ActionInterface*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: { QString _r = _t->getSettingsOptionNamePrefix();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->onCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->onDistanceEditingFinished(); break;
        case 6: _t->onTickLengthEditingFinished(); break;
        case 7: _t->onTickAngleEditingFinished(); break;
        case 8: _t->onTickOffsetEditingFinished(); break;
        case 9: _t->onCircleStartAngleEditingFinished(); break;
        case 10: _t->onDrawTickOnEdgesIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->onTickSnapIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->onRelAngleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->onDivideClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->onModeClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< RS_ActionInterface* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LC_SliceDivideOptions::staticMetaObject = { {
    QMetaObject::SuperData::link<LC_ActionOptionsWidget::staticMetaObject>(),
    qt_meta_stringdata_LC_SliceDivideOptions.data,
    qt_meta_data_LC_SliceDivideOptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LC_SliceDivideOptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LC_SliceDivideOptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LC_SliceDivideOptions.stringdata0))
        return static_cast<void*>(this);
    return LC_ActionOptionsWidget::qt_metacast(_clname);
}

int LC_SliceDivideOptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LC_ActionOptionsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
