/********************************************************************************
** Form generated from reading UI file 'qg_dlgscale.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QG_DLGSCALE_H
#define UI_QG_DLGSCALE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_QG_DlgScale
{
public:
    QGridLayout *gridLayout;
    QGroupBox *bgNumber;
    QVBoxLayout *vboxLayout;
    QRadioButton *rbDeleteOrigin;
    QRadioButton *rbCopy;
    QRadioButton *rbMultiCopy;
    QLineEdit *leNumber;
    QSpacerItem *spacer7;
    QToolButton *toolButton;
    QHBoxLayout *hboxLayout;
    QCheckBox *cbIsotropic;
    QHBoxLayout *horizontalLayout;
    QLabel *lFactor;
    QLineEdit *leFactorX;
    QSpacerItem *horizontalSpacer;
    QLabel *lFactor_2;
    QLineEdit *leFactorY;
    QPushButton *bFindFactor;
    QCheckBox *cbCurrentAttributes;
    QCheckBox *cbCurrentLayer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *QG_DlgScale)
    {
        if (QG_DlgScale->objectName().isEmpty())
            QG_DlgScale->setObjectName(QString::fromUtf8("QG_DlgScale"));
        QG_DlgScale->resize(591, 329);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(QG_DlgScale->sizePolicy().hasHeightForWidth());
        QG_DlgScale->setSizePolicy(sizePolicy);
        QG_DlgScale->setMinimumSize(QSize(300, 190));
        gridLayout = new QGridLayout(QG_DlgScale);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        bgNumber = new QGroupBox(QG_DlgScale);
        bgNumber->setObjectName(QString::fromUtf8("bgNumber"));
        vboxLayout = new QVBoxLayout(bgNumber);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        rbDeleteOrigin = new QRadioButton(bgNumber);
        rbDeleteOrigin->setObjectName(QString::fromUtf8("rbDeleteOrigin"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(rbDeleteOrigin->sizePolicy().hasHeightForWidth());
        rbDeleteOrigin->setSizePolicy(sizePolicy1);
        rbDeleteOrigin->setMinimumSize(QSize(0, 18));

        vboxLayout->addWidget(rbDeleteOrigin);

        rbCopy = new QRadioButton(bgNumber);
        rbCopy->setObjectName(QString::fromUtf8("rbCopy"));
        sizePolicy1.setHeightForWidth(rbCopy->sizePolicy().hasHeightForWidth());
        rbCopy->setSizePolicy(sizePolicy1);
        rbCopy->setMinimumSize(QSize(0, 18));

        vboxLayout->addWidget(rbCopy);

        rbMultiCopy = new QRadioButton(bgNumber);
        rbMultiCopy->setObjectName(QString::fromUtf8("rbMultiCopy"));
        sizePolicy1.setHeightForWidth(rbMultiCopy->sizePolicy().hasHeightForWidth());
        rbMultiCopy->setSizePolicy(sizePolicy1);
        rbMultiCopy->setMinimumSize(QSize(0, 18));

        vboxLayout->addWidget(rbMultiCopy);

        leNumber = new QLineEdit(bgNumber);
        leNumber->setObjectName(QString::fromUtf8("leNumber"));

        vboxLayout->addWidget(leNumber);

        spacer7 = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer7);


        gridLayout->addWidget(bgNumber, 0, 0, 3, 1);

        toolButton = new QToolButton(QG_DlgScale);
        toolButton->setObjectName(QString::fromUtf8("toolButton"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/extui/dlgmodifyscale.png"), QSize(), QIcon::Normal, QIcon::Off);
        toolButton->setIcon(icon);
        toolButton->setIconSize(QSize(80, 80));

        gridLayout->addWidget(toolButton, 0, 1, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        cbIsotropic = new QCheckBox(QG_DlgScale);
        cbIsotropic->setObjectName(QString::fromUtf8("cbIsotropic"));
        cbIsotropic->setChecked(true);

        hboxLayout->addWidget(cbIsotropic);


        gridLayout->addLayout(hboxLayout, 1, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        lFactor = new QLabel(QG_DlgScale);
        lFactor->setObjectName(QString::fromUtf8("lFactor"));
        lFactor->setWordWrap(false);

        horizontalLayout->addWidget(lFactor);

        leFactorX = new QLineEdit(QG_DlgScale);
        leFactorX->setObjectName(QString::fromUtf8("leFactorX"));
        leFactorX->setText(QString::fromUtf8("1.0"));

        horizontalLayout->addWidget(leFactorX);

        horizontalSpacer = new QSpacerItem(19, 36, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        lFactor_2 = new QLabel(QG_DlgScale);
        lFactor_2->setObjectName(QString::fromUtf8("lFactor_2"));
        lFactor_2->setWordWrap(false);

        horizontalLayout->addWidget(lFactor_2);

        leFactorY = new QLineEdit(QG_DlgScale);
        leFactorY->setObjectName(QString::fromUtf8("leFactorY"));
        leFactorY->setText(QString::fromUtf8("1.0"));
        leFactorY->setReadOnly(true);

        horizontalLayout->addWidget(leFactorY);

        bFindFactor = new QPushButton(QG_DlgScale);
        bFindFactor->setObjectName(QString::fromUtf8("bFindFactor"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/extui/modifyscale.png"), QSize(), QIcon::Normal, QIcon::Off);
        bFindFactor->setIcon(icon1);

        horizontalLayout->addWidget(bFindFactor);


        gridLayout->addLayout(horizontalLayout, 2, 1, 1, 1);

        cbCurrentAttributes = new QCheckBox(QG_DlgScale);
        cbCurrentAttributes->setObjectName(QString::fromUtf8("cbCurrentAttributes"));

        gridLayout->addWidget(cbCurrentAttributes, 3, 1, 1, 1);

        cbCurrentLayer = new QCheckBox(QG_DlgScale);
        cbCurrentLayer->setObjectName(QString::fromUtf8("cbCurrentLayer"));

        gridLayout->addWidget(cbCurrentLayer, 4, 1, 1, 1);

        buttonBox = new QDialogButtonBox(QG_DlgScale);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Cancel|QDialogButtonBox::StandardButton::Ok);

        gridLayout->addWidget(buttonBox, 5, 0, 1, 2);


        retranslateUi(QG_DlgScale);
        QObject::connect(rbDeleteOrigin, SIGNAL(toggled(bool)), leNumber, SLOT(setDisabled(bool)));
        QObject::connect(rbCopy, SIGNAL(toggled(bool)), leNumber, SLOT(setDisabled(bool)));
        QObject::connect(rbMultiCopy, SIGNAL(toggled(bool)), leNumber, SLOT(setEnabled(bool)));
        QObject::connect(buttonBox, SIGNAL(accepted()), QG_DlgScale, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), QG_DlgScale, SLOT(reject()));

        QMetaObject::connectSlotsByName(QG_DlgScale);
    } // setupUi

    void retranslateUi(QDialog *QG_DlgScale)
    {
        QG_DlgScale->setWindowTitle(QCoreApplication::translate("QG_DlgScale", "Scaling Options", nullptr));
        bgNumber->setTitle(QCoreApplication::translate("QG_DlgScale", "Number of copies", nullptr));
        rbDeleteOrigin->setText(QCoreApplication::translate("QG_DlgScale", "&Delete Original", nullptr));
        rbCopy->setText(QCoreApplication::translate("QG_DlgScale", "&Keep Original", nullptr));
        rbMultiCopy->setText(QCoreApplication::translate("QG_DlgScale", "&Multiple Copies", nullptr));
        toolButton->setText(QCoreApplication::translate("QG_DlgScale", "...", nullptr));
#if QT_CONFIG(tooltip)
        cbIsotropic->setToolTip(QCoreApplication::translate("QG_DlgScale", "Scale by the same factor at both x- and y- directions", nullptr));
#endif // QT_CONFIG(tooltip)
        cbIsotropic->setText(QCoreApplication::translate("QG_DlgScale", "Isotropic Scaling", nullptr));
        lFactor->setText(QCoreApplication::translate("QG_DlgScale", "X", nullptr));
        lFactor_2->setText(QCoreApplication::translate("QG_DlgScale", "Y", nullptr));
#if QT_CONFIG(tooltip)
        bFindFactor->setToolTip(QCoreApplication::translate("QG_DlgScale", "<html><head/><body><p>Find the scaling ratios around the center by a reference point and a target point.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        bFindFactor->setText(QString());
        cbCurrentAttributes->setText(QCoreApplication::translate("QG_DlgScale", "Use current &attributes", nullptr));
        cbCurrentLayer->setText(QCoreApplication::translate("QG_DlgScale", "Use current &layer", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QG_DlgScale: public Ui_QG_DlgScale {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QG_DLGSCALE_H
