/********************************************************************************
** Form generated from reading UI file 'lc_linefrompointtolineoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_LINEFROMPOINTTOLINEOPTIONS_H
#define UI_LC_LINEFROMPOINTTOLINEOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_LineFromPointToLineOptions
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *horizontalSpacer;
    QLabel *lblAngle;
    QLabel *label_3;
    QCheckBox *cbOrthogonal;
    QLineEdit *leAngle;
    QLabel *label_4;
    QComboBox *cbSizeMode;
    QFrame *frmOffset;
    QGridLayout *gridLayout_3;
    QLabel *lblOffset;
    QLineEdit *leOffset;
    QFrame *frmLength;
    QGridLayout *gridLayout_2;
    QComboBox *cbSnap;
    QLabel *lblLength;
    QLineEdit *leLength;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QWidget *LC_LineFromPointToLineOptions)
    {
        if (LC_LineFromPointToLineOptions->objectName().isEmpty())
            LC_LineFromPointToLineOptions->setObjectName(QString::fromUtf8("LC_LineFromPointToLineOptions"));
        LC_LineFromPointToLineOptions->resize(685, 24);
        gridLayout = new QGridLayout(LC_LineFromPointToLineOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(3, 1, 3, 1);
        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 11, 1, 1);

        lblAngle = new QLabel(LC_LineFromPointToLineOptions);
        lblAngle->setObjectName(QString::fromUtf8("lblAngle"));

        gridLayout->addWidget(lblAngle, 0, 1, 1, 1);

        label_3 = new QLabel(LC_LineFromPointToLineOptions);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 0, 3, 1, 1);

        cbOrthogonal = new QCheckBox(LC_LineFromPointToLineOptions);
        cbOrthogonal->setObjectName(QString::fromUtf8("cbOrthogonal"));

        gridLayout->addWidget(cbOrthogonal, 0, 0, 1, 1);

        leAngle = new QLineEdit(LC_LineFromPointToLineOptions);
        leAngle->setObjectName(QString::fromUtf8("leAngle"));
        leAngle->setMinimumSize(QSize(30, 0));
        leAngle->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leAngle, 0, 2, 1, 1);

        label_4 = new QLabel(LC_LineFromPointToLineOptions);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout->addWidget(label_4, 0, 10, 1, 1);

        cbSizeMode = new QComboBox(LC_LineFromPointToLineOptions);
        cbSizeMode->addItem(QString());
        cbSizeMode->addItem(QString());
        cbSizeMode->setObjectName(QString::fromUtf8("cbSizeMode"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(cbSizeMode->sizePolicy().hasHeightForWidth());
        cbSizeMode->setSizePolicy(sizePolicy);
        cbSizeMode->setMinimumSize(QSize(100, 0));

        gridLayout->addWidget(cbSizeMode, 0, 4, 1, 1);

        frmOffset = new QFrame(LC_LineFromPointToLineOptions);
        frmOffset->setObjectName(QString::fromUtf8("frmOffset"));
        frmOffset->setFrameShape(QFrame::NoFrame);
        frmOffset->setFrameShadow(QFrame::Raised);
        gridLayout_3 = new QGridLayout(frmOffset);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        lblOffset = new QLabel(frmOffset);
        lblOffset->setObjectName(QString::fromUtf8("lblOffset"));

        gridLayout_3->addWidget(lblOffset, 0, 0, 1, 1);

        leOffset = new QLineEdit(frmOffset);
        leOffset->setObjectName(QString::fromUtf8("leOffset"));
        sizePolicy.setHeightForWidth(leOffset->sizePolicy().hasHeightForWidth());
        leOffset->setSizePolicy(sizePolicy);
        leOffset->setMinimumSize(QSize(30, 0));
        leOffset->setMaximumSize(QSize(50, 16777215));

        gridLayout_3->addWidget(leOffset, 0, 1, 1, 1);


        gridLayout->addWidget(frmOffset, 0, 5, 1, 1);

        frmLength = new QFrame(LC_LineFromPointToLineOptions);
        frmLength->setObjectName(QString::fromUtf8("frmLength"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frmLength->sizePolicy().hasHeightForWidth());
        frmLength->setSizePolicy(sizePolicy1);
        frmLength->setFrameShape(QFrame::NoFrame);
        frmLength->setFrameShadow(QFrame::Raised);
        gridLayout_2 = new QGridLayout(frmLength);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(3, 0, 3, 0);
        cbSnap = new QComboBox(frmLength);
        cbSnap->addItem(QString());
        cbSnap->addItem(QString());
        cbSnap->addItem(QString());
        cbSnap->setObjectName(QString::fromUtf8("cbSnap"));
        QSizePolicy sizePolicy2(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(cbSnap->sizePolicy().hasHeightForWidth());
        cbSnap->setSizePolicy(sizePolicy2);
        cbSnap->setMinimumSize(QSize(80, 0));
        cbSnap->setMaximumSize(QSize(80, 16777215));

        gridLayout_2->addWidget(cbSnap, 1, 3, 1, 1);

        lblLength = new QLabel(frmLength);
        lblLength->setObjectName(QString::fromUtf8("lblLength"));
        QSizePolicy sizePolicy3(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(lblLength->sizePolicy().hasHeightForWidth());
        lblLength->setSizePolicy(sizePolicy3);

        gridLayout_2->addWidget(lblLength, 1, 0, 1, 1);

        leLength = new QLineEdit(frmLength);
        leLength->setObjectName(QString::fromUtf8("leLength"));
        sizePolicy.setHeightForWidth(leLength->sizePolicy().hasHeightForWidth());
        leLength->setSizePolicy(sizePolicy);
        leLength->setMinimumSize(QSize(30, 0));
        leLength->setMaximumSize(QSize(40, 16777215));

        gridLayout_2->addWidget(leLength, 1, 1, 1, 1);

        label_2 = new QLabel(frmLength);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout_2->addWidget(label_2, 1, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 1, 4, 1, 1);


        gridLayout->addWidget(frmLength, 0, 7, 1, 1);


        retranslateUi(LC_LineFromPointToLineOptions);

        QMetaObject::connectSlotsByName(LC_LineFromPointToLineOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_LineFromPointToLineOptions)
    {
        LC_LineFromPointToLineOptions->setWindowTitle(QCoreApplication::translate("LC_LineFromPointToLineOptions", "Form", nullptr));
        lblAngle->setText(QCoreApplication::translate("LC_LineFromPointToLineOptions", "Angle:", nullptr));
        label_3->setText(QCoreApplication::translate("LC_LineFromPointToLineOptions", "Size:", nullptr));
        cbOrthogonal->setText(QCoreApplication::translate("LC_LineFromPointToLineOptions", "Orthogonal", nullptr));
        label_4->setText(QString());
        cbSizeMode->setItemText(0, QCoreApplication::translate("LC_LineFromPointToLineOptions", "To Intersection", nullptr));
        cbSizeMode->setItemText(1, QCoreApplication::translate("LC_LineFromPointToLineOptions", "Fixed Length", nullptr));

        lblOffset->setText(QCoreApplication::translate("LC_LineFromPointToLineOptions", "End offset:", nullptr));
        cbSnap->setItemText(0, QCoreApplication::translate("LC_LineFromPointToLineOptions", "Start", nullptr));
        cbSnap->setItemText(1, QCoreApplication::translate("LC_LineFromPointToLineOptions", "Middle", nullptr));
        cbSnap->setItemText(2, QCoreApplication::translate("LC_LineFromPointToLineOptions", "End", nullptr));

        lblLength->setText(QCoreApplication::translate("LC_LineFromPointToLineOptions", "Length:", nullptr));
        label_2->setText(QCoreApplication::translate("LC_LineFromPointToLineOptions", "Snap:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_LineFromPointToLineOptions: public Ui_LC_LineFromPointToLineOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_LINEFROMPOINTTOLINEOPTIONS_H
