/********************************************************************************
** Form generated from reading UI file 'lc_duplicateoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_DUPLICATEOPTIONS_H
#define UI_LC_DUPLICATEOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_DuplicateOptions
{
public:
    QGridLayout *gridLayout;
    QLabel *label_4;
    QCheckBox *cbInPlace;
    QLabel *label;
    QComboBox *cbLayer;
    QComboBox *cbPen;
    QLabel *label_2;
    QLineEdit *leOffsetY;
    QLabel *label_3;
    QLineEdit *leOffsetX;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *LC_DuplicateOptions)
    {
        if (LC_DuplicateOptions->objectName().isEmpty())
            LC_DuplicateOptions->setObjectName(QString::fromUtf8("LC_DuplicateOptions"));
        LC_DuplicateOptions->resize(554, 24);
        gridLayout = new QGridLayout(LC_DuplicateOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        label_4 = new QLabel(LC_DuplicateOptions);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout->addWidget(label_4, 0, 7, 1, 1);

        cbInPlace = new QCheckBox(LC_DuplicateOptions);
        cbInPlace->setObjectName(QString::fromUtf8("cbInPlace"));

        gridLayout->addWidget(cbInPlace, 0, 0, 1, 1);

        label = new QLabel(LC_DuplicateOptions);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 1, 1, 1);

        cbLayer = new QComboBox(LC_DuplicateOptions);
        cbLayer->addItem(QString());
        cbLayer->addItem(QString());
        cbLayer->setObjectName(QString::fromUtf8("cbLayer"));

        gridLayout->addWidget(cbLayer, 0, 8, 1, 1);

        cbPen = new QComboBox(LC_DuplicateOptions);
        cbPen->addItem(QString());
        cbPen->addItem(QString());
        cbPen->addItem(QString());
        cbPen->setObjectName(QString::fromUtf8("cbPen"));

        gridLayout->addWidget(cbPen, 0, 6, 1, 1);

        label_2 = new QLabel(LC_DuplicateOptions);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 3, 1, 1);

        leOffsetY = new QLineEdit(LC_DuplicateOptions);
        leOffsetY->setObjectName(QString::fromUtf8("leOffsetY"));
        leOffsetY->setMinimumSize(QSize(30, 0));
        leOffsetY->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leOffsetY, 0, 4, 1, 1);

        label_3 = new QLabel(LC_DuplicateOptions);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 0, 5, 1, 1);

        leOffsetX = new QLineEdit(LC_DuplicateOptions);
        leOffsetX->setObjectName(QString::fromUtf8("leOffsetX"));
        leOffsetX->setMinimumSize(QSize(30, 0));
        leOffsetX->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leOffsetX, 0, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 9, 1, 1);


        retranslateUi(LC_DuplicateOptions);

        QMetaObject::connectSlotsByName(LC_DuplicateOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_DuplicateOptions)
    {
        LC_DuplicateOptions->setWindowTitle(QCoreApplication::translate("LC_DuplicateOptions", "Form", nullptr));
        label_4->setText(QCoreApplication::translate("LC_DuplicateOptions", "Layer:", nullptr));
        cbInPlace->setText(QCoreApplication::translate("LC_DuplicateOptions", "In Place", nullptr));
        label->setText(QCoreApplication::translate("LC_DuplicateOptions", "Offset X:", nullptr));
        cbLayer->setItemText(0, QCoreApplication::translate("LC_DuplicateOptions", "Active", nullptr));
        cbLayer->setItemText(1, QCoreApplication::translate("LC_DuplicateOptions", "Original", nullptr));

        cbPen->setItemText(0, QCoreApplication::translate("LC_DuplicateOptions", "Active", nullptr));
        cbPen->setItemText(1, QCoreApplication::translate("LC_DuplicateOptions", "Original", nullptr));
        cbPen->setItemText(2, QCoreApplication::translate("LC_DuplicateOptions", "Original Resolved", nullptr));

        label_2->setText(QCoreApplication::translate("LC_DuplicateOptions", "Offset Y:", nullptr));
        label_3->setText(QCoreApplication::translate("LC_DuplicateOptions", "Pen:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_DuplicateOptions: public Ui_LC_DuplicateOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_DUPLICATEOPTIONS_H
