/********************************************************************************
** Form generated from reading UI file 'qg_widgetpen.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QG_WIDGETPEN_H
#define UI_QG_WIDGETPEN_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "qg_colorbox.h"
#include "qg_linetypebox.h"
#include "qg_widthbox.h"

QT_BEGIN_NAMESPACE

class Ui_QG_WidgetPen
{
public:
    QGridLayout *gridLayout;
    QGroupBox *bgPen;
    QGridLayout *gridLayout1;
    QLabel *lColor;
    QLabel *lWidth;
    QLabel *lLineType;
    QSpacerItem *spacer8;
    QG_ColorBox *cbColor;
    QG_WidthBox *cbWidth;
    QG_LineTypeBox *cbLineType;
    QSpacerItem *spacer9;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *horizontalSpacer_3;

    void setupUi(QWidget *QG_WidgetPen)
    {
        if (QG_WidgetPen->objectName().isEmpty())
            QG_WidgetPen->setObjectName(QString::fromUtf8("QG_WidgetPen"));
        QG_WidgetPen->resize(180, 180);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(QG_WidgetPen->sizePolicy().hasHeightForWidth());
        QG_WidgetPen->setSizePolicy(sizePolicy);
        QG_WidgetPen->setMinimumSize(QSize(0, 0));
        gridLayout = new QGridLayout(QG_WidgetPen);
        gridLayout->setSpacing(0);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        bgPen = new QGroupBox(QG_WidgetPen);
        bgPen->setObjectName(QString::fromUtf8("bgPen"));
        gridLayout1 = new QGridLayout(bgPen);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        lColor = new QLabel(bgPen);
        lColor->setObjectName(QString::fromUtf8("lColor"));
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lColor->sizePolicy().hasHeightForWidth());
        lColor->setSizePolicy(sizePolicy1);
        lColor->setWordWrap(false);

        gridLayout1->addWidget(lColor, 0, 0, 1, 1);

        lWidth = new QLabel(bgPen);
        lWidth->setObjectName(QString::fromUtf8("lWidth"));
        sizePolicy1.setHeightForWidth(lWidth->sizePolicy().hasHeightForWidth());
        lWidth->setSizePolicy(sizePolicy1);
        lWidth->setWordWrap(false);

        gridLayout1->addWidget(lWidth, 1, 0, 1, 1);

        lLineType = new QLabel(bgPen);
        lLineType->setObjectName(QString::fromUtf8("lLineType"));
        sizePolicy1.setHeightForWidth(lLineType->sizePolicy().hasHeightForWidth());
        lLineType->setSizePolicy(sizePolicy1);
        lLineType->setWordWrap(false);

        gridLayout1->addWidget(lLineType, 2, 0, 1, 1);

        spacer8 = new QSpacerItem(21, 1, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout1->addItem(spacer8, 3, 0, 1, 1);

        cbColor = new QG_ColorBox(bgPen);
        cbColor->setObjectName(QString::fromUtf8("cbColor"));

        gridLayout1->addWidget(cbColor, 0, 1, 1, 1);

        cbWidth = new QG_WidthBox(bgPen);
        cbWidth->setObjectName(QString::fromUtf8("cbWidth"));

        gridLayout1->addWidget(cbWidth, 1, 1, 1, 1);

        cbLineType = new QG_LineTypeBox(bgPen);
        cbLineType->setObjectName(QString::fromUtf8("cbLineType"));

        gridLayout1->addWidget(cbLineType, 2, 1, 1, 1);

        spacer9 = new QSpacerItem(21, 1, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout1->addItem(spacer9, 3, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout1->addItem(horizontalSpacer, 0, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout1->addItem(horizontalSpacer_2, 1, 2, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout1->addItem(horizontalSpacer_3, 2, 2, 1, 1);


        gridLayout->addWidget(bgPen, 0, 0, 1, 1);


        retranslateUi(QG_WidgetPen);

        QMetaObject::connectSlotsByName(QG_WidgetPen);
    } // setupUi

    void retranslateUi(QWidget *QG_WidgetPen)
    {
        QG_WidgetPen->setWindowTitle(QCoreApplication::translate("QG_WidgetPen", "Pen", nullptr));
        bgPen->setTitle(QCoreApplication::translate("QG_WidgetPen", "Pen", nullptr));
        lColor->setText(QCoreApplication::translate("QG_WidgetPen", "Color:", nullptr));
        lWidth->setText(QCoreApplication::translate("QG_WidgetPen", "Width:", nullptr));
        lLineType->setText(QCoreApplication::translate("QG_WidgetPen", "Line type:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QG_WidgetPen: public Ui_QG_WidgetPen {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QG_WIDGETPEN_H
