/********************************************************************************
** Form generated from reading UI file 'lc_lineanglereloptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_LINEANGLERELOPTIONS_H
#define UI_LC_LINEANGLERELOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_LineAngleRelOptions
{
public:
    QGridLayout *gridLayout;
    QLineEdit *leOffset;
    QLabel *label_2;
    QComboBox *cbLineSnapMode;
    QLabel *leLineSnapMode;
    QCheckBox *cbRelativeAngle;
    QLineEdit *leDistance;
    QComboBox *cbTickSnapMode;
    QLineEdit *leAngle;
    QCheckBox *cbDivide;
    QLineEdit *leLength;
    QLabel *label_3;
    QLabel *label;
    QCheckBox *cbFree;
    QSpacerItem *horizontalSpacer;
    QLabel *lblAngle;
    QLabel *lblDistance;

    void setupUi(QWidget *LC_LineAngleRelOptions)
    {
        if (LC_LineAngleRelOptions->objectName().isEmpty())
            LC_LineAngleRelOptions->setObjectName(QString::fromUtf8("LC_LineAngleRelOptions"));
        LC_LineAngleRelOptions->resize(882, 24);
        gridLayout = new QGridLayout(LC_LineAngleRelOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        leOffset = new QLineEdit(LC_LineAngleRelOptions);
        leOffset->setObjectName(QString::fromUtf8("leOffset"));
        leOffset->setMinimumSize(QSize(30, 0));
        leOffset->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leOffset, 0, 15, 1, 1);

        label_2 = new QLabel(LC_LineAngleRelOptions);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 14, 1, 1);

        cbLineSnapMode = new QComboBox(LC_LineAngleRelOptions);
        cbLineSnapMode->addItem(QString());
        cbLineSnapMode->addItem(QString());
        cbLineSnapMode->addItem(QString());
        cbLineSnapMode->addItem(QString());
        cbLineSnapMode->setObjectName(QString::fromUtf8("cbLineSnapMode"));

        gridLayout->addWidget(cbLineSnapMode, 0, 9, 1, 1);

        leLineSnapMode = new QLabel(LC_LineAngleRelOptions);
        leLineSnapMode->setObjectName(QString::fromUtf8("leLineSnapMode"));

        gridLayout->addWidget(leLineSnapMode, 0, 8, 1, 1);

        cbRelativeAngle = new QCheckBox(LC_LineAngleRelOptions);
        cbRelativeAngle->setObjectName(QString::fromUtf8("cbRelativeAngle"));

        gridLayout->addWidget(cbRelativeAngle, 0, 5, 1, 1);

        leDistance = new QLineEdit(LC_LineAngleRelOptions);
        leDistance->setObjectName(QString::fromUtf8("leDistance"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(leDistance->sizePolicy().hasHeightForWidth());
        leDistance->setSizePolicy(sizePolicy);
        leDistance->setMinimumSize(QSize(30, 0));
        leDistance->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leDistance, 0, 11, 1, 1);

        cbTickSnapMode = new QComboBox(LC_LineAngleRelOptions);
        cbTickSnapMode->addItem(QString());
        cbTickSnapMode->addItem(QString());
        cbTickSnapMode->addItem(QString());
        cbTickSnapMode->setObjectName(QString::fromUtf8("cbTickSnapMode"));

        gridLayout->addWidget(cbTickSnapMode, 0, 13, 1, 1);

        leAngle = new QLineEdit(LC_LineAngleRelOptions);
        leAngle->setObjectName(QString::fromUtf8("leAngle"));
        leAngle->setMinimumSize(QSize(30, 0));
        leAngle->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leAngle, 0, 4, 1, 1);

        cbDivide = new QCheckBox(LC_LineAngleRelOptions);
        cbDivide->setObjectName(QString::fromUtf8("cbDivide"));

        gridLayout->addWidget(cbDivide, 0, 16, 1, 1);

        leLength = new QLineEdit(LC_LineAngleRelOptions);
        leLength->setObjectName(QString::fromUtf8("leLength"));
        leLength->setMinimumSize(QSize(30, 0));
        leLength->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leLength, 0, 1, 1, 1);

        label_3 = new QLabel(LC_LineAngleRelOptions);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 0, 12, 1, 1);

        label = new QLabel(LC_LineAngleRelOptions);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        cbFree = new QCheckBox(LC_LineAngleRelOptions);
        cbFree->setObjectName(QString::fromUtf8("cbFree"));

        gridLayout->addWidget(cbFree, 0, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 17, 1, 1);

        lblAngle = new QLabel(LC_LineAngleRelOptions);
        lblAngle->setObjectName(QString::fromUtf8("lblAngle"));

        gridLayout->addWidget(lblAngle, 0, 3, 1, 1);

        lblDistance = new QLabel(LC_LineAngleRelOptions);
        lblDistance->setObjectName(QString::fromUtf8("lblDistance"));

        gridLayout->addWidget(lblDistance, 0, 10, 1, 1);


        retranslateUi(LC_LineAngleRelOptions);

        QMetaObject::connectSlotsByName(LC_LineAngleRelOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_LineAngleRelOptions)
    {
        LC_LineAngleRelOptions->setWindowTitle(QCoreApplication::translate("LC_LineAngleRelOptions", "Form", nullptr));
        label_2->setText(QCoreApplication::translate("LC_LineAngleRelOptions", "Offset:", nullptr));
        cbLineSnapMode->setItemText(0, QCoreApplication::translate("LC_LineAngleRelOptions", "Free", nullptr));
        cbLineSnapMode->setItemText(1, QCoreApplication::translate("LC_LineAngleRelOptions", "Start", nullptr));
        cbLineSnapMode->setItemText(2, QCoreApplication::translate("LC_LineAngleRelOptions", "Middle", nullptr));
        cbLineSnapMode->setItemText(3, QCoreApplication::translate("LC_LineAngleRelOptions", "End", nullptr));

        leLineSnapMode->setText(QCoreApplication::translate("LC_LineAngleRelOptions", "Line Snap:", nullptr));
        cbRelativeAngle->setText(QCoreApplication::translate("LC_LineAngleRelOptions", "Rel", nullptr));
        cbTickSnapMode->setItemText(0, QCoreApplication::translate("LC_LineAngleRelOptions", "Start", nullptr));
        cbTickSnapMode->setItemText(1, QCoreApplication::translate("LC_LineAngleRelOptions", "Middle", nullptr));
        cbTickSnapMode->setItemText(2, QCoreApplication::translate("LC_LineAngleRelOptions", "End", nullptr));

        cbDivide->setText(QCoreApplication::translate("LC_LineAngleRelOptions", "Divide", nullptr));
        label_3->setText(QCoreApplication::translate("LC_LineAngleRelOptions", "Tick Snap:", nullptr));
        label->setText(QCoreApplication::translate("LC_LineAngleRelOptions", "Length:", nullptr));
        cbFree->setText(QCoreApplication::translate("LC_LineAngleRelOptions", "Free", nullptr));
        lblAngle->setText(QCoreApplication::translate("LC_LineAngleRelOptions", "Angle:", nullptr));
        lblDistance->setText(QCoreApplication::translate("LC_LineAngleRelOptions", "Snap Distance:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_LineAngleRelOptions: public Ui_LC_LineAngleRelOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_LINEANGLERELOPTIONS_H
