/****************************************************************************
** Meta object code from reading C++ file 'qg_dlgoptionsdrawing.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/forms/qg_dlgoptionsdrawing.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qg_dlgoptionsdrawing.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QG_DlgOptionsDrawing_t {
    QByteArrayData data[27];
    char stringdata0[493];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QG_DlgOptionsDrawing_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QG_DlgOptionsDrawing_t qt_meta_stringdata_QG_DlgOptionsDrawing = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QG_DlgOptionsDrawing"
QT_MOC_LITERAL(1, 21, 10), // "setGraphic"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 11), // "RS_Graphic*"
QT_MOC_LITERAL(4, 45, 1), // "g"
QT_MOC_LITERAL(5, 47, 8), // "validate"
QT_MOC_LITERAL(6, 56, 21), // "updateLengthPrecision"
QT_MOC_LITERAL(7, 78, 20), // "updateAnglePrecision"
QT_MOC_LITERAL(8, 99, 13), // "updatePreview"
QT_MOC_LITERAL(9, 113, 15), // "updatePaperSize"
QT_MOC_LITERAL(10, 129, 16), // "updateUnitLabels"
QT_MOC_LITERAL(11, 146, 24), // "updateDimLengthPrecision"
QT_MOC_LITERAL(12, 171, 23), // "updateDimAnglePrecision"
QT_MOC_LITERAL(13, 195, 18), // "updatePaperPreview"
QT_MOC_LITERAL(14, 214, 14), // "languageChange"
QT_MOC_LITERAL(15, 229, 26), // "on_rbIsometricGrid_clicked"
QT_MOC_LITERAL(16, 256, 26), // "on_rbCrosshairLeft_toggled"
QT_MOC_LITERAL(17, 283, 7), // "checked"
QT_MOC_LITERAL(18, 291, 25), // "on_rbCrosshairTop_toggled"
QT_MOC_LITERAL(19, 317, 27), // "on_rbCrosshairRight_toggled"
QT_MOC_LITERAL(20, 345, 27), // "on_rbOrthogonalGrid_clicked"
QT_MOC_LITERAL(21, 373, 19), // "on_cbGridOn_toggled"
QT_MOC_LITERAL(22, 393, 22), // "on_rbLandscape_toggled"
QT_MOC_LITERAL(23, 416, 21), // "on_cbDimFxLon_toggled"
QT_MOC_LITERAL(24, 438, 27), // "on_tabWidget_currentChanged"
QT_MOC_LITERAL(25, 466, 5), // "index"
QT_MOC_LITERAL(26, 472, 20) // "on_rbRelSize_toggled"

    },
    "QG_DlgOptionsDrawing\0setGraphic\0\0"
    "RS_Graphic*\0g\0validate\0updateLengthPrecision\0"
    "updateAnglePrecision\0updatePreview\0"
    "updatePaperSize\0updateUnitLabels\0"
    "updateDimLengthPrecision\0"
    "updateDimAnglePrecision\0updatePaperPreview\0"
    "languageChange\0on_rbIsometricGrid_clicked\0"
    "on_rbCrosshairLeft_toggled\0checked\0"
    "on_rbCrosshairTop_toggled\0"
    "on_rbCrosshairRight_toggled\0"
    "on_rbOrthogonalGrid_clicked\0"
    "on_cbGridOn_toggled\0on_rbLandscape_toggled\0"
    "on_cbDimFxLon_toggled\0on_tabWidget_currentChanged\0"
    "index\0on_rbRelSize_toggled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QG_DlgOptionsDrawing[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  119,    2, 0x0a /* Public */,
       5,    0,  122,    2, 0x0a /* Public */,
       6,    0,  123,    2, 0x0a /* Public */,
       7,    0,  124,    2, 0x0a /* Public */,
       8,    0,  125,    2, 0x0a /* Public */,
       9,    0,  126,    2, 0x0a /* Public */,
      10,    0,  127,    2, 0x0a /* Public */,
      11,    0,  128,    2, 0x0a /* Public */,
      12,    0,  129,    2, 0x0a /* Public */,
      13,    0,  130,    2, 0x0a /* Public */,
      14,    0,  131,    2, 0x09 /* Protected */,
      15,    0,  132,    2, 0x08 /* Private */,
      16,    1,  133,    2, 0x08 /* Private */,
      18,    1,  136,    2, 0x08 /* Private */,
      19,    1,  139,    2, 0x08 /* Private */,
      20,    0,  142,    2, 0x08 /* Private */,
      21,    1,  143,    2, 0x08 /* Private */,
      22,    1,  146,    2, 0x08 /* Private */,
      23,    1,  149,    2, 0x08 /* Private */,
      24,    1,  152,    2, 0x08 /* Private */,
      26,    1,  155,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, QMetaType::Bool,   17,

       0        // eod
};

void QG_DlgOptionsDrawing::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QG_DlgOptionsDrawing *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setGraphic((*reinterpret_cast< RS_Graphic*(*)>(_a[1]))); break;
        case 1: _t->validate(); break;
        case 2: _t->updateLengthPrecision(); break;
        case 3: _t->updateAnglePrecision(); break;
        case 4: _t->updatePreview(); break;
        case 5: _t->updatePaperSize(); break;
        case 6: _t->updateUnitLabels(); break;
        case 7: _t->updateDimLengthPrecision(); break;
        case 8: _t->updateDimAnglePrecision(); break;
        case 9: _t->updatePaperPreview(); break;
        case 10: _t->languageChange(); break;
        case 11: _t->on_rbIsometricGrid_clicked(); break;
        case 12: _t->on_rbCrosshairLeft_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->on_rbCrosshairTop_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->on_rbCrosshairRight_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->on_rbOrthogonalGrid_clicked(); break;
        case 16: _t->on_cbGridOn_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->on_rbLandscape_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->on_cbDimFxLon_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->on_tabWidget_currentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->on_rbRelSize_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QG_DlgOptionsDrawing::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_QG_DlgOptionsDrawing.data,
    qt_meta_data_QG_DlgOptionsDrawing,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QG_DlgOptionsDrawing::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QG_DlgOptionsDrawing::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QG_DlgOptionsDrawing.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::QG_DlgOptionsDrawing"))
        return static_cast< Ui::QG_DlgOptionsDrawing*>(this);
    return QDialog::qt_metacast(_clname);
}

int QG_DlgOptionsDrawing::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
