/****************************************************************************
** Meta object code from reading C++ file 'lc_linepointsoptions.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/forms/lc_linepointsoptions.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_linepointsoptions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LC_LinePointsOptions_t {
    QByteArrayData data[12];
    char stringdata0[224];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LC_LinePointsOptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LC_LinePointsOptions_t qt_meta_stringdata_LC_LinePointsOptions = {
    {
QT_MOC_LITERAL(0, 0, 20), // "LC_LinePointsOptions"
QT_MOC_LITERAL(1, 21, 25), // "onPointsCountValueChanged"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 5), // "value"
QT_MOC_LITERAL(4, 54, 28), // "onEdgePointsModeIndexChanged"
QT_MOC_LITERAL(5, 83, 5), // "index"
QT_MOC_LITERAL(6, 89, 22), // "onFixedDistanceClicked"
QT_MOC_LITERAL(7, 112, 14), // "onAngleClicked"
QT_MOC_LITERAL(8, 127, 19), // "onWithinLineClicked"
QT_MOC_LITERAL(9, 147, 25), // "onDistanceEditingFinished"
QT_MOC_LITERAL(10, 173, 22), // "onAngleEditingFinished"
QT_MOC_LITERAL(11, 196, 27) // "getSettingsOptionNamePrefix"

    },
    "LC_LinePointsOptions\0onPointsCountValueChanged\0"
    "\0value\0onEdgePointsModeIndexChanged\0"
    "index\0onFixedDistanceClicked\0"
    "onAngleClicked\0onWithinLineClicked\0"
    "onDistanceEditingFinished\0"
    "onAngleEditingFinished\0"
    "getSettingsOptionNamePrefix"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LC_LinePointsOptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x09 /* Protected */,
       4,    1,   57,    2, 0x09 /* Protected */,
       6,    1,   60,    2, 0x09 /* Protected */,
       7,    1,   63,    2, 0x09 /* Protected */,
       8,    1,   66,    2, 0x09 /* Protected */,
       9,    0,   69,    2, 0x09 /* Protected */,
      10,    0,   70,    2, 0x09 /* Protected */,
      11,    0,   71,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QString,

       0        // eod
};

void LC_LinePointsOptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LC_LinePointsOptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onPointsCountValueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->onEdgePointsModeIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onFixedDistanceClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onAngleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->onWithinLineClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->onDistanceEditingFinished(); break;
        case 6: _t->onAngleEditingFinished(); break;
        case 7: { QString _r = _t->getSettingsOptionNamePrefix();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LC_LinePointsOptions::staticMetaObject = { {
    QMetaObject::SuperData::link<LC_ActionOptionsWidget::staticMetaObject>(),
    qt_meta_stringdata_LC_LinePointsOptions.data,
    qt_meta_data_LC_LinePointsOptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LC_LinePointsOptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LC_LinePointsOptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LC_LinePointsOptions.stringdata0))
        return static_cast<void*>(this);
    return LC_ActionOptionsWidget::qt_metacast(_clname);
}

int LC_LinePointsOptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LC_ActionOptionsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
