/********************************************************************************
** Form generated from reading UI file 'lc_modifybreakdivideoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_MODIFYBREAKDIVIDEOPTIONS_H
#define UI_LC_MODIFYBREAKDIVIDEOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_ModifyBreakDivideOptions
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *horizontalSpacer;
    QCheckBox *cbRemoveSegments;
    QCheckBox *cbRemoveSelected;

    void setupUi(QWidget *LC_ModifyBreakDivideOptions)
    {
        if (LC_ModifyBreakDivideOptions->objectName().isEmpty())
            LC_ModifyBreakDivideOptions->setObjectName(QString::fromUtf8("LC_ModifyBreakDivideOptions"));
        LC_ModifyBreakDivideOptions->resize(255, 22);
        gridLayout = new QGridLayout(LC_ModifyBreakDivideOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 2, 1, 1);

        cbRemoveSegments = new QCheckBox(LC_ModifyBreakDivideOptions);
        cbRemoveSegments->setObjectName(QString::fromUtf8("cbRemoveSegments"));

        gridLayout->addWidget(cbRemoveSegments, 0, 0, 1, 1);

        cbRemoveSelected = new QCheckBox(LC_ModifyBreakDivideOptions);
        cbRemoveSelected->setObjectName(QString::fromUtf8("cbRemoveSelected"));

        gridLayout->addWidget(cbRemoveSelected, 0, 1, 1, 1);


        retranslateUi(LC_ModifyBreakDivideOptions);

        QMetaObject::connectSlotsByName(LC_ModifyBreakDivideOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_ModifyBreakDivideOptions)
    {
        LC_ModifyBreakDivideOptions->setWindowTitle(QCoreApplication::translate("LC_ModifyBreakDivideOptions", "Form", nullptr));
        cbRemoveSegments->setText(QCoreApplication::translate("LC_ModifyBreakDivideOptions", "Remove Segments", nullptr));
        cbRemoveSelected->setText(QCoreApplication::translate("LC_ModifyBreakDivideOptions", "Remove Selected", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_ModifyBreakDivideOptions: public Ui_LC_ModifyBreakDivideOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_MODIFYBREAKDIVIDEOPTIONS_H
