/****************************************************************************
** Meta object code from reading C++ file 'lc_layertreewidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/lc_layertreewidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_layertreewidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LC_LayerTreeWidget_t {
    QByteArrayData data[27];
    char stringdata0[485];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LC_LayerTreeWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LC_LayerTreeWidget_t qt_meta_stringdata_LC_LayerTreeWidget = {
    {
QT_MOC_LITERAL(0, 0, 18), // "LC_LayerTreeWidget"
QT_MOC_LITERAL(1, 19, 6), // "escape"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 15), // "slotTreeClicked"
QT_MOC_LITERAL(4, 43, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 55, 8), // "layerIdx"
QT_MOC_LITERAL(6, 64, 21), // "slotTreeDoubleClicked"
QT_MOC_LITERAL(7, 86, 24), // "slotFilteringMaskChanged"
QT_MOC_LITERAL(8, 111, 15), // "expandAllLayers"
QT_MOC_LITERAL(9, 127, 17), // "collapseAllLayers"
QT_MOC_LITERAL(10, 145, 23), // "collapseSecondaryLayers"
QT_MOC_LITERAL(11, 169, 19), // "onCustomContextMenu"
QT_MOC_LITERAL(12, 189, 5), // "point"
QT_MOC_LITERAL(13, 195, 28), // "addChildLayerForSelectedItem"
QT_MOC_LITERAL(14, 224, 17), // "editSelectedLayer"
QT_MOC_LITERAL(15, 242, 36), // "addInformationalLayerForSelec..."
QT_MOC_LITERAL(16, 279, 45), // "addAddAlternativePositionLaye..."
QT_MOC_LITERAL(17, 325, 13), // "showAllLayers"
QT_MOC_LITERAL(18, 339, 13), // "hideAllLayers"
QT_MOC_LITERAL(19, 353, 27), // "hideOtherThanSelectedLayers"
QT_MOC_LITERAL(20, 381, 15), // "unlockAllLayers"
QT_MOC_LITERAL(21, 397, 13), // "lockAllLayers"
QT_MOC_LITERAL(22, 411, 14), // "printAllLayers"
QT_MOC_LITERAL(23, 426, 16), // "noPrintAllLayers"
QT_MOC_LITERAL(24, 443, 8), // "addLayer"
QT_MOC_LITERAL(25, 452, 14), // "toggleFlatView"
QT_MOC_LITERAL(26, 467, 17) // "removeEmptyLayers"

    },
    "LC_LayerTreeWidget\0escape\0\0slotTreeClicked\0"
    "QModelIndex\0layerIdx\0slotTreeDoubleClicked\0"
    "slotFilteringMaskChanged\0expandAllLayers\0"
    "collapseAllLayers\0collapseSecondaryLayers\0"
    "onCustomContextMenu\0point\0"
    "addChildLayerForSelectedItem\0"
    "editSelectedLayer\0"
    "addInformationalLayerForSelectedItem\0"
    "addAddAlternativePositionLayerForSelectedItem\0"
    "showAllLayers\0hideAllLayers\0"
    "hideOtherThanSelectedLayers\0unlockAllLayers\0"
    "lockAllLayers\0printAllLayers\0"
    "noPrintAllLayers\0addLayer\0toggleFlatView\0"
    "removeEmptyLayers"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LC_LayerTreeWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      22,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  124,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  125,    2, 0x0a /* Public */,
       6,    1,  128,    2, 0x0a /* Public */,
       7,    0,  131,    2, 0x0a /* Public */,
       8,    0,  132,    2, 0x0a /* Public */,
       9,    0,  133,    2, 0x0a /* Public */,
      10,    0,  134,    2, 0x0a /* Public */,
      11,    1,  135,    2, 0x0a /* Public */,
      13,    0,  138,    2, 0x0a /* Public */,
      14,    0,  139,    2, 0x0a /* Public */,
      15,    0,  140,    2, 0x0a /* Public */,
      16,    0,  141,    2, 0x0a /* Public */,
      17,    0,  142,    2, 0x0a /* Public */,
      18,    0,  143,    2, 0x0a /* Public */,
      19,    0,  144,    2, 0x0a /* Public */,
      20,    0,  145,    2, 0x0a /* Public */,
      21,    0,  146,    2, 0x0a /* Public */,
      22,    0,  147,    2, 0x0a /* Public */,
      23,    0,  148,    2, 0x0a /* Public */,
      24,    0,  149,    2, 0x0a /* Public */,
      25,    0,  150,    2, 0x0a /* Public */,
      26,    0,  151,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void LC_LayerTreeWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LC_LayerTreeWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->escape(); break;
        case 1: _t->slotTreeClicked((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 2: _t->slotTreeDoubleClicked((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 3: _t->slotFilteringMaskChanged(); break;
        case 4: _t->expandAllLayers(); break;
        case 5: _t->collapseAllLayers(); break;
        case 6: _t->collapseSecondaryLayers(); break;
        case 7: _t->onCustomContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 8: _t->addChildLayerForSelectedItem(); break;
        case 9: _t->editSelectedLayer(); break;
        case 10: _t->addInformationalLayerForSelectedItem(); break;
        case 11: _t->addAddAlternativePositionLayerForSelectedItem(); break;
        case 12: _t->showAllLayers(); break;
        case 13: _t->hideAllLayers(); break;
        case 14: _t->hideOtherThanSelectedLayers(); break;
        case 15: _t->unlockAllLayers(); break;
        case 16: _t->lockAllLayers(); break;
        case 17: _t->printAllLayers(); break;
        case 18: _t->noPrintAllLayers(); break;
        case 19: _t->addLayer(); break;
        case 20: _t->toggleFlatView(); break;
        case 21: _t->removeEmptyLayers(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (LC_LayerTreeWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LC_LayerTreeWidget::escape)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LC_LayerTreeWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_LC_LayerTreeWidget.data,
    qt_meta_data_LC_LayerTreeWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LC_LayerTreeWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LC_LayerTreeWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LC_LayerTreeWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "RS_LayerListListener"))
        return static_cast< RS_LayerListListener*>(this);
    return QWidget::qt_metacast(_clname);
}

int LC_LayerTreeWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void LC_LayerTreeWidget::escape()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
