/****************************************************************************
** Meta object code from reading C++ file 'qg_dlgoptionsgeneral.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/forms/qg_dlgoptionsgeneral.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qg_dlgoptionsgeneral.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QG_DlgOptionsGeneral_t {
    QByteArrayData data[25];
    char stringdata0[460];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QG_DlgOptionsGeneral_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QG_DlgOptionsGeneral_t qt_meta_stringdata_QG_DlgOptionsGeneral = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QG_DlgOptionsGeneral"
QT_MOC_LITERAL(1, 21, 16), // "setRestartNeeded"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 2), // "ok"
QT_MOC_LITERAL(4, 42, 19), // "onAutoBackupChanged"
QT_MOC_LITERAL(5, 62, 5), // "state"
QT_MOC_LITERAL(6, 68, 14), // "languageChange"
QT_MOC_LITERAL(7, 83, 15), // "setTemplateFile"
QT_MOC_LITERAL(8, 99, 14), // "setLibraryPath"
QT_MOC_LITERAL(9, 114, 27), // "on_tabWidget_currentChanged"
QT_MOC_LITERAL(10, 142, 5), // "index"
QT_MOC_LITERAL(11, 148, 24), // "on_pb_background_clicked"
QT_MOC_LITERAL(12, 173, 18), // "on_pb_grid_clicked"
QT_MOC_LITERAL(13, 192, 18), // "on_pb_meta_clicked"
QT_MOC_LITERAL(14, 211, 22), // "on_pb_selected_clicked"
QT_MOC_LITERAL(15, 234, 25), // "on_pb_highlighted_clicked"
QT_MOC_LITERAL(16, 260, 19), // "on_pb_start_clicked"
QT_MOC_LITERAL(17, 280, 20), // "on_pb_handle_clicked"
QT_MOC_LITERAL(18, 301, 17), // "on_pb_end_clicked"
QT_MOC_LITERAL(19, 319, 23), // "on_pb_clear_all_clicked"
QT_MOC_LITERAL(20, 343, 28), // "on_pb_clear_geometry_clicked"
QT_MOC_LITERAL(21, 372, 24), // "on_pb_snap_color_clicked"
QT_MOC_LITERAL(22, 397, 31), // "on_pb_relativeZeroColor_clicked"
QT_MOC_LITERAL(23, 429, 15), // "setVariableFile"
QT_MOC_LITERAL(24, 445, 14) // "setFontsFolder"

    },
    "QG_DlgOptionsGeneral\0setRestartNeeded\0"
    "\0ok\0onAutoBackupChanged\0state\0"
    "languageChange\0setTemplateFile\0"
    "setLibraryPath\0on_tabWidget_currentChanged\0"
    "index\0on_pb_background_clicked\0"
    "on_pb_grid_clicked\0on_pb_meta_clicked\0"
    "on_pb_selected_clicked\0on_pb_highlighted_clicked\0"
    "on_pb_start_clicked\0on_pb_handle_clicked\0"
    "on_pb_end_clicked\0on_pb_clear_all_clicked\0"
    "on_pb_clear_geometry_clicked\0"
    "on_pb_snap_color_clicked\0"
    "on_pb_relativeZeroColor_clicked\0"
    "setVariableFile\0setFontsFolder"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QG_DlgOptionsGeneral[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  119,    2, 0x0a /* Public */,
       3,    0,  120,    2, 0x0a /* Public */,
       4,    1,  121,    2, 0x0a /* Public */,
       6,    0,  124,    2, 0x09 /* Protected */,
       7,    0,  125,    2, 0x09 /* Protected */,
       8,    0,  126,    2, 0x09 /* Protected */,
       9,    1,  127,    2, 0x08 /* Private */,
      11,    0,  130,    2, 0x08 /* Private */,
      12,    0,  131,    2, 0x08 /* Private */,
      13,    0,  132,    2, 0x08 /* Private */,
      14,    0,  133,    2, 0x08 /* Private */,
      15,    0,  134,    2, 0x08 /* Private */,
      16,    0,  135,    2, 0x08 /* Private */,
      17,    0,  136,    2, 0x08 /* Private */,
      18,    0,  137,    2, 0x08 /* Private */,
      19,    0,  138,    2, 0x08 /* Private */,
      20,    0,  139,    2, 0x08 /* Private */,
      21,    0,  140,    2, 0x08 /* Private */,
      22,    0,  141,    2, 0x08 /* Private */,
      23,    0,  142,    2, 0x08 /* Private */,
      24,    0,  143,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QG_DlgOptionsGeneral::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QG_DlgOptionsGeneral *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setRestartNeeded(); break;
        case 1: _t->ok(); break;
        case 2: _t->onAutoBackupChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->languageChange(); break;
        case 4: _t->setTemplateFile(); break;
        case 5: _t->setLibraryPath(); break;
        case 6: _t->on_tabWidget_currentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->on_pb_background_clicked(); break;
        case 8: _t->on_pb_grid_clicked(); break;
        case 9: _t->on_pb_meta_clicked(); break;
        case 10: _t->on_pb_selected_clicked(); break;
        case 11: _t->on_pb_highlighted_clicked(); break;
        case 12: _t->on_pb_start_clicked(); break;
        case 13: _t->on_pb_handle_clicked(); break;
        case 14: _t->on_pb_end_clicked(); break;
        case 15: _t->on_pb_clear_all_clicked(); break;
        case 16: _t->on_pb_clear_geometry_clicked(); break;
        case 17: _t->on_pb_snap_color_clicked(); break;
        case 18: _t->on_pb_relativeZeroColor_clicked(); break;
        case 19: _t->setVariableFile(); break;
        case 20: _t->setFontsFolder(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QG_DlgOptionsGeneral::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_QG_DlgOptionsGeneral.data,
    qt_meta_data_QG_DlgOptionsGeneral,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QG_DlgOptionsGeneral::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QG_DlgOptionsGeneral::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QG_DlgOptionsGeneral.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::QG_DlgOptionsGeneral"))
        return static_cast< Ui::QG_DlgOptionsGeneral*>(this);
    return QDialog::qt_metacast(_clname);
}

int QG_DlgOptionsGeneral::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
