/****************************************************************************
** Meta object code from reading C++ file 'qc_applicationwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/main/qc_applicationwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qc_applicationwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QC_ApplicationWindow_t {
    QByteArrayData data[107];
    char stringdata0[1486];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QC_ApplicationWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QC_ApplicationWindow_t qt_meta_stringdata_QC_ApplicationWindow = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QC_ApplicationWindow"
QT_MOC_LITERAL(1, 21, 11), // "gridChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 2), // "on"
QT_MOC_LITERAL(4, 37, 12), // "draftChanged"
QT_MOC_LITERAL(5, 50, 19), // "printPreviewChanged"
QT_MOC_LITERAL(6, 70, 14), // "windowsChanged"
QT_MOC_LITERAL(7, 85, 11), // "windowsLeft"
QT_MOC_LITERAL(8, 97, 29), // "signalEnableRelativeZeroSnaps"
QT_MOC_LITERAL(9, 127, 11), // "relayAction"
QT_MOC_LITERAL(10, 139, 8), // "QAction*"
QT_MOC_LITERAL(11, 148, 8), // "q_action"
QT_MOC_LITERAL(12, 157, 9), // "slotFocus"
QT_MOC_LITERAL(13, 167, 8), // "slotBack"
QT_MOC_LITERAL(14, 176, 18), // "slotKillAllActions"
QT_MOC_LITERAL(15, 195, 9), // "slotEnter"
QT_MOC_LITERAL(16, 205, 20), // "slotFocusCommandLine"
QT_MOC_LITERAL(17, 226, 9), // "slotError"
QT_MOC_LITERAL(18, 236, 3), // "msg"
QT_MOC_LITERAL(19, 240, 19), // "slotWindowActivated"
QT_MOC_LITERAL(20, 260, 14), // "QMdiSubWindow*"
QT_MOC_LITERAL(21, 275, 1), // "w"
QT_MOC_LITERAL(22, 277, 6), // "forced"
QT_MOC_LITERAL(23, 284, 26), // "slotWindowsMenuAboutToShow"
QT_MOC_LITERAL(24, 311, 24), // "slotWindowsMenuActivated"
QT_MOC_LITERAL(25, 336, 11), // "slotCascade"
QT_MOC_LITERAL(26, 348, 8), // "slotTile"
QT_MOC_LITERAL(27, 357, 18), // "slotTileHorizontal"
QT_MOC_LITERAL(28, 376, 16), // "slotTileVertical"
QT_MOC_LITERAL(29, 393, 16), // "slotSetMaximized"
QT_MOC_LITERAL(30, 410, 19), // "slotTabShapeRounded"
QT_MOC_LITERAL(31, 430, 22), // "slotTabShapeTriangular"
QT_MOC_LITERAL(32, 453, 20), // "slotTabPositionNorth"
QT_MOC_LITERAL(33, 474, 20), // "slotTabPositionSouth"
QT_MOC_LITERAL(34, 495, 19), // "slotTabPositionEast"
QT_MOC_LITERAL(35, 515, 19), // "slotTabPositionWest"
QT_MOC_LITERAL(36, 535, 13), // "slotToggleTab"
QT_MOC_LITERAL(37, 549, 12), // "slotZoomAuto"
QT_MOC_LITERAL(38, 562, 14), // "slotPenChanged"
QT_MOC_LITERAL(39, 577, 6), // "RS_Pen"
QT_MOC_LITERAL(40, 584, 1), // "p"
QT_MOC_LITERAL(41, 586, 17), // "slotEnableActions"
QT_MOC_LITERAL(42, 604, 6), // "enable"
QT_MOC_LITERAL(43, 611, 11), // "slotFileNew"
QT_MOC_LITERAL(44, 623, 13), // "QC_MDIWindow*"
QT_MOC_LITERAL(45, 637, 12), // "RS_Document*"
QT_MOC_LITERAL(46, 650, 3), // "doc"
QT_MOC_LITERAL(47, 654, 14), // "slotFileNewNew"
QT_MOC_LITERAL(48, 669, 19), // "slotFileNewTemplate"
QT_MOC_LITERAL(49, 689, 12), // "slotFileOpen"
QT_MOC_LITERAL(50, 702, 8), // "fileName"
QT_MOC_LITERAL(51, 711, 15), // "RS2::FormatType"
QT_MOC_LITERAL(52, 727, 4), // "type"
QT_MOC_LITERAL(53, 732, 18), // "slotFileOpenRecent"
QT_MOC_LITERAL(54, 751, 6), // "action"
QT_MOC_LITERAL(55, 758, 12), // "slotFileSave"
QT_MOC_LITERAL(56, 771, 14), // "slotFileSaveAs"
QT_MOC_LITERAL(57, 786, 15), // "slotFileSaveAll"
QT_MOC_LITERAL(58, 802, 16), // "slotFileAutoSave"
QT_MOC_LITERAL(59, 819, 14), // "slotFileExport"
QT_MOC_LITERAL(60, 834, 4), // "name"
QT_MOC_LITERAL(61, 839, 6), // "format"
QT_MOC_LITERAL(62, 846, 4), // "size"
QT_MOC_LITERAL(63, 851, 7), // "borders"
QT_MOC_LITERAL(64, 859, 5), // "black"
QT_MOC_LITERAL(65, 865, 2), // "bw"
QT_MOC_LITERAL(66, 868, 15), // "slotFileClosing"
QT_MOC_LITERAL(67, 884, 16), // "slotFileCloseAll"
QT_MOC_LITERAL(68, 901, 13), // "slotFilePrint"
QT_MOC_LITERAL(69, 915, 8), // "printPDF"
QT_MOC_LITERAL(70, 924, 16), // "slotFilePrintPDF"
QT_MOC_LITERAL(71, 941, 20), // "slotFilePrintPreview"
QT_MOC_LITERAL(72, 962, 12), // "slotFileQuit"
QT_MOC_LITERAL(73, 975, 12), // "slotViewGrid"
QT_MOC_LITERAL(74, 988, 6), // "toggle"
QT_MOC_LITERAL(75, 995, 13), // "slotViewDraft"
QT_MOC_LITERAL(76, 1009, 17), // "slotViewStatusBar"
QT_MOC_LITERAL(77, 1027, 18), // "slotOptionsGeneral"
QT_MOC_LITERAL(78, 1046, 15), // "slotImportBlock"
QT_MOC_LITERAL(79, 1062, 15), // "showAboutWindow"
QT_MOC_LITERAL(80, 1078, 21), // "slotUpdateActiveLayer"
QT_MOC_LITERAL(81, 1100, 8), // "execPlug"
QT_MOC_LITERAL(82, 1109, 16), // "toggleFullscreen"
QT_MOC_LITERAL(83, 1126, 7), // "checked"
QT_MOC_LITERAL(84, 1134, 21), // "setPreviousZoomEnable"
QT_MOC_LITERAL(85, 1156, 11), // "hideOptions"
QT_MOC_LITERAL(86, 1168, 19), // "widgetOptionsDialog"
QT_MOC_LITERAL(87, 1188, 21), // "modifyCommandTitleBar"
QT_MOC_LITERAL(88, 1210, 18), // "Qt::DockWidgetArea"
QT_MOC_LITERAL(89, 1229, 4), // "area"
QT_MOC_LITERAL(90, 1234, 16), // "reloadStyleSheet"
QT_MOC_LITERAL(91, 1251, 16), // "updateGridStatus"
QT_MOC_LITERAL(92, 1268, 17), // "showDeviceOptions"
QT_MOC_LITERAL(93, 1286, 12), // "updateDevice"
QT_MOC_LITERAL(94, 1299, 17), // "invokeMenuCreator"
QT_MOC_LITERAL(95, 1317, 20), // "invokeToolbarCreator"
QT_MOC_LITERAL(96, 1338, 13), // "createToolbar"
QT_MOC_LITERAL(97, 1352, 12), // "toolbar_name"
QT_MOC_LITERAL(98, 1365, 14), // "destroyToolbar"
QT_MOC_LITERAL(99, 1380, 11), // "destroyMenu"
QT_MOC_LITERAL(100, 1392, 9), // "activator"
QT_MOC_LITERAL(101, 1402, 12), // "unassignMenu"
QT_MOC_LITERAL(102, 1415, 9), // "menu_name"
QT_MOC_LITERAL(103, 1425, 10), // "assignMenu"
QT_MOC_LITERAL(104, 1436, 18), // "invokeMenuAssigner"
QT_MOC_LITERAL(105, 1455, 10), // "updateMenu"
QT_MOC_LITERAL(106, 1466, 19) // "invokeLicenseWindow"

    },
    "QC_ApplicationWindow\0gridChanged\0\0on\0"
    "draftChanged\0printPreviewChanged\0"
    "windowsChanged\0windowsLeft\0"
    "signalEnableRelativeZeroSnaps\0relayAction\0"
    "QAction*\0q_action\0slotFocus\0slotBack\0"
    "slotKillAllActions\0slotEnter\0"
    "slotFocusCommandLine\0slotError\0msg\0"
    "slotWindowActivated\0QMdiSubWindow*\0w\0"
    "forced\0slotWindowsMenuAboutToShow\0"
    "slotWindowsMenuActivated\0slotCascade\0"
    "slotTile\0slotTileHorizontal\0"
    "slotTileVertical\0slotSetMaximized\0"
    "slotTabShapeRounded\0slotTabShapeTriangular\0"
    "slotTabPositionNorth\0slotTabPositionSouth\0"
    "slotTabPositionEast\0slotTabPositionWest\0"
    "slotToggleTab\0slotZoomAuto\0slotPenChanged\0"
    "RS_Pen\0p\0slotEnableActions\0enable\0"
    "slotFileNew\0QC_MDIWindow*\0RS_Document*\0"
    "doc\0slotFileNewNew\0slotFileNewTemplate\0"
    "slotFileOpen\0fileName\0RS2::FormatType\0"
    "type\0slotFileOpenRecent\0action\0"
    "slotFileSave\0slotFileSaveAs\0slotFileSaveAll\0"
    "slotFileAutoSave\0slotFileExport\0name\0"
    "format\0size\0borders\0black\0bw\0"
    "slotFileClosing\0slotFileCloseAll\0"
    "slotFilePrint\0printPDF\0slotFilePrintPDF\0"
    "slotFilePrintPreview\0slotFileQuit\0"
    "slotViewGrid\0toggle\0slotViewDraft\0"
    "slotViewStatusBar\0slotOptionsGeneral\0"
    "slotImportBlock\0showAboutWindow\0"
    "slotUpdateActiveLayer\0execPlug\0"
    "toggleFullscreen\0checked\0setPreviousZoomEnable\0"
    "hideOptions\0widgetOptionsDialog\0"
    "modifyCommandTitleBar\0Qt::DockWidgetArea\0"
    "area\0reloadStyleSheet\0updateGridStatus\0"
    "showDeviceOptions\0updateDevice\0"
    "invokeMenuCreator\0invokeToolbarCreator\0"
    "createToolbar\0toolbar_name\0destroyToolbar\0"
    "destroyMenu\0activator\0unassignMenu\0"
    "menu_name\0assignMenu\0invokeMenuAssigner\0"
    "updateMenu\0invokeLicenseWindow"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QC_ApplicationWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      81,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  419,    2, 0x06 /* Public */,
       4,    1,  422,    2, 0x06 /* Public */,
       5,    1,  425,    2, 0x06 /* Public */,
       6,    1,  428,    2, 0x06 /* Public */,
       8,    1,  431,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    1,  434,    2, 0x0a /* Public */,
      12,    0,  437,    2, 0x0a /* Public */,
      13,    0,  438,    2, 0x0a /* Public */,
      14,    0,  439,    2, 0x0a /* Public */,
      15,    0,  440,    2, 0x0a /* Public */,
      16,    0,  441,    2, 0x0a /* Public */,
      17,    1,  442,    2, 0x0a /* Public */,
      19,    1,  445,    2, 0x0a /* Public */,
      19,    2,  448,    2, 0x0a /* Public */,
      19,    1,  453,    2, 0x2a /* Public | MethodCloned */,
      23,    0,  456,    2, 0x0a /* Public */,
      24,    1,  457,    2, 0x0a /* Public */,
      25,    0,  460,    2, 0x0a /* Public */,
      26,    0,  461,    2, 0x0a /* Public */,
      27,    0,  462,    2, 0x0a /* Public */,
      28,    0,  463,    2, 0x0a /* Public */,
      29,    0,  464,    2, 0x0a /* Public */,
      30,    0,  465,    2, 0x0a /* Public */,
      31,    0,  466,    2, 0x0a /* Public */,
      32,    0,  467,    2, 0x0a /* Public */,
      33,    0,  468,    2, 0x0a /* Public */,
      34,    0,  469,    2, 0x0a /* Public */,
      35,    0,  470,    2, 0x0a /* Public */,
      36,    0,  471,    2, 0x0a /* Public */,
      37,    0,  472,    2, 0x0a /* Public */,
      38,    1,  473,    2, 0x0a /* Public */,
      41,    1,  476,    2, 0x0a /* Public */,
      43,    1,  479,    2, 0x0a /* Public */,
      43,    0,  482,    2, 0x2a /* Public | MethodCloned */,
      47,    0,  483,    2, 0x0a /* Public */,
      48,    0,  484,    2, 0x0a /* Public */,
      49,    0,  485,    2, 0x0a /* Public */,
      49,    2,  486,    2, 0x0a /* Public */,
      49,    1,  491,    2, 0x0a /* Public */,
      53,    1,  494,    2, 0x0a /* Public */,
      55,    0,  497,    2, 0x0a /* Public */,
      56,    0,  498,    2, 0x0a /* Public */,
      57,    0,  499,    2, 0x0a /* Public */,
      58,    0,  500,    2, 0x0a /* Public */,
      59,    0,  501,    2, 0x0a /* Public */,
      59,    6,  502,    2, 0x0a /* Public */,
      59,    5,  515,    2, 0x2a /* Public | MethodCloned */,
      66,    1,  526,    2, 0x0a /* Public */,
      67,    0,  529,    2, 0x0a /* Public */,
      68,    1,  530,    2, 0x0a /* Public */,
      68,    0,  533,    2, 0x2a /* Public | MethodCloned */,
      70,    0,  534,    2, 0x0a /* Public */,
      71,    1,  535,    2, 0x0a /* Public */,
      72,    0,  538,    2, 0x0a /* Public */,
      73,    1,  539,    2, 0x0a /* Public */,
      75,    1,  542,    2, 0x0a /* Public */,
      76,    1,  545,    2, 0x0a /* Public */,
      77,    0,  548,    2, 0x0a /* Public */,
      78,    0,  549,    2, 0x0a /* Public */,
      79,    0,  550,    2, 0x0a /* Public */,
      80,    0,  551,    2, 0x0a /* Public */,
      81,    0,  552,    2, 0x0a /* Public */,
      82,    1,  553,    2, 0x0a /* Public */,
      84,    1,  556,    2, 0x0a /* Public */,
      85,    1,  559,    2, 0x0a /* Public */,
      86,    0,  562,    2, 0x0a /* Public */,
      87,    1,  563,    2, 0x0a /* Public */,
      90,    0,  566,    2, 0x0a /* Public */,
      91,    1,  567,    2, 0x0a /* Public */,
      92,    0,  570,    2, 0x0a /* Public */,
      93,    1,  571,    2, 0x0a /* Public */,
      94,    0,  574,    2, 0x0a /* Public */,
      95,    0,  575,    2, 0x0a /* Public */,
      96,    1,  576,    2, 0x0a /* Public */,
      98,    1,  579,    2, 0x0a /* Public */,
      99,    1,  582,    2, 0x0a /* Public */,
     101,    2,  585,    2, 0x0a /* Public */,
     103,    2,  590,    2, 0x0a /* Public */,
     104,    1,  595,    2, 0x0a /* Public */,
     105,    1,  598,    2, 0x0a /* Public */,
     106,    0,  601,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   18,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 20, QMetaType::Bool,   21,   22,
    QMetaType::Void, 0x80000000 | 20,   21,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 39,   40,
    QMetaType::Void, QMetaType::Bool,   42,
    0x80000000 | 44, 0x80000000 | 45,   46,
    0x80000000 | 44,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 51,   50,   52,
    QMetaType::Void, QMetaType::QString,   50,
    QMetaType::Void, 0x80000000 | 10,   54,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QSize, QMetaType::QSize, QMetaType::Bool, QMetaType::Bool,   60,   61,   62,   63,   64,   65,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QSize, QMetaType::QSize, QMetaType::Bool,   60,   61,   62,   63,   64,
    QMetaType::Void, 0x80000000 | 44,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   69,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   74,
    QMetaType::Void, QMetaType::Bool,   74,
    QMetaType::Void, QMetaType::Bool,   74,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   83,
    QMetaType::Void, QMetaType::Bool,   42,
    QMetaType::Void, 0x80000000 | 44,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 88,   89,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   97,
    QMetaType::Void, QMetaType::QString,   97,
    QMetaType::Void, QMetaType::QString,  100,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,  100,  102,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,  100,  102,
    QMetaType::Void, QMetaType::QString,  102,
    QMetaType::Void, QMetaType::QString,  102,
    QMetaType::Void,

       0        // eod
};

void QC_ApplicationWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QC_ApplicationWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->gridChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->draftChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->printPreviewChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->windowsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->signalEnableRelativeZeroSnaps((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 5: _t->relayAction((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 6: _t->slotFocus(); break;
        case 7: _t->slotBack(); break;
        case 8: _t->slotKillAllActions(); break;
        case 9: _t->slotEnter(); break;
        case 10: _t->slotFocusCommandLine(); break;
        case 11: _t->slotError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->slotWindowActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->slotWindowActivated((*reinterpret_cast< QMdiSubWindow*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 14: _t->slotWindowActivated((*reinterpret_cast< QMdiSubWindow*(*)>(_a[1]))); break;
        case 15: _t->slotWindowsMenuAboutToShow(); break;
        case 16: _t->slotWindowsMenuActivated((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->slotCascade(); break;
        case 18: _t->slotTile(); break;
        case 19: _t->slotTileHorizontal(); break;
        case 20: _t->slotTileVertical(); break;
        case 21: _t->slotSetMaximized(); break;
        case 22: _t->slotTabShapeRounded(); break;
        case 23: _t->slotTabShapeTriangular(); break;
        case 24: _t->slotTabPositionNorth(); break;
        case 25: _t->slotTabPositionSouth(); break;
        case 26: _t->slotTabPositionEast(); break;
        case 27: _t->slotTabPositionWest(); break;
        case 28: _t->slotToggleTab(); break;
        case 29: _t->slotZoomAuto(); break;
        case 30: _t->slotPenChanged((*reinterpret_cast< RS_Pen(*)>(_a[1]))); break;
        case 31: _t->slotEnableActions((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 32: { QC_MDIWindow* _r = _t->slotFileNew((*reinterpret_cast< RS_Document*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QC_MDIWindow**>(_a[0]) = std::move(_r); }  break;
        case 33: { QC_MDIWindow* _r = _t->slotFileNew();
            if (_a[0]) *reinterpret_cast< QC_MDIWindow**>(_a[0]) = std::move(_r); }  break;
        case 34: _t->slotFileNewNew(); break;
        case 35: _t->slotFileNewTemplate(); break;
        case 36: _t->slotFileOpen(); break;
        case 37: _t->slotFileOpen((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< RS2::FormatType(*)>(_a[2]))); break;
        case 38: _t->slotFileOpen((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 39: _t->slotFileOpenRecent((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 40: _t->slotFileSave(); break;
        case 41: _t->slotFileSaveAs(); break;
        case 42: { bool _r = _t->slotFileSaveAll();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 43: _t->slotFileAutoSave(); break;
        case 44: _t->slotFileExport(); break;
        case 45: { bool _r = _t->slotFileExport((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< QSize(*)>(_a[3])),(*reinterpret_cast< QSize(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5])),(*reinterpret_cast< bool(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 46: { bool _r = _t->slotFileExport((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< QSize(*)>(_a[3])),(*reinterpret_cast< QSize(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 47: _t->slotFileClosing((*reinterpret_cast< QC_MDIWindow*(*)>(_a[1]))); break;
        case 48: { bool _r = _t->slotFileCloseAll();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 49: _t->slotFilePrint((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 50: _t->slotFilePrint(); break;
        case 51: _t->slotFilePrintPDF(); break;
        case 52: _t->slotFilePrintPreview((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 53: _t->slotFileQuit(); break;
        case 54: _t->slotViewGrid((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 55: _t->slotViewDraft((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 56: _t->slotViewStatusBar((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 57: _t->slotOptionsGeneral(); break;
        case 58: _t->slotImportBlock(); break;
        case 59: _t->showAboutWindow(); break;
        case 60: _t->slotUpdateActiveLayer(); break;
        case 61: _t->execPlug(); break;
        case 62: _t->toggleFullscreen((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 63: _t->setPreviousZoomEnable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 64: _t->hideOptions((*reinterpret_cast< QC_MDIWindow*(*)>(_a[1]))); break;
        case 65: _t->widgetOptionsDialog(); break;
        case 66: _t->modifyCommandTitleBar((*reinterpret_cast< Qt::DockWidgetArea(*)>(_a[1]))); break;
        case 67: _t->reloadStyleSheet(); break;
        case 68: _t->updateGridStatus((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 69: _t->showDeviceOptions(); break;
        case 70: _t->updateDevice((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 71: _t->invokeMenuCreator(); break;
        case 72: _t->invokeToolbarCreator(); break;
        case 73: _t->createToolbar((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 74: _t->destroyToolbar((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 75: _t->destroyMenu((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 76: _t->unassignMenu((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 77: _t->assignMenu((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 78: _t->invokeMenuAssigner((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 79: _t->updateMenu((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 80: _t->invokeLicenseWindow(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMdiSubWindow* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMdiSubWindow* >(); break;
            }
            break;
        case 47:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QC_MDIWindow* >(); break;
            }
            break;
        case 64:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QC_MDIWindow* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QC_ApplicationWindow::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QC_ApplicationWindow::gridChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QC_ApplicationWindow::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QC_ApplicationWindow::draftChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QC_ApplicationWindow::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QC_ApplicationWindow::printPreviewChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QC_ApplicationWindow::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QC_ApplicationWindow::windowsChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QC_ApplicationWindow::*)(const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QC_ApplicationWindow::signalEnableRelativeZeroSnaps)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QC_ApplicationWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<MainWindowX::staticMetaObject>(),
    qt_meta_stringdata_QC_ApplicationWindow.data,
    qt_meta_data_QC_ApplicationWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QC_ApplicationWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QC_ApplicationWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QC_ApplicationWindow.stringdata0))
        return static_cast<void*>(this);
    return MainWindowX::qt_metacast(_clname);
}

int QC_ApplicationWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MainWindowX::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 81)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 81;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 81)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 81;
    }
    return _id;
}

// SIGNAL 0
void QC_ApplicationWindow::gridChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QC_ApplicationWindow::draftChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QC_ApplicationWindow::printPreviewChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QC_ApplicationWindow::windowsChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QC_ApplicationWindow::signalEnableRelativeZeroSnaps(const bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
