/********************************************************************************
** Form generated from reading UI file 'lc_penpalettewidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_PENPALETTEWIDGET_H
#define UI_LC_PENPALETTEWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include "qg_colorbox.h"
#include "qg_linetypebox.h"
#include "qg_widthbox.h"

QT_BEGIN_NAMESPACE

class Ui_LC_PenPaletteWidget
{
public:
    QGridLayout *gridLayout;
    QFrame *line;
    QHBoxLayout *layPenColor;
    QLabel *lblName;
    QLineEdit *lePenName;
    QLabel *lColor;
    QG_ColorBox *cbColor;
    QSpacerItem *horizontalSpacer;
    QTableView *tableView;
    QHBoxLayout *layFilter;
    QLineEdit *leFilterMask;
    QCheckBox *cbHighlightMode;
    QHBoxLayout *layTypeWidth;
    QLabel *lType;
    QG_LineTypeBox *cbType;
    QLabel *label;
    QG_WidthBox *cbWidth;
    QSpacerItem *horizontalSpacer_2;
    QToolButton *tbSettings;
    QHBoxLayout *layButtons;
    QToolButton *tbEditSelectResolvedFromEntity;
    QToolButton *tbEditSelectFromEntity;
    QToolButton *tbEditSelectEntitiesByResolvedPen;
    QToolButton *tbEditSelectEntities;
    QToolButton *tbEditSave;
    QToolButton *tbEditApplyToSelection;
    QToolButton *tbEditApplyFromCurrent;
    QToolButton *tbSetAsCurrent;
    QToolButton *tbUpdateCurrentPenByActiveLayer;
    QToolButton *tbPickFromActiveLayer;
    QToolButton *tbApplyToActiveLayer;
    QToolButton *tbRemove;

    void setupUi(QWidget *LC_PenPaletteWidget)
    {
        if (LC_PenPaletteWidget->objectName().isEmpty())
            LC_PenPaletteWidget->setObjectName(QString::fromUtf8("LC_PenPaletteWidget"));
        LC_PenPaletteWidget->resize(404, 246);
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(LC_PenPaletteWidget->sizePolicy().hasHeightForWidth());
        LC_PenPaletteWidget->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(LC_PenPaletteWidget);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        line = new QFrame(LC_PenPaletteWidget);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line, 8, 0, 1, 2);

        layPenColor = new QHBoxLayout();
        layPenColor->setObjectName(QString::fromUtf8("layPenColor"));
        lblName = new QLabel(LC_PenPaletteWidget);
        lblName->setObjectName(QString::fromUtf8("lblName"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lblName->sizePolicy().hasHeightForWidth());
        lblName->setSizePolicy(sizePolicy1);

        layPenColor->addWidget(lblName);

        lePenName = new QLineEdit(LC_PenPaletteWidget);
        lePenName->setObjectName(QString::fromUtf8("lePenName"));
        lePenName->setEnabled(true);
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(lePenName->sizePolicy().hasHeightForWidth());
        lePenName->setSizePolicy(sizePolicy2);
        lePenName->setMinimumSize(QSize(60, 22));
        lePenName->setMaximumSize(QSize(16777215, 16777215));
        lePenName->setBaseSize(QSize(130, 22));

        layPenColor->addWidget(lePenName);

        lColor = new QLabel(LC_PenPaletteWidget);
        lColor->setObjectName(QString::fromUtf8("lColor"));
        sizePolicy1.setHeightForWidth(lColor->sizePolicy().hasHeightForWidth());
        lColor->setSizePolicy(sizePolicy1);
        lColor->setMinimumSize(QSize(40, 0));

        layPenColor->addWidget(lColor);

        cbColor = new QG_ColorBox(LC_PenPaletteWidget);
        cbColor->setObjectName(QString::fromUtf8("cbColor"));

        layPenColor->addWidget(cbColor);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        layPenColor->addItem(horizontalSpacer);


        gridLayout->addLayout(layPenColor, 6, 0, 1, 2);

        tableView = new QTableView(LC_PenPaletteWidget);
        tableView->setObjectName(QString::fromUtf8("tableView"));
        QSizePolicy sizePolicy3(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(1);
        sizePolicy3.setHeightForWidth(tableView->sizePolicy().hasHeightForWidth());
        tableView->setSizePolicy(sizePolicy3);
        tableView->setMinimumSize(QSize(100, 100));
        tableView->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        tableView->setAutoScrollMargin(6);
        tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        tableView->setProperty("showDropIndicator", QVariant(false));
        tableView->setDragDropOverwriteMode(false);
        tableView->setSelectionMode(QAbstractItemView::SingleSelection);
        tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
        tableView->setIconSize(QSize(24, 24));
        tableView->setShowGrid(false);
        tableView->setGridStyle(Qt::DashLine);
        tableView->setCornerButtonEnabled(false);
        tableView->horizontalHeader()->setVisible(false);
        tableView->horizontalHeader()->setMinimumSectionSize(24);
        tableView->horizontalHeader()->setStretchLastSection(true);
        tableView->verticalHeader()->setVisible(false);

        gridLayout->addWidget(tableView, 12, 0, 1, 2);

        layFilter = new QHBoxLayout();
        layFilter->setObjectName(QString::fromUtf8("layFilter"));
        leFilterMask = new QLineEdit(LC_PenPaletteWidget);
        leFilterMask->setObjectName(QString::fromUtf8("leFilterMask"));
        QSizePolicy sizePolicy4(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(leFilterMask->sizePolicy().hasHeightForWidth());
        leFilterMask->setSizePolicy(sizePolicy4);
        leFilterMask->setClearButtonEnabled(true);

        layFilter->addWidget(leFilterMask);

        cbHighlightMode = new QCheckBox(LC_PenPaletteWidget);
        cbHighlightMode->setObjectName(QString::fromUtf8("cbHighlightMode"));

        layFilter->addWidget(cbHighlightMode);


        gridLayout->addLayout(layFilter, 10, 0, 1, 2);

        layTypeWidth = new QHBoxLayout();
        layTypeWidth->setObjectName(QString::fromUtf8("layTypeWidth"));
        lType = new QLabel(LC_PenPaletteWidget);
        lType->setObjectName(QString::fromUtf8("lType"));
        sizePolicy1.setHeightForWidth(lType->sizePolicy().hasHeightForWidth());
        lType->setSizePolicy(sizePolicy1);
        lType->setMinimumSize(QSize(40, 0));

        layTypeWidth->addWidget(lType);

        cbType = new QG_LineTypeBox(LC_PenPaletteWidget);
        cbType->setObjectName(QString::fromUtf8("cbType"));

        layTypeWidth->addWidget(cbType);

        label = new QLabel(LC_PenPaletteWidget);
        label->setObjectName(QString::fromUtf8("label"));
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        label->setMinimumSize(QSize(40, 0));

        layTypeWidth->addWidget(label);

        cbWidth = new QG_WidthBox(LC_PenPaletteWidget);
        cbWidth->setObjectName(QString::fromUtf8("cbWidth"));

        layTypeWidth->addWidget(cbWidth);

        horizontalSpacer_2 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        layTypeWidth->addItem(horizontalSpacer_2);


        gridLayout->addLayout(layTypeWidth, 7, 0, 1, 2);

        tbSettings = new QToolButton(LC_PenPaletteWidget);
        tbSettings->setObjectName(QString::fromUtf8("tbSettings"));
        tbSettings->setStyleSheet(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/actions/configure.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbSettings->setIcon(icon);

        gridLayout->addWidget(tbSettings, 1, 1, 1, 1);

        layButtons = new QHBoxLayout();
        layButtons->setObjectName(QString::fromUtf8("layButtons"));
        tbEditSelectResolvedFromEntity = new QToolButton(LC_PenPaletteWidget);
        tbEditSelectResolvedFromEntity->setObjectName(QString::fromUtf8("tbEditSelectResolvedFromEntity"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/extui/relzeromove.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbEditSelectResolvedFromEntity->setIcon(icon1);

        layButtons->addWidget(tbEditSelectResolvedFromEntity);

        tbEditSelectFromEntity = new QToolButton(LC_PenPaletteWidget);
        tbEditSelectFromEntity->setObjectName(QString::fromUtf8("tbEditSelectFromEntity"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/extui/selectsingle.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbEditSelectFromEntity->setIcon(icon2);

        layButtons->addWidget(tbEditSelectFromEntity);

        tbEditSelectEntitiesByResolvedPen = new QToolButton(LC_PenPaletteWidget);
        tbEditSelectEntitiesByResolvedPen->setObjectName(QString::fromUtf8("tbEditSelectEntitiesByResolvedPen"));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/extui/selectinvert.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbEditSelectEntitiesByResolvedPen->setIcon(icon3);

        layButtons->addWidget(tbEditSelectEntitiesByResolvedPen);

        tbEditSelectEntities = new QToolButton(LC_PenPaletteWidget);
        tbEditSelectEntities->setObjectName(QString::fromUtf8("tbEditSelectEntities"));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/extui/selectinters.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbEditSelectEntities->setIcon(icon4);

        layButtons->addWidget(tbEditSelectEntities);

        tbEditSave = new QToolButton(LC_PenPaletteWidget);
        tbEditSave->setObjectName(QString::fromUtf8("tbEditSave"));
        tbEditSave->setEnabled(false);
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/actions/filesave.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbEditSave->setIcon(icon5);

        layButtons->addWidget(tbEditSave);

        tbEditApplyToSelection = new QToolButton(LC_PenPaletteWidget);
        tbEditApplyToSelection->setObjectName(QString::fromUtf8("tbEditApplyToSelection"));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/extui/selectdoubles.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbEditApplyToSelection->setIcon(icon6);

        layButtons->addWidget(tbEditApplyToSelection);

        tbEditApplyFromCurrent = new QToolButton(LC_PenPaletteWidget);
        tbEditApplyFromCurrent->setObjectName(QString::fromUtf8("tbEditApplyFromCurrent"));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/actions/editredo.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbEditApplyFromCurrent->setIcon(icon7);

        layButtons->addWidget(tbEditApplyFromCurrent);

        tbSetAsCurrent = new QToolButton(LC_PenPaletteWidget);
        tbSetAsCurrent->setObjectName(QString::fromUtf8("tbSetAsCurrent"));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/actions/editundo.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbSetAsCurrent->setIcon(icon8);

        layButtons->addWidget(tbSetAsCurrent);

        tbUpdateCurrentPenByActiveLayer = new QToolButton(LC_PenPaletteWidget);
        tbUpdateCurrentPenByActiveLayer->setObjectName(QString::fromUtf8("tbUpdateCurrentPenByActiveLayer"));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/extui/back.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbUpdateCurrentPenByActiveLayer->setIcon(icon9);

        layButtons->addWidget(tbUpdateCurrentPenByActiveLayer);

        tbPickFromActiveLayer = new QToolButton(LC_PenPaletteWidget);
        tbPickFromActiveLayer->setObjectName(QString::fromUtf8("tbPickFromActiveLayer"));
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/extui/selectlayer.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbPickFromActiveLayer->setIcon(icon10);

        layButtons->addWidget(tbPickFromActiveLayer);

        tbApplyToActiveLayer = new QToolButton(LC_PenPaletteWidget);
        tbApplyToActiveLayer->setObjectName(QString::fromUtf8("tbApplyToActiveLayer"));
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/ui/layeredit.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbApplyToActiveLayer->setIcon(icon11);

        layButtons->addWidget(tbApplyToActiveLayer);

        tbRemove = new QToolButton(LC_PenPaletteWidget);
        tbRemove->setObjectName(QString::fromUtf8("tbRemove"));
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/icons/remove.svg"), QSize(), QIcon::Normal, QIcon::Off);
        tbRemove->setIcon(icon12);

        layButtons->addWidget(tbRemove);


        gridLayout->addLayout(layButtons, 1, 0, 1, 1);


        retranslateUi(LC_PenPaletteWidget);

        QMetaObject::connectSlotsByName(LC_PenPaletteWidget);
    } // setupUi

    void retranslateUi(QWidget *LC_PenPaletteWidget)
    {
        LC_PenPaletteWidget->setWindowTitle(QCoreApplication::translate("LC_PenPaletteWidget", "Pen Palette", nullptr));
        lblName->setText(QCoreApplication::translate("LC_PenPaletteWidget", "Pen Name:", nullptr));
        lColor->setText(QCoreApplication::translate("LC_PenPaletteWidget", "Color:", nullptr));
        leFilterMask->setPlaceholderText(QCoreApplication::translate("LC_PenPaletteWidget", "Filter...", nullptr));
        cbHighlightMode->setText(QCoreApplication::translate("LC_PenPaletteWidget", "Highlight Mode", nullptr));
        lType->setText(QCoreApplication::translate("LC_PenPaletteWidget", "Type:", nullptr));
        label->setText(QCoreApplication::translate("LC_PenPaletteWidget", "Width:", nullptr));
#if QT_CONFIG(tooltip)
        tbSettings->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Settings", nullptr));
#endif // QT_CONFIG(tooltip)
        tbSettings->setText(QString());
#if QT_CONFIG(tooltip)
        tbEditSelectResolvedFromEntity->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Fill Pen Editor by Entity Drawing Pen", nullptr));
#endif // QT_CONFIG(tooltip)
        tbEditSelectResolvedFromEntity->setText(QString());
#if QT_CONFIG(tooltip)
        tbEditSelectFromEntity->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Fill Pen Editor by Entity Attributes", nullptr));
#endif // QT_CONFIG(tooltip)
        tbEditSelectFromEntity->setText(QString());
#if QT_CONFIG(tooltip)
        tbEditSelectEntitiesByResolvedPen->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Select Entities By Drawing Pen", nullptr));
#endif // QT_CONFIG(tooltip)
        tbEditSelectEntitiesByResolvedPen->setText(QString());
#if QT_CONFIG(tooltip)
        tbEditSelectEntities->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Select Entities By Pen Attributes", nullptr));
#endif // QT_CONFIG(tooltip)
        tbEditSelectEntities->setText(QString());
#if QT_CONFIG(tooltip)
        tbEditSave->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Save", nullptr));
#endif // QT_CONFIG(tooltip)
        tbEditSave->setText(QCoreApplication::translate("LC_PenPaletteWidget", "Save", nullptr));
#if QT_CONFIG(tooltip)
        tbEditApplyToSelection->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Apply Pen To Selection", nullptr));
#endif // QT_CONFIG(tooltip)
        tbEditApplyToSelection->setText(QString());
#if QT_CONFIG(tooltip)
        tbEditApplyFromCurrent->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Fill Editor Pen by Current Pen", nullptr));
#endif // QT_CONFIG(tooltip)
        tbEditApplyFromCurrent->setText(QString());
#if QT_CONFIG(tooltip)
        tbSetAsCurrent->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Apply Editor Pen As Current Pen", nullptr));
#endif // QT_CONFIG(tooltip)
        tbSetAsCurrent->setText(QCoreApplication::translate("LC_PenPaletteWidget", "to current", nullptr));
#if QT_CONFIG(tooltip)
        tbUpdateCurrentPenByActiveLayer->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Update Current Pen by Active Layer' Pen", nullptr));
#endif // QT_CONFIG(tooltip)
        tbUpdateCurrentPenByActiveLayer->setText(QString());
#if QT_CONFIG(tooltip)
        tbPickFromActiveLayer->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Fill Pen Editor from Active Layer's Pen", nullptr));
#endif // QT_CONFIG(tooltip)
        tbPickFromActiveLayer->setText(QCoreApplication::translate("LC_PenPaletteWidget", "Pick from Active Layer", nullptr));
#if QT_CONFIG(tooltip)
        tbApplyToActiveLayer->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Apply Editor Pen to Active Layer Pen", nullptr));
#endif // QT_CONFIG(tooltip)
        tbApplyToActiveLayer->setText(QCoreApplication::translate("LC_PenPaletteWidget", "Apply To Layer", nullptr));
#if QT_CONFIG(tooltip)
        tbRemove->setToolTip(QCoreApplication::translate("LC_PenPaletteWidget", "Remove Selected Pen", nullptr));
#endif // QT_CONFIG(tooltip)
        tbRemove->setText(QCoreApplication::translate("LC_PenPaletteWidget", "Delete Pen", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_PenPaletteWidget: public Ui_LC_PenPaletteWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_PENPALETTEWIDGET_H
