/****************************************************************************
** Meta object code from reading C++ file 'qg_blockwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/qg_blockwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qg_blockwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QG_BlockWidget_t {
    QByteArrayData data[11];
    char stringdata0[134];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QG_BlockWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QG_BlockWidget_t qt_meta_stringdata_QG_BlockWidget = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QG_BlockWidget"
QT_MOC_LITERAL(1, 15, 6), // "escape"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 13), // "slotActivated"
QT_MOC_LITERAL(4, 37, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 49, 8), // "blockIdx"
QT_MOC_LITERAL(6, 58, 20), // "slotSelectionChanged"
QT_MOC_LITERAL(7, 79, 14), // "QItemSelection"
QT_MOC_LITERAL(8, 94, 8), // "selected"
QT_MOC_LITERAL(9, 103, 10), // "deselected"
QT_MOC_LITERAL(10, 114, 19) // "slotUpdateBlockList"

    },
    "QG_BlockWidget\0escape\0\0slotActivated\0"
    "QModelIndex\0blockIdx\0slotSelectionChanged\0"
    "QItemSelection\0selected\0deselected\0"
    "slotUpdateBlockList"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QG_BlockWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   35,    2, 0x0a /* Public */,
       6,    2,   38,    2, 0x0a /* Public */,
      10,    0,   43,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 7,    8,    9,
    QMetaType::Void,

       0        // eod
};

void QG_BlockWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QG_BlockWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->escape(); break;
        case 1: _t->slotActivated((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 2: _t->slotSelectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 3: _t->slotUpdateBlockList(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QG_BlockWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QG_BlockWidget::escape)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QG_BlockWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_QG_BlockWidget.data,
    qt_meta_data_QG_BlockWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QG_BlockWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QG_BlockWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QG_BlockWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "RS_BlockListListener"))
        return static_cast< RS_BlockListListener*>(this);
    return QWidget::qt_metacast(_clname);
}

int QG_BlockWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QG_BlockWidget::escape()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
