/********************************************************************************
** Form generated from reading UI file 'lc_circlebyarcoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_CIRCLEBYARCOPTIONS_H
#define UI_LC_CIRCLEBYARCOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_CircleByArcOptions
{
public:
    QGridLayout *gridLayout;
    QComboBox *cbPen;
    QLabel *lblRadiusShift;
    QSpacerItem *horizontalSpacer;
    QLabel *label_2;
    QComboBox *cbLayer;
    QLineEdit *leRadiusShift;
    QCheckBox *cbReplace;
    QLabel *label;

    void setupUi(QWidget *LC_CircleByArcOptions)
    {
        if (LC_CircleByArcOptions->objectName().isEmpty())
            LC_CircleByArcOptions->setObjectName(QString::fromUtf8("LC_CircleByArcOptions"));
        LC_CircleByArcOptions->resize(485, 24);
        gridLayout = new QGridLayout(LC_CircleByArcOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        cbPen = new QComboBox(LC_CircleByArcOptions);
        cbPen->addItem(QString());
        cbPen->addItem(QString());
        cbPen->addItem(QString());
        cbPen->setObjectName(QString::fromUtf8("cbPen"));

        gridLayout->addWidget(cbPen, 0, 4, 1, 1);

        lblRadiusShift = new QLabel(LC_CircleByArcOptions);
        lblRadiusShift->setObjectName(QString::fromUtf8("lblRadiusShift"));

        gridLayout->addWidget(lblRadiusShift, 0, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 7, 1, 1);

        label_2 = new QLabel(LC_CircleByArcOptions);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 5, 1, 1);

        cbLayer = new QComboBox(LC_CircleByArcOptions);
        cbLayer->addItem(QString());
        cbLayer->addItem(QString());
        cbLayer->setObjectName(QString::fromUtf8("cbLayer"));

        gridLayout->addWidget(cbLayer, 0, 6, 1, 1);

        leRadiusShift = new QLineEdit(LC_CircleByArcOptions);
        leRadiusShift->setObjectName(QString::fromUtf8("leRadiusShift"));
        leRadiusShift->setMinimumSize(QSize(30, 0));
        leRadiusShift->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leRadiusShift, 0, 2, 1, 1);

        cbReplace = new QCheckBox(LC_CircleByArcOptions);
        cbReplace->setObjectName(QString::fromUtf8("cbReplace"));

        gridLayout->addWidget(cbReplace, 0, 0, 1, 1);

        label = new QLabel(LC_CircleByArcOptions);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 3, 1, 1);


        retranslateUi(LC_CircleByArcOptions);

        QMetaObject::connectSlotsByName(LC_CircleByArcOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_CircleByArcOptions)
    {
        LC_CircleByArcOptions->setWindowTitle(QCoreApplication::translate("LC_CircleByArcOptions", "Form", nullptr));
        cbPen->setItemText(0, QCoreApplication::translate("LC_CircleByArcOptions", "Active", nullptr));
        cbPen->setItemText(1, QCoreApplication::translate("LC_CircleByArcOptions", "Original", nullptr));
        cbPen->setItemText(2, QCoreApplication::translate("LC_CircleByArcOptions", "Original Resolved", nullptr));

        lblRadiusShift->setText(QCoreApplication::translate("LC_CircleByArcOptions", "Radius Shift:", nullptr));
        label_2->setText(QCoreApplication::translate("LC_CircleByArcOptions", "Layer:", nullptr));
        cbLayer->setItemText(0, QCoreApplication::translate("LC_CircleByArcOptions", "Active", nullptr));
        cbLayer->setItemText(1, QCoreApplication::translate("LC_CircleByArcOptions", "Original", nullptr));

        cbReplace->setText(QCoreApplication::translate("LC_CircleByArcOptions", "Replace arc", nullptr));
        label->setText(QCoreApplication::translate("LC_CircleByArcOptions", "Pen:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_CircleByArcOptions: public Ui_LC_CircleByArcOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_CIRCLEBYARCOPTIONS_H
