/****************************************************************************
** Meta object code from reading C++ file 'lc_linejoinoptions.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/forms/lc_linejoinoptions.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_linejoinoptions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LC_LineJoinOptions_t {
    QByteArrayData data[17];
    char stringdata0[271];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LC_LineJoinOptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LC_LineJoinOptions_t qt_meta_stringdata_LC_LineJoinOptions = {
    {
QT_MOC_LITERAL(0, 0, 18), // "LC_LineJoinOptions"
QT_MOC_LITERAL(1, 19, 14), // "languageChange"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 11), // "doSetAction"
QT_MOC_LITERAL(4, 47, 19), // "RS_ActionInterface*"
QT_MOC_LITERAL(5, 67, 1), // "a"
QT_MOC_LITERAL(6, 69, 6), // "update"
QT_MOC_LITERAL(7, 76, 20), // "checkActionRttiValid"
QT_MOC_LITERAL(8, 97, 15), // "RS2::ActionType"
QT_MOC_LITERAL(9, 113, 10), // "actionType"
QT_MOC_LITERAL(10, 124, 20), // "onUsePolylineClicked"
QT_MOC_LITERAL(11, 145, 5), // "value"
QT_MOC_LITERAL(12, 151, 30), // "onAttributesSourceIndexChanged"
QT_MOC_LITERAL(13, 182, 5), // "index"
QT_MOC_LITERAL(14, 188, 28), // "onEdgeModelLine1IndexChanged"
QT_MOC_LITERAL(15, 217, 28), // "onEdgeModelLine2IndexChanged"
QT_MOC_LITERAL(16, 246, 24) // "onRemoveOriginalsClicked"

    },
    "LC_LineJoinOptions\0languageChange\0\0"
    "doSetAction\0RS_ActionInterface*\0a\0"
    "update\0checkActionRttiValid\0RS2::ActionType\0"
    "actionType\0onUsePolylineClicked\0value\0"
    "onAttributesSourceIndexChanged\0index\0"
    "onEdgeModelLine1IndexChanged\0"
    "onEdgeModelLine2IndexChanged\0"
    "onRemoveOriginalsClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LC_LineJoinOptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x09 /* Protected */,
       3,    2,   55,    2, 0x09 /* Protected */,
       7,    1,   60,    2, 0x09 /* Protected */,
      10,    1,   63,    2, 0x09 /* Protected */,
      12,    1,   66,    2, 0x09 /* Protected */,
      14,    1,   69,    2, 0x09 /* Protected */,
      15,    1,   72,    2, 0x09 /* Protected */,
      16,    1,   75,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Bool,    5,    6,
    QMetaType::Bool, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Bool,   11,

       0        // eod
};

void LC_LineJoinOptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LC_LineJoinOptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->languageChange(); break;
        case 1: _t->doSetAction((*reinterpret_cast< RS_ActionInterface*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: { bool _r = _t->checkActionRttiValid((*reinterpret_cast< RS2::ActionType(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->onUsePolylineClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->onAttributesSourceIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->onEdgeModelLine1IndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->onEdgeModelLine2IndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->onRemoveOriginalsClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< RS_ActionInterface* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LC_LineJoinOptions::staticMetaObject = { {
    QMetaObject::SuperData::link<LC_ActionOptionsWidget::staticMetaObject>(),
    qt_meta_stringdata_LC_LineJoinOptions.data,
    qt_meta_data_LC_LineJoinOptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LC_LineJoinOptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LC_LineJoinOptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LC_LineJoinOptions.stringdata0))
        return static_cast<void*>(this);
    return LC_ActionOptionsWidget::qt_metacast(_clname);
}

int LC_LineJoinOptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LC_ActionOptionsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
