/****************************************************************************
** Meta object code from reading C++ file 'widgetcreator.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/generic/widgetcreator.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'widgetcreator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_WidgetCreator_t {
    QByteArrayData data[16];
    char stringdata0[213];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_WidgetCreator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_WidgetCreator_t qt_meta_stringdata_WidgetCreator = {
    {
QT_MOC_LITERAL(0, 0, 13), // "WidgetCreator"
QT_MOC_LITERAL(1, 14, 14), // "widgetToCreate"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 15), // "widgetToDestroy"
QT_MOC_LITERAL(4, 46, 14), // "widgetToAssign"
QT_MOC_LITERAL(5, 61, 14), // "widgetToUpdate"
QT_MOC_LITERAL(6, 76, 15), // "addChosenAction"
QT_MOC_LITERAL(7, 92, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(8, 109, 4), // "item"
QT_MOC_LITERAL(9, 114, 18), // "removeChosenAction"
QT_MOC_LITERAL(10, 133, 8), // "setLists"
QT_MOC_LITERAL(11, 142, 13), // "destroyWidget"
QT_MOC_LITERAL(12, 156, 12), // "createWidget"
QT_MOC_LITERAL(13, 169, 17), // "requestAssignment"
QT_MOC_LITERAL(14, 187, 13), // "requestUpdate"
QT_MOC_LITERAL(15, 201, 11) // "setCategory"

    },
    "WidgetCreator\0widgetToCreate\0\0"
    "widgetToDestroy\0widgetToAssign\0"
    "widgetToUpdate\0addChosenAction\0"
    "QListWidgetItem*\0item\0removeChosenAction\0"
    "setLists\0destroyWidget\0createWidget\0"
    "requestAssignment\0requestUpdate\0"
    "setCategory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_WidgetCreator[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x06 /* Public */,
       3,    1,   87,    2, 0x06 /* Public */,
       4,    1,   90,    2, 0x06 /* Public */,
       5,    1,   93,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   96,    2, 0x08 /* Private */,
       6,    1,   97,    2, 0x08 /* Private */,
       9,    0,  100,    2, 0x08 /* Private */,
       9,    1,  101,    2, 0x08 /* Private */,
      10,    1,  104,    2, 0x08 /* Private */,
      11,    0,  107,    2, 0x08 /* Private */,
      12,    0,  108,    2, 0x08 /* Private */,
      13,    0,  109,    2, 0x08 /* Private */,
      14,    0,  110,    2, 0x08 /* Private */,
      15,    1,  111,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void WidgetCreator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<WidgetCreator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->widgetToCreate((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->widgetToDestroy((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->widgetToAssign((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->widgetToUpdate((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->addChosenAction(); break;
        case 5: _t->addChosenAction((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 6: _t->removeChosenAction(); break;
        case 7: _t->removeChosenAction((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 8: _t->setLists((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 9: _t->destroyWidget(); break;
        case 10: _t->createWidget(); break;
        case 11: _t->requestAssignment(); break;
        case 12: _t->requestUpdate(); break;
        case 13: _t->setCategory((*reinterpret_cast< QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (WidgetCreator::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WidgetCreator::widgetToCreate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (WidgetCreator::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WidgetCreator::widgetToDestroy)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (WidgetCreator::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WidgetCreator::widgetToAssign)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (WidgetCreator::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WidgetCreator::widgetToUpdate)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject WidgetCreator::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_WidgetCreator.data,
    qt_meta_data_WidgetCreator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *WidgetCreator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WidgetCreator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_WidgetCreator.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int WidgetCreator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void WidgetCreator::widgetToCreate(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void WidgetCreator::widgetToDestroy(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void WidgetCreator::widgetToAssign(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void WidgetCreator::widgetToUpdate(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
