/****************************************************************************
** Meta object code from reading C++ file 'lc_rectangle3pointsoptions.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/forms/lc_rectangle3pointsoptions.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_rectangle3pointsoptions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LC_Rectangle3PointsOptions_t {
    QByteArrayData data[17];
    char stringdata0[350];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LC_Rectangle3PointsOptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LC_Rectangle3PointsOptions_t qt_meta_stringdata_LC_Rectangle3PointsOptions = {
    {
QT_MOC_LITERAL(0, 0, 26), // "LC_Rectangle3PointsOptions"
QT_MOC_LITERAL(1, 27, 22), // "onAngleEditingFinished"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 21), // "onCornersIndexChanged"
QT_MOC_LITERAL(4, 73, 5), // "index"
QT_MOC_LITERAL(5, 79, 21), // "onLenYEditingFinished"
QT_MOC_LITERAL(6, 101, 21), // "onLenXEditingFinished"
QT_MOC_LITERAL(7, 123, 23), // "onRadiusEditingFinished"
QT_MOC_LITERAL(8, 147, 20), // "onUsePolylineClicked"
QT_MOC_LITERAL(9, 168, 5), // "value"
QT_MOC_LITERAL(10, 174, 30), // "onSnapToCornerArcCenterClicked"
QT_MOC_LITERAL(11, 205, 19), // "onQuadrangleClicked"
QT_MOC_LITERAL(12, 225, 27), // "onInnerAngleEditingFinished"
QT_MOC_LITERAL(13, 253, 24), // "onInnerAngleFixedClicked"
QT_MOC_LITERAL(14, 278, 23), // "onBaseAngleFixedClicked"
QT_MOC_LITERAL(15, 302, 19), // "onEdgesIndexChanged"
QT_MOC_LITERAL(16, 322, 27) // "getSettingsOptionNamePrefix"

    },
    "LC_Rectangle3PointsOptions\0"
    "onAngleEditingFinished\0\0onCornersIndexChanged\0"
    "index\0onLenYEditingFinished\0"
    "onLenXEditingFinished\0onRadiusEditingFinished\0"
    "onUsePolylineClicked\0value\0"
    "onSnapToCornerArcCenterClicked\0"
    "onQuadrangleClicked\0onInnerAngleEditingFinished\0"
    "onInnerAngleFixedClicked\0"
    "onBaseAngleFixedClicked\0onEdgesIndexChanged\0"
    "getSettingsOptionNamePrefix"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LC_Rectangle3PointsOptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x09 /* Protected */,
       3,    1,   80,    2, 0x09 /* Protected */,
       5,    0,   83,    2, 0x09 /* Protected */,
       6,    0,   84,    2, 0x09 /* Protected */,
       7,    0,   85,    2, 0x09 /* Protected */,
       8,    1,   86,    2, 0x09 /* Protected */,
      10,    1,   89,    2, 0x09 /* Protected */,
      11,    1,   92,    2, 0x09 /* Protected */,
      12,    0,   95,    2, 0x09 /* Protected */,
      13,    1,   96,    2, 0x09 /* Protected */,
      14,    1,   99,    2, 0x09 /* Protected */,
      15,    1,  102,    2, 0x09 /* Protected */,
      16,    0,  105,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::QString,

       0        // eod
};

void LC_Rectangle3PointsOptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LC_Rectangle3PointsOptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onAngleEditingFinished(); break;
        case 1: _t->onCornersIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onLenYEditingFinished(); break;
        case 3: _t->onLenXEditingFinished(); break;
        case 4: _t->onRadiusEditingFinished(); break;
        case 5: _t->onUsePolylineClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->onSnapToCornerArcCenterClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->onQuadrangleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->onInnerAngleEditingFinished(); break;
        case 9: _t->onInnerAngleFixedClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->onBaseAngleFixedClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->onEdgesIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: { QString _r = _t->getSettingsOptionNamePrefix();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LC_Rectangle3PointsOptions::staticMetaObject = { {
    QMetaObject::SuperData::link<LC_ActionOptionsWidget::staticMetaObject>(),
    qt_meta_stringdata_LC_Rectangle3PointsOptions.data,
    qt_meta_data_LC_Rectangle3PointsOptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LC_Rectangle3PointsOptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LC_Rectangle3PointsOptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LC_Rectangle3PointsOptions.stringdata0))
        return static_cast<void*>(this);
    return LC_ActionOptionsWidget::qt_metacast(_clname);
}

int LC_Rectangle3PointsOptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LC_ActionOptionsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
