/********************************************************************************
** Form generated from reading UI file 'lc_linepointsoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_LINEPOINTSOPTIONS_H
#define UI_LC_LINEPOINTSOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_LinePointsOptions
{
public:
    QGridLayout *gridLayout;
    QCheckBox *cbAngle;
    QSpacerItem *horizontalSpacer;
    QFrame *frmFixed;
    QGridLayout *gridLayout_2;
    QCheckBox *cbWithinLine;
    QLineEdit *leDistance;
    QFrame *line;
    QComboBox *cbEdgePoints;
    QLineEdit *leAngle;
    QSpinBox *sbPointsCount;
    QLabel *label_2;
    QLabel *label;
    QCheckBox *cbFixedDistance;

    void setupUi(QWidget *LC_LinePointsOptions)
    {
        if (LC_LinePointsOptions->objectName().isEmpty())
            LC_LinePointsOptions->setObjectName(QString::fromUtf8("LC_LinePointsOptions"));
        LC_LinePointsOptions->resize(690, 25);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(LC_LinePointsOptions->sizePolicy().hasHeightForWidth());
        LC_LinePointsOptions->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(LC_LinePointsOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        cbAngle = new QCheckBox(LC_LinePointsOptions);
        cbAngle->setObjectName(QString::fromUtf8("cbAngle"));

        gridLayout->addWidget(cbAngle, 0, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 13, 1, 1);

        frmFixed = new QFrame(LC_LinePointsOptions);
        frmFixed->setObjectName(QString::fromUtf8("frmFixed"));
        frmFixed->setFrameShape(QFrame::NoFrame);
        frmFixed->setFrameShadow(QFrame::Raised);
        gridLayout_2 = new QGridLayout(frmFixed);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(0, 0, 2, 0);
        cbWithinLine = new QCheckBox(frmFixed);
        cbWithinLine->setObjectName(QString::fromUtf8("cbWithinLine"));

        gridLayout_2->addWidget(cbWithinLine, 0, 2, 1, 1);

        leDistance = new QLineEdit(frmFixed);
        leDistance->setObjectName(QString::fromUtf8("leDistance"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(leDistance->sizePolicy().hasHeightForWidth());
        leDistance->setSizePolicy(sizePolicy1);
        leDistance->setMinimumSize(QSize(50, 0));
        leDistance->setMaximumSize(QSize(50, 16777215));
        leDistance->setFrame(true);

        gridLayout_2->addWidget(leDistance, 0, 1, 1, 1);


        gridLayout->addWidget(frmFixed, 0, 6, 1, 1);

        line = new QFrame(LC_LinePointsOptions);
        line->setObjectName(QString::fromUtf8("line"));
        line->setMaximumSize(QSize(16777215, 18));
        line->setFrameShape(QFrame::VLine);
        line->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line, 0, 9, 1, 1);

        cbEdgePoints = new QComboBox(LC_LinePointsOptions);
        cbEdgePoints->addItem(QString());
        cbEdgePoints->addItem(QString());
        cbEdgePoints->addItem(QString());
        cbEdgePoints->addItem(QString());
        cbEdgePoints->setObjectName(QString::fromUtf8("cbEdgePoints"));
        sizePolicy1.setHeightForWidth(cbEdgePoints->sizePolicy().hasHeightForWidth());
        cbEdgePoints->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(cbEdgePoints, 0, 12, 1, 1);

        leAngle = new QLineEdit(LC_LinePointsOptions);
        leAngle->setObjectName(QString::fromUtf8("leAngle"));
        sizePolicy1.setHeightForWidth(leAngle->sizePolicy().hasHeightForWidth());
        leAngle->setSizePolicy(sizePolicy1);
        leAngle->setMinimumSize(QSize(30, 0));
        leAngle->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leAngle, 0, 1, 1, 1);

        sbPointsCount = new QSpinBox(LC_LinePointsOptions);
        sbPointsCount->setObjectName(QString::fromUtf8("sbPointsCount"));
        sbPointsCount->setMinimum(1);
        sbPointsCount->setMaximum(1000);

        gridLayout->addWidget(sbPointsCount, 0, 4, 1, 1);

        label_2 = new QLabel(LC_LinePointsOptions);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 11, 1, 1);

        label = new QLabel(LC_LinePointsOptions);
        label->setObjectName(QString::fromUtf8("label"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(label, 0, 2, 1, 1);

        cbFixedDistance = new QCheckBox(LC_LinePointsOptions);
        cbFixedDistance->setObjectName(QString::fromUtf8("cbFixedDistance"));
        sizePolicy1.setHeightForWidth(cbFixedDistance->sizePolicy().hasHeightForWidth());
        cbFixedDistance->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(cbFixedDistance, 0, 5, 1, 1);


        retranslateUi(LC_LinePointsOptions);

        QMetaObject::connectSlotsByName(LC_LinePointsOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_LinePointsOptions)
    {
        LC_LinePointsOptions->setWindowTitle(QCoreApplication::translate("LC_LinePointsOptions", "Form", nullptr));
        cbAngle->setText(QCoreApplication::translate("LC_LinePointsOptions", "Angle:", nullptr));
        cbWithinLine->setText(QCoreApplication::translate("LC_LinePointsOptions", "Fit Line", nullptr));
        cbEdgePoints->setItemText(0, QCoreApplication::translate("LC_LinePointsOptions", "None", nullptr));
        cbEdgePoints->setItemText(1, QCoreApplication::translate("LC_LinePointsOptions", "Both", nullptr));
        cbEdgePoints->setItemText(2, QCoreApplication::translate("LC_LinePointsOptions", "Start", nullptr));
        cbEdgePoints->setItemText(3, QCoreApplication::translate("LC_LinePointsOptions", "End", nullptr));

        label_2->setText(QCoreApplication::translate("LC_LinePointsOptions", "Edge Points:", nullptr));
        label->setText(QCoreApplication::translate("LC_LinePointsOptions", "Number of points:", nullptr));
        cbFixedDistance->setText(QCoreApplication::translate("LC_LinePointsOptions", "Fixed Distance", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_LinePointsOptions: public Ui_LC_LinePointsOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_LINEPOINTSOPTIONS_H
