/****************************************************************************
** Meta object code from reading C++ file 'qg_actionhandler.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/qg_actionhandler.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qg_actionhandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QG_ActionHandler_t {
    QByteArrayData data[204];
    char stringdata0[3671];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QG_ActionHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QG_ActionHandler_t qt_meta_stringdata_QG_ActionHandler = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QG_ActionHandler"
QT_MOC_LITERAL(1, 17, 19), // "slotFileNewTemplate"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 12), // "slotFileOpen"
QT_MOC_LITERAL(4, 51, 14), // "slotFileSaveAs"
QT_MOC_LITERAL(5, 66, 22), // "slotFileExportMakerCam"
QT_MOC_LITERAL(6, 89, 10), // "slotZoomIn"
QT_MOC_LITERAL(7, 100, 11), // "slotZoomOut"
QT_MOC_LITERAL(8, 112, 12), // "slotZoomAuto"
QT_MOC_LITERAL(9, 125, 14), // "slotZoomWindow"
QT_MOC_LITERAL(10, 140, 11), // "slotZoomPan"
QT_MOC_LITERAL(11, 152, 16), // "slotZoomPrevious"
QT_MOC_LITERAL(12, 169, 14), // "slotZoomRedraw"
QT_MOC_LITERAL(13, 184, 28), // "slotToolRegenerateDimensions"
QT_MOC_LITERAL(14, 213, 22), // "slotEditKillAllActions"
QT_MOC_LITERAL(15, 236, 12), // "slotEditUndo"
QT_MOC_LITERAL(16, 249, 12), // "slotEditRedo"
QT_MOC_LITERAL(17, 262, 11), // "slotEditCut"
QT_MOC_LITERAL(18, 274, 12), // "slotEditCopy"
QT_MOC_LITERAL(19, 287, 13), // "slotEditPaste"
QT_MOC_LITERAL(20, 301, 15), // "slotOrderBottom"
QT_MOC_LITERAL(21, 317, 14), // "slotOrderLower"
QT_MOC_LITERAL(22, 332, 14), // "slotOrderRaise"
QT_MOC_LITERAL(23, 347, 12), // "slotOrderTop"
QT_MOC_LITERAL(24, 360, 16), // "slotSelectSingle"
QT_MOC_LITERAL(25, 377, 17), // "slotSelectContour"
QT_MOC_LITERAL(26, 395, 16), // "slotSelectWindow"
QT_MOC_LITERAL(27, 412, 18), // "slotDeselectWindow"
QT_MOC_LITERAL(28, 431, 13), // "slotSelectAll"
QT_MOC_LITERAL(29, 445, 15), // "slotDeselectAll"
QT_MOC_LITERAL(30, 461, 16), // "slotSelectInvert"
QT_MOC_LITERAL(31, 478, 21), // "slotSelectIntersected"
QT_MOC_LITERAL(32, 500, 23), // "slotDeselectIntersected"
QT_MOC_LITERAL(33, 524, 15), // "slotSelectLayer"
QT_MOC_LITERAL(34, 540, 13), // "slotDrawPoint"
QT_MOC_LITERAL(35, 554, 12), // "slotDrawLine"
QT_MOC_LITERAL(36, 567, 17), // "slotDrawLineAngle"
QT_MOC_LITERAL(37, 585, 22), // "slotDrawLineHorizontal"
QT_MOC_LITERAL(38, 608, 20), // "slotDrawLineVertical"
QT_MOC_LITERAL(39, 629, 16), // "slotDrawLineFree"
QT_MOC_LITERAL(40, 646, 19), // "slotDrawLineHorVert"
QT_MOC_LITERAL(41, 666, 20), // "slotDrawLineParallel"
QT_MOC_LITERAL(42, 687, 27), // "slotDrawLineParallelThrough"
QT_MOC_LITERAL(43, 715, 21), // "slotDrawLineRectangle"
QT_MOC_LITERAL(44, 737, 24), // "slotDrawLineRectangleRel"
QT_MOC_LITERAL(45, 762, 27), // "slotDrawLineRectangle1Point"
QT_MOC_LITERAL(46, 790, 28), // "slotDrawLineRectangle2Points"
QT_MOC_LITERAL(47, 819, 20), // "slotDrawLineBisector"
QT_MOC_LITERAL(48, 840, 20), // "slotDrawLineTangent1"
QT_MOC_LITERAL(49, 861, 20), // "slotDrawLineTangent2"
QT_MOC_LITERAL(50, 882, 22), // "slotDrawLineOrthogonal"
QT_MOC_LITERAL(51, 905, 19), // "slotDrawLineOrthTan"
QT_MOC_LITERAL(52, 925, 20), // "slotDrawLineRelAngle"
QT_MOC_LITERAL(53, 946, 19), // "slotDrawLinePolygon"
QT_MOC_LITERAL(54, 966, 20), // "slotDrawLinePolygon2"
QT_MOC_LITERAL(55, 987, 20), // "slotDrawLinePolygon3"
QT_MOC_LITERAL(56, 1008, 18), // "slotDrawLinePoints"
QT_MOC_LITERAL(57, 1027, 14), // "slotDrawCircle"
QT_MOC_LITERAL(58, 1042, 19), // "slotDrawCircleCross"
QT_MOC_LITERAL(59, 1062, 17), // "slotDrawLineSnake"
QT_MOC_LITERAL(60, 1080, 18), // "slotDrawLineSnakeX"
QT_MOC_LITERAL(61, 1099, 18), // "slotDrawLineSnakeY"
QT_MOC_LITERAL(62, 1118, 25), // "slotDrawLineOrthogonalRel"
QT_MOC_LITERAL(63, 1144, 24), // "slotDrawLineOrthogonalTo"
QT_MOC_LITERAL(64, 1169, 20), // "slotDrawLineAngleRel"
QT_MOC_LITERAL(65, 1190, 23), // "slotDrawSliceDivideLine"
QT_MOC_LITERAL(66, 1214, 25), // "slotDrawSliceDivideCircle"
QT_MOC_LITERAL(67, 1240, 16), // "slotDrawCircleCR"
QT_MOC_LITERAL(68, 1257, 16), // "slotDrawCircle2P"
QT_MOC_LITERAL(69, 1274, 17), // "slotDrawCircle2PR"
QT_MOC_LITERAL(70, 1292, 16), // "slotDrawCircle3P"
QT_MOC_LITERAL(71, 1309, 22), // "slotDrawCircleParallel"
QT_MOC_LITERAL(72, 1332, 22), // "slotDrawCircleInscribe"
QT_MOC_LITERAL(73, 1355, 18), // "slotDrawCircleTan2"
QT_MOC_LITERAL(74, 1374, 18), // "slotDrawCircleTan3"
QT_MOC_LITERAL(75, 1393, 21), // "slotDrawCircleTan1_2P"
QT_MOC_LITERAL(76, 1415, 21), // "slotDrawCircleTan2_1P"
QT_MOC_LITERAL(77, 1437, 19), // "slotDrawCircleByArc"
QT_MOC_LITERAL(78, 1457, 11), // "slotDrawArc"
QT_MOC_LITERAL(79, 1469, 13), // "slotDrawArc3P"
QT_MOC_LITERAL(80, 1483, 19), // "slotDrawArcParallel"
QT_MOC_LITERAL(81, 1503, 21), // "slotDrawArcTangential"
QT_MOC_LITERAL(82, 1525, 19), // "slotDrawEllipseAxis"
QT_MOC_LITERAL(83, 1545, 22), // "slotDrawEllipseArcAxis"
QT_MOC_LITERAL(84, 1568, 24), // "slotDrawEllipseFociPoint"
QT_MOC_LITERAL(85, 1593, 22), // "slotDrawEllipse4Points"
QT_MOC_LITERAL(86, 1616, 28), // "slotDrawEllipseCenter3Points"
QT_MOC_LITERAL(87, 1645, 23), // "slotDrawEllipseInscribe"
QT_MOC_LITERAL(88, 1669, 23), // "slotDrawParabola4Points"
QT_MOC_LITERAL(89, 1693, 18), // "slotDrawParabolaFD"
QT_MOC_LITERAL(90, 1712, 14), // "slotDrawSpline"
QT_MOC_LITERAL(91, 1727, 20), // "slotDrawSplinePoints"
QT_MOC_LITERAL(92, 1748, 13), // "slotDrawMText"
QT_MOC_LITERAL(93, 1762, 12), // "slotDrawText"
QT_MOC_LITERAL(94, 1775, 13), // "slotDrawHatch"
QT_MOC_LITERAL(95, 1789, 13), // "slotDrawImage"
QT_MOC_LITERAL(96, 1803, 16), // "slotDrawPolyline"
QT_MOC_LITERAL(97, 1820, 15), // "slotPolylineAdd"
QT_MOC_LITERAL(98, 1836, 18), // "slotPolylineAppend"
QT_MOC_LITERAL(99, 1855, 15), // "slotPolylineDel"
QT_MOC_LITERAL(100, 1871, 22), // "slotPolylineDelBetween"
QT_MOC_LITERAL(101, 1894, 16), // "slotPolylineTrim"
QT_MOC_LITERAL(102, 1911, 23), // "slotPolylineEquidistant"
QT_MOC_LITERAL(103, 1935, 19), // "slotPolylineSegment"
QT_MOC_LITERAL(104, 1955, 14), // "slotDimAligned"
QT_MOC_LITERAL(105, 1970, 13), // "slotDimLinear"
QT_MOC_LITERAL(106, 1984, 16), // "slotDimLinearHor"
QT_MOC_LITERAL(107, 2001, 16), // "slotDimLinearVer"
QT_MOC_LITERAL(108, 2018, 13), // "slotDimRadial"
QT_MOC_LITERAL(109, 2032, 16), // "slotDimDiametric"
QT_MOC_LITERAL(110, 2049, 14), // "slotDimAngular"
QT_MOC_LITERAL(111, 2064, 10), // "slotDimArc"
QT_MOC_LITERAL(112, 2075, 13), // "slotDimLeader"
QT_MOC_LITERAL(113, 2089, 20), // "slotModifyAttributes"
QT_MOC_LITERAL(114, 2110, 16), // "slotModifyDelete"
QT_MOC_LITERAL(115, 2127, 21), // "slotModifyDeleteQuick"
QT_MOC_LITERAL(116, 2149, 20), // "slotModifyDeleteFree"
QT_MOC_LITERAL(117, 2170, 14), // "slotModifyMove"
QT_MOC_LITERAL(118, 2185, 15), // "slotModifyScale"
QT_MOC_LITERAL(119, 2201, 25), // "slotModifyRevertDirection"
QT_MOC_LITERAL(120, 2227, 16), // "slotModifyRotate"
QT_MOC_LITERAL(121, 2244, 16), // "slotModifyMirror"
QT_MOC_LITERAL(122, 2261, 20), // "slotModifyMoveRotate"
QT_MOC_LITERAL(123, 2282, 17), // "slotModifyRotate2"
QT_MOC_LITERAL(124, 2300, 16), // "slotModifyEntity"
QT_MOC_LITERAL(125, 2317, 14), // "slotModifyTrim"
QT_MOC_LITERAL(126, 2332, 15), // "slotModifyTrim2"
QT_MOC_LITERAL(127, 2348, 20), // "slotModifyTrimAmount"
QT_MOC_LITERAL(128, 2369, 13), // "slotModifyCut"
QT_MOC_LITERAL(129, 2383, 17), // "slotModifyStretch"
QT_MOC_LITERAL(130, 2401, 15), // "slotModifyBevel"
QT_MOC_LITERAL(131, 2417, 15), // "slotModifyRound"
QT_MOC_LITERAL(132, 2433, 16), // "slotModifyOffset"
QT_MOC_LITERAL(133, 2450, 21), // "slotModifyExplodeText"
QT_MOC_LITERAL(134, 2472, 12), // "slotSetSnaps"
QT_MOC_LITERAL(135, 2485, 11), // "RS_SnapMode"
QT_MOC_LITERAL(136, 2497, 1), // "s"
QT_MOC_LITERAL(137, 2499, 12), // "slotSnapFree"
QT_MOC_LITERAL(138, 2512, 12), // "slotSnapGrid"
QT_MOC_LITERAL(139, 2525, 16), // "slotSnapEndpoint"
QT_MOC_LITERAL(140, 2542, 16), // "slotSnapOnEntity"
QT_MOC_LITERAL(141, 2559, 14), // "slotSnapCenter"
QT_MOC_LITERAL(142, 2574, 14), // "slotSnapMiddle"
QT_MOC_LITERAL(143, 2589, 12), // "slotSnapDist"
QT_MOC_LITERAL(144, 2602, 20), // "slotSnapMiddleManual"
QT_MOC_LITERAL(145, 2623, 20), // "slotSnapIntersection"
QT_MOC_LITERAL(146, 2644, 26), // "slotSnapIntersectionManual"
QT_MOC_LITERAL(147, 2671, 19), // "slotRestrictNothing"
QT_MOC_LITERAL(148, 2691, 22), // "slotRestrictOrthogonal"
QT_MOC_LITERAL(149, 2714, 22), // "slotRestrictHorizontal"
QT_MOC_LITERAL(150, 2737, 20), // "slotRestrictVertical"
QT_MOC_LITERAL(151, 2758, 12), // "disableSnaps"
QT_MOC_LITERAL(152, 2771, 19), // "disableRestrictions"
QT_MOC_LITERAL(153, 2791, 19), // "slotSetRelativeZero"
QT_MOC_LITERAL(154, 2811, 20), // "slotLockRelativeZero"
QT_MOC_LITERAL(155, 2832, 2), // "on"
QT_MOC_LITERAL(156, 2835, 14), // "slotInfoInside"
QT_MOC_LITERAL(157, 2850, 12), // "slotInfoDist"
QT_MOC_LITERAL(158, 2863, 13), // "slotInfoDist2"
QT_MOC_LITERAL(159, 2877, 13), // "slotInfoAngle"
QT_MOC_LITERAL(160, 2891, 19), // "slotInfoTotalLength"
QT_MOC_LITERAL(161, 2911, 12), // "slotInfoArea"
QT_MOC_LITERAL(162, 2924, 21), // "slotLayersDefreezeAll"
QT_MOC_LITERAL(163, 2946, 19), // "slotLayersFreezeAll"
QT_MOC_LITERAL(164, 2966, 19), // "slotLayersUnlockAll"
QT_MOC_LITERAL(165, 2986, 17), // "slotLayersLockAll"
QT_MOC_LITERAL(166, 3004, 13), // "slotLayersAdd"
QT_MOC_LITERAL(167, 3018, 16), // "slotLayersRemove"
QT_MOC_LITERAL(168, 3035, 14), // "slotLayersEdit"
QT_MOC_LITERAL(169, 3050, 20), // "slotLayersToggleView"
QT_MOC_LITERAL(170, 3071, 20), // "slotLayersToggleLock"
QT_MOC_LITERAL(171, 3092, 21), // "slotLayersTogglePrint"
QT_MOC_LITERAL(172, 3114, 28), // "slotLayersToggleConstruction"
QT_MOC_LITERAL(173, 3143, 24), // "slotLayersExportSelected"
QT_MOC_LITERAL(174, 3168, 23), // "slotLayersExportVisible"
QT_MOC_LITERAL(175, 3192, 21), // "slotBlocksDefreezeAll"
QT_MOC_LITERAL(176, 3214, 19), // "slotBlocksFreezeAll"
QT_MOC_LITERAL(177, 3234, 13), // "slotBlocksAdd"
QT_MOC_LITERAL(178, 3248, 16), // "slotBlocksRemove"
QT_MOC_LITERAL(179, 3265, 20), // "slotBlocksAttributes"
QT_MOC_LITERAL(180, 3286, 14), // "slotBlocksEdit"
QT_MOC_LITERAL(181, 3301, 14), // "slotBlocksSave"
QT_MOC_LITERAL(182, 3316, 16), // "slotBlocksInsert"
QT_MOC_LITERAL(183, 3333, 20), // "slotBlocksToggleView"
QT_MOC_LITERAL(184, 3354, 16), // "slotBlocksCreate"
QT_MOC_LITERAL(185, 3371, 17), // "slotBlocksExplode"
QT_MOC_LITERAL(186, 3389, 18), // "slotModifyLineJoin"
QT_MOC_LITERAL(187, 3408, 19), // "slotModifyDuplicate"
QT_MOC_LITERAL(188, 3428, 12), // "slotDrawStar"
QT_MOC_LITERAL(189, 3441, 21), // "slotModifyBreakDivide"
QT_MOC_LITERAL(190, 3463, 17), // "slotModifyLineGap"
QT_MOC_LITERAL(191, 3481, 18), // "slotOptionsDrawing"
QT_MOC_LITERAL(192, 3500, 16), // "toggleVisibility"
QT_MOC_LITERAL(193, 3517, 9), // "RS_Layer*"
QT_MOC_LITERAL(194, 3527, 5), // "layer"
QT_MOC_LITERAL(195, 3533, 10), // "toggleLock"
QT_MOC_LITERAL(196, 3544, 11), // "togglePrint"
QT_MOC_LITERAL(197, 3556, 18), // "toggleConstruction"
QT_MOC_LITERAL(198, 3575, 17), // "slotRedockWidgets"
QT_MOC_LITERAL(199, 3593, 11), // "slotPenPick"
QT_MOC_LITERAL(200, 3605, 19), // "slotPenPickResolved"
QT_MOC_LITERAL(201, 3625, 12), // "slotPenApply"
QT_MOC_LITERAL(202, 3638, 11), // "slotPenCopy"
QT_MOC_LITERAL(203, 3650, 20) // "slotPenSyncFromLayer"

    },
    "QG_ActionHandler\0slotFileNewTemplate\0"
    "\0slotFileOpen\0slotFileSaveAs\0"
    "slotFileExportMakerCam\0slotZoomIn\0"
    "slotZoomOut\0slotZoomAuto\0slotZoomWindow\0"
    "slotZoomPan\0slotZoomPrevious\0"
    "slotZoomRedraw\0slotToolRegenerateDimensions\0"
    "slotEditKillAllActions\0slotEditUndo\0"
    "slotEditRedo\0slotEditCut\0slotEditCopy\0"
    "slotEditPaste\0slotOrderBottom\0"
    "slotOrderLower\0slotOrderRaise\0"
    "slotOrderTop\0slotSelectSingle\0"
    "slotSelectContour\0slotSelectWindow\0"
    "slotDeselectWindow\0slotSelectAll\0"
    "slotDeselectAll\0slotSelectInvert\0"
    "slotSelectIntersected\0slotDeselectIntersected\0"
    "slotSelectLayer\0slotDrawPoint\0"
    "slotDrawLine\0slotDrawLineAngle\0"
    "slotDrawLineHorizontal\0slotDrawLineVertical\0"
    "slotDrawLineFree\0slotDrawLineHorVert\0"
    "slotDrawLineParallel\0slotDrawLineParallelThrough\0"
    "slotDrawLineRectangle\0slotDrawLineRectangleRel\0"
    "slotDrawLineRectangle1Point\0"
    "slotDrawLineRectangle2Points\0"
    "slotDrawLineBisector\0slotDrawLineTangent1\0"
    "slotDrawLineTangent2\0slotDrawLineOrthogonal\0"
    "slotDrawLineOrthTan\0slotDrawLineRelAngle\0"
    "slotDrawLinePolygon\0slotDrawLinePolygon2\0"
    "slotDrawLinePolygon3\0slotDrawLinePoints\0"
    "slotDrawCircle\0slotDrawCircleCross\0"
    "slotDrawLineSnake\0slotDrawLineSnakeX\0"
    "slotDrawLineSnakeY\0slotDrawLineOrthogonalRel\0"
    "slotDrawLineOrthogonalTo\0slotDrawLineAngleRel\0"
    "slotDrawSliceDivideLine\0"
    "slotDrawSliceDivideCircle\0slotDrawCircleCR\0"
    "slotDrawCircle2P\0slotDrawCircle2PR\0"
    "slotDrawCircle3P\0slotDrawCircleParallel\0"
    "slotDrawCircleInscribe\0slotDrawCircleTan2\0"
    "slotDrawCircleTan3\0slotDrawCircleTan1_2P\0"
    "slotDrawCircleTan2_1P\0slotDrawCircleByArc\0"
    "slotDrawArc\0slotDrawArc3P\0slotDrawArcParallel\0"
    "slotDrawArcTangential\0slotDrawEllipseAxis\0"
    "slotDrawEllipseArcAxis\0slotDrawEllipseFociPoint\0"
    "slotDrawEllipse4Points\0"
    "slotDrawEllipseCenter3Points\0"
    "slotDrawEllipseInscribe\0slotDrawParabola4Points\0"
    "slotDrawParabolaFD\0slotDrawSpline\0"
    "slotDrawSplinePoints\0slotDrawMText\0"
    "slotDrawText\0slotDrawHatch\0slotDrawImage\0"
    "slotDrawPolyline\0slotPolylineAdd\0"
    "slotPolylineAppend\0slotPolylineDel\0"
    "slotPolylineDelBetween\0slotPolylineTrim\0"
    "slotPolylineEquidistant\0slotPolylineSegment\0"
    "slotDimAligned\0slotDimLinear\0"
    "slotDimLinearHor\0slotDimLinearVer\0"
    "slotDimRadial\0slotDimDiametric\0"
    "slotDimAngular\0slotDimArc\0slotDimLeader\0"
    "slotModifyAttributes\0slotModifyDelete\0"
    "slotModifyDeleteQuick\0slotModifyDeleteFree\0"
    "slotModifyMove\0slotModifyScale\0"
    "slotModifyRevertDirection\0slotModifyRotate\0"
    "slotModifyMirror\0slotModifyMoveRotate\0"
    "slotModifyRotate2\0slotModifyEntity\0"
    "slotModifyTrim\0slotModifyTrim2\0"
    "slotModifyTrimAmount\0slotModifyCut\0"
    "slotModifyStretch\0slotModifyBevel\0"
    "slotModifyRound\0slotModifyOffset\0"
    "slotModifyExplodeText\0slotSetSnaps\0"
    "RS_SnapMode\0s\0slotSnapFree\0slotSnapGrid\0"
    "slotSnapEndpoint\0slotSnapOnEntity\0"
    "slotSnapCenter\0slotSnapMiddle\0"
    "slotSnapDist\0slotSnapMiddleManual\0"
    "slotSnapIntersection\0slotSnapIntersectionManual\0"
    "slotRestrictNothing\0slotRestrictOrthogonal\0"
    "slotRestrictHorizontal\0slotRestrictVertical\0"
    "disableSnaps\0disableRestrictions\0"
    "slotSetRelativeZero\0slotLockRelativeZero\0"
    "on\0slotInfoInside\0slotInfoDist\0"
    "slotInfoDist2\0slotInfoAngle\0"
    "slotInfoTotalLength\0slotInfoArea\0"
    "slotLayersDefreezeAll\0slotLayersFreezeAll\0"
    "slotLayersUnlockAll\0slotLayersLockAll\0"
    "slotLayersAdd\0slotLayersRemove\0"
    "slotLayersEdit\0slotLayersToggleView\0"
    "slotLayersToggleLock\0slotLayersTogglePrint\0"
    "slotLayersToggleConstruction\0"
    "slotLayersExportSelected\0"
    "slotLayersExportVisible\0slotBlocksDefreezeAll\0"
    "slotBlocksFreezeAll\0slotBlocksAdd\0"
    "slotBlocksRemove\0slotBlocksAttributes\0"
    "slotBlocksEdit\0slotBlocksSave\0"
    "slotBlocksInsert\0slotBlocksToggleView\0"
    "slotBlocksCreate\0slotBlocksExplode\0"
    "slotModifyLineJoin\0slotModifyDuplicate\0"
    "slotDrawStar\0slotModifyBreakDivide\0"
    "slotModifyLineGap\0slotOptionsDrawing\0"
    "toggleVisibility\0RS_Layer*\0layer\0"
    "toggleLock\0togglePrint\0toggleConstruction\0"
    "slotRedockWidgets\0slotPenPick\0"
    "slotPenPickResolved\0slotPenApply\0"
    "slotPenCopy\0slotPenSyncFromLayer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QG_ActionHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     197,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  999,    2, 0x0a /* Public */,
       3,    0, 1000,    2, 0x0a /* Public */,
       4,    0, 1001,    2, 0x0a /* Public */,
       5,    0, 1002,    2, 0x0a /* Public */,
       6,    0, 1003,    2, 0x0a /* Public */,
       7,    0, 1004,    2, 0x0a /* Public */,
       8,    0, 1005,    2, 0x0a /* Public */,
       9,    0, 1006,    2, 0x0a /* Public */,
      10,    0, 1007,    2, 0x0a /* Public */,
      11,    0, 1008,    2, 0x0a /* Public */,
      12,    0, 1009,    2, 0x0a /* Public */,
      13,    0, 1010,    2, 0x0a /* Public */,
      14,    0, 1011,    2, 0x0a /* Public */,
      15,    0, 1012,    2, 0x0a /* Public */,
      16,    0, 1013,    2, 0x0a /* Public */,
      17,    0, 1014,    2, 0x0a /* Public */,
      18,    0, 1015,    2, 0x0a /* Public */,
      19,    0, 1016,    2, 0x0a /* Public */,
      20,    0, 1017,    2, 0x0a /* Public */,
      21,    0, 1018,    2, 0x0a /* Public */,
      22,    0, 1019,    2, 0x0a /* Public */,
      23,    0, 1020,    2, 0x0a /* Public */,
      24,    0, 1021,    2, 0x0a /* Public */,
      25,    0, 1022,    2, 0x0a /* Public */,
      26,    0, 1023,    2, 0x0a /* Public */,
      27,    0, 1024,    2, 0x0a /* Public */,
      28,    0, 1025,    2, 0x0a /* Public */,
      29,    0, 1026,    2, 0x0a /* Public */,
      30,    0, 1027,    2, 0x0a /* Public */,
      31,    0, 1028,    2, 0x0a /* Public */,
      32,    0, 1029,    2, 0x0a /* Public */,
      33,    0, 1030,    2, 0x0a /* Public */,
      34,    0, 1031,    2, 0x0a /* Public */,
      35,    0, 1032,    2, 0x0a /* Public */,
      36,    0, 1033,    2, 0x0a /* Public */,
      37,    0, 1034,    2, 0x0a /* Public */,
      38,    0, 1035,    2, 0x0a /* Public */,
      39,    0, 1036,    2, 0x0a /* Public */,
      40,    0, 1037,    2, 0x0a /* Public */,
      41,    0, 1038,    2, 0x0a /* Public */,
      42,    0, 1039,    2, 0x0a /* Public */,
      43,    0, 1040,    2, 0x0a /* Public */,
      44,    0, 1041,    2, 0x0a /* Public */,
      45,    0, 1042,    2, 0x0a /* Public */,
      46,    0, 1043,    2, 0x0a /* Public */,
      47,    0, 1044,    2, 0x0a /* Public */,
      48,    0, 1045,    2, 0x0a /* Public */,
      49,    0, 1046,    2, 0x0a /* Public */,
      50,    0, 1047,    2, 0x0a /* Public */,
      51,    0, 1048,    2, 0x0a /* Public */,
      52,    0, 1049,    2, 0x0a /* Public */,
      53,    0, 1050,    2, 0x0a /* Public */,
      54,    0, 1051,    2, 0x0a /* Public */,
      55,    0, 1052,    2, 0x0a /* Public */,
      56,    0, 1053,    2, 0x0a /* Public */,
      57,    0, 1054,    2, 0x0a /* Public */,
      58,    0, 1055,    2, 0x0a /* Public */,
      59,    0, 1056,    2, 0x0a /* Public */,
      60,    0, 1057,    2, 0x0a /* Public */,
      61,    0, 1058,    2, 0x0a /* Public */,
      62,    0, 1059,    2, 0x0a /* Public */,
      63,    0, 1060,    2, 0x0a /* Public */,
      64,    0, 1061,    2, 0x0a /* Public */,
      65,    0, 1062,    2, 0x0a /* Public */,
      66,    0, 1063,    2, 0x0a /* Public */,
      67,    0, 1064,    2, 0x0a /* Public */,
      68,    0, 1065,    2, 0x0a /* Public */,
      69,    0, 1066,    2, 0x0a /* Public */,
      70,    0, 1067,    2, 0x0a /* Public */,
      71,    0, 1068,    2, 0x0a /* Public */,
      72,    0, 1069,    2, 0x0a /* Public */,
      73,    0, 1070,    2, 0x0a /* Public */,
      74,    0, 1071,    2, 0x0a /* Public */,
      75,    0, 1072,    2, 0x0a /* Public */,
      76,    0, 1073,    2, 0x0a /* Public */,
      77,    0, 1074,    2, 0x0a /* Public */,
      78,    0, 1075,    2, 0x0a /* Public */,
      79,    0, 1076,    2, 0x0a /* Public */,
      80,    0, 1077,    2, 0x0a /* Public */,
      81,    0, 1078,    2, 0x0a /* Public */,
      82,    0, 1079,    2, 0x0a /* Public */,
      83,    0, 1080,    2, 0x0a /* Public */,
      84,    0, 1081,    2, 0x0a /* Public */,
      85,    0, 1082,    2, 0x0a /* Public */,
      86,    0, 1083,    2, 0x0a /* Public */,
      87,    0, 1084,    2, 0x0a /* Public */,
      88,    0, 1085,    2, 0x0a /* Public */,
      89,    0, 1086,    2, 0x0a /* Public */,
      90,    0, 1087,    2, 0x0a /* Public */,
      91,    0, 1088,    2, 0x0a /* Public */,
      92,    0, 1089,    2, 0x0a /* Public */,
      93,    0, 1090,    2, 0x0a /* Public */,
      94,    0, 1091,    2, 0x0a /* Public */,
      95,    0, 1092,    2, 0x0a /* Public */,
      96,    0, 1093,    2, 0x0a /* Public */,
      97,    0, 1094,    2, 0x0a /* Public */,
      98,    0, 1095,    2, 0x0a /* Public */,
      99,    0, 1096,    2, 0x0a /* Public */,
     100,    0, 1097,    2, 0x0a /* Public */,
     101,    0, 1098,    2, 0x0a /* Public */,
     102,    0, 1099,    2, 0x0a /* Public */,
     103,    0, 1100,    2, 0x0a /* Public */,
     104,    0, 1101,    2, 0x0a /* Public */,
     105,    0, 1102,    2, 0x0a /* Public */,
     106,    0, 1103,    2, 0x0a /* Public */,
     107,    0, 1104,    2, 0x0a /* Public */,
     108,    0, 1105,    2, 0x0a /* Public */,
     109,    0, 1106,    2, 0x0a /* Public */,
     110,    0, 1107,    2, 0x0a /* Public */,
     111,    0, 1108,    2, 0x0a /* Public */,
     112,    0, 1109,    2, 0x0a /* Public */,
     113,    0, 1110,    2, 0x0a /* Public */,
     114,    0, 1111,    2, 0x0a /* Public */,
     115,    0, 1112,    2, 0x0a /* Public */,
     116,    0, 1113,    2, 0x0a /* Public */,
     117,    0, 1114,    2, 0x0a /* Public */,
     118,    0, 1115,    2, 0x0a /* Public */,
     119,    0, 1116,    2, 0x0a /* Public */,
     120,    0, 1117,    2, 0x0a /* Public */,
     121,    0, 1118,    2, 0x0a /* Public */,
     122,    0, 1119,    2, 0x0a /* Public */,
     123,    0, 1120,    2, 0x0a /* Public */,
     124,    0, 1121,    2, 0x0a /* Public */,
     125,    0, 1122,    2, 0x0a /* Public */,
     126,    0, 1123,    2, 0x0a /* Public */,
     127,    0, 1124,    2, 0x0a /* Public */,
     128,    0, 1125,    2, 0x0a /* Public */,
     129,    0, 1126,    2, 0x0a /* Public */,
     130,    0, 1127,    2, 0x0a /* Public */,
     131,    0, 1128,    2, 0x0a /* Public */,
     132,    0, 1129,    2, 0x0a /* Public */,
     133,    0, 1130,    2, 0x0a /* Public */,
     134,    1, 1131,    2, 0x0a /* Public */,
     137,    0, 1134,    2, 0x0a /* Public */,
     138,    0, 1135,    2, 0x0a /* Public */,
     139,    0, 1136,    2, 0x0a /* Public */,
     140,    0, 1137,    2, 0x0a /* Public */,
     141,    0, 1138,    2, 0x0a /* Public */,
     142,    0, 1139,    2, 0x0a /* Public */,
     143,    0, 1140,    2, 0x0a /* Public */,
     144,    0, 1141,    2, 0x0a /* Public */,
     145,    0, 1142,    2, 0x0a /* Public */,
     146,    0, 1143,    2, 0x0a /* Public */,
     147,    0, 1144,    2, 0x0a /* Public */,
     148,    0, 1145,    2, 0x0a /* Public */,
     149,    0, 1146,    2, 0x0a /* Public */,
     150,    0, 1147,    2, 0x0a /* Public */,
     151,    0, 1148,    2, 0x0a /* Public */,
     152,    0, 1149,    2, 0x0a /* Public */,
     153,    0, 1150,    2, 0x0a /* Public */,
     154,    1, 1151,    2, 0x0a /* Public */,
     156,    0, 1154,    2, 0x0a /* Public */,
     157,    0, 1155,    2, 0x0a /* Public */,
     158,    0, 1156,    2, 0x0a /* Public */,
     159,    0, 1157,    2, 0x0a /* Public */,
     160,    0, 1158,    2, 0x0a /* Public */,
     161,    0, 1159,    2, 0x0a /* Public */,
     162,    0, 1160,    2, 0x0a /* Public */,
     163,    0, 1161,    2, 0x0a /* Public */,
     164,    0, 1162,    2, 0x0a /* Public */,
     165,    0, 1163,    2, 0x0a /* Public */,
     166,    0, 1164,    2, 0x0a /* Public */,
     167,    0, 1165,    2, 0x0a /* Public */,
     168,    0, 1166,    2, 0x0a /* Public */,
     169,    0, 1167,    2, 0x0a /* Public */,
     170,    0, 1168,    2, 0x0a /* Public */,
     171,    0, 1169,    2, 0x0a /* Public */,
     172,    0, 1170,    2, 0x0a /* Public */,
     173,    0, 1171,    2, 0x0a /* Public */,
     174,    0, 1172,    2, 0x0a /* Public */,
     175,    0, 1173,    2, 0x0a /* Public */,
     176,    0, 1174,    2, 0x0a /* Public */,
     177,    0, 1175,    2, 0x0a /* Public */,
     178,    0, 1176,    2, 0x0a /* Public */,
     179,    0, 1177,    2, 0x0a /* Public */,
     180,    0, 1178,    2, 0x0a /* Public */,
     181,    0, 1179,    2, 0x0a /* Public */,
     182,    0, 1180,    2, 0x0a /* Public */,
     183,    0, 1181,    2, 0x0a /* Public */,
     184,    0, 1182,    2, 0x0a /* Public */,
     185,    0, 1183,    2, 0x0a /* Public */,
     186,    0, 1184,    2, 0x0a /* Public */,
     187,    0, 1185,    2, 0x0a /* Public */,
     188,    0, 1186,    2, 0x0a /* Public */,
     189,    0, 1187,    2, 0x0a /* Public */,
     190,    0, 1188,    2, 0x0a /* Public */,
     191,    0, 1189,    2, 0x0a /* Public */,
     192,    1, 1190,    2, 0x0a /* Public */,
     195,    1, 1193,    2, 0x0a /* Public */,
     196,    1, 1196,    2, 0x0a /* Public */,
     197,    1, 1199,    2, 0x0a /* Public */,
     198,    0, 1202,    2, 0x0a /* Public */,
     199,    0, 1203,    2, 0x0a /* Public */,
     200,    0, 1204,    2, 0x0a /* Public */,
     201,    0, 1205,    2, 0x0a /* Public */,
     202,    0, 1206,    2, 0x0a /* Public */,
     203,    0, 1207,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 135,  136,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  155,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 193,  194,
    QMetaType::Void, 0x80000000 | 193,  194,
    QMetaType::Void, 0x80000000 | 193,  194,
    QMetaType::Void, 0x80000000 | 193,  194,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QG_ActionHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QG_ActionHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotFileNewTemplate(); break;
        case 1: _t->slotFileOpen(); break;
        case 2: _t->slotFileSaveAs(); break;
        case 3: _t->slotFileExportMakerCam(); break;
        case 4: _t->slotZoomIn(); break;
        case 5: _t->slotZoomOut(); break;
        case 6: _t->slotZoomAuto(); break;
        case 7: _t->slotZoomWindow(); break;
        case 8: _t->slotZoomPan(); break;
        case 9: _t->slotZoomPrevious(); break;
        case 10: _t->slotZoomRedraw(); break;
        case 11: _t->slotToolRegenerateDimensions(); break;
        case 12: _t->slotEditKillAllActions(); break;
        case 13: _t->slotEditUndo(); break;
        case 14: _t->slotEditRedo(); break;
        case 15: _t->slotEditCut(); break;
        case 16: _t->slotEditCopy(); break;
        case 17: _t->slotEditPaste(); break;
        case 18: _t->slotOrderBottom(); break;
        case 19: _t->slotOrderLower(); break;
        case 20: _t->slotOrderRaise(); break;
        case 21: _t->slotOrderTop(); break;
        case 22: _t->slotSelectSingle(); break;
        case 23: _t->slotSelectContour(); break;
        case 24: _t->slotSelectWindow(); break;
        case 25: _t->slotDeselectWindow(); break;
        case 26: _t->slotSelectAll(); break;
        case 27: _t->slotDeselectAll(); break;
        case 28: _t->slotSelectInvert(); break;
        case 29: _t->slotSelectIntersected(); break;
        case 30: _t->slotDeselectIntersected(); break;
        case 31: _t->slotSelectLayer(); break;
        case 32: _t->slotDrawPoint(); break;
        case 33: _t->slotDrawLine(); break;
        case 34: _t->slotDrawLineAngle(); break;
        case 35: _t->slotDrawLineHorizontal(); break;
        case 36: _t->slotDrawLineVertical(); break;
        case 37: _t->slotDrawLineFree(); break;
        case 38: _t->slotDrawLineHorVert(); break;
        case 39: _t->slotDrawLineParallel(); break;
        case 40: _t->slotDrawLineParallelThrough(); break;
        case 41: _t->slotDrawLineRectangle(); break;
        case 42: _t->slotDrawLineRectangleRel(); break;
        case 43: _t->slotDrawLineRectangle1Point(); break;
        case 44: _t->slotDrawLineRectangle2Points(); break;
        case 45: _t->slotDrawLineBisector(); break;
        case 46: _t->slotDrawLineTangent1(); break;
        case 47: _t->slotDrawLineTangent2(); break;
        case 48: _t->slotDrawLineOrthogonal(); break;
        case 49: _t->slotDrawLineOrthTan(); break;
        case 50: _t->slotDrawLineRelAngle(); break;
        case 51: _t->slotDrawLinePolygon(); break;
        case 52: _t->slotDrawLinePolygon2(); break;
        case 53: _t->slotDrawLinePolygon3(); break;
        case 54: _t->slotDrawLinePoints(); break;
        case 55: _t->slotDrawCircle(); break;
        case 56: _t->slotDrawCircleCross(); break;
        case 57: _t->slotDrawLineSnake(); break;
        case 58: _t->slotDrawLineSnakeX(); break;
        case 59: _t->slotDrawLineSnakeY(); break;
        case 60: _t->slotDrawLineOrthogonalRel(); break;
        case 61: _t->slotDrawLineOrthogonalTo(); break;
        case 62: _t->slotDrawLineAngleRel(); break;
        case 63: _t->slotDrawSliceDivideLine(); break;
        case 64: _t->slotDrawSliceDivideCircle(); break;
        case 65: _t->slotDrawCircleCR(); break;
        case 66: _t->slotDrawCircle2P(); break;
        case 67: _t->slotDrawCircle2PR(); break;
        case 68: _t->slotDrawCircle3P(); break;
        case 69: _t->slotDrawCircleParallel(); break;
        case 70: _t->slotDrawCircleInscribe(); break;
        case 71: _t->slotDrawCircleTan2(); break;
        case 72: _t->slotDrawCircleTan3(); break;
        case 73: _t->slotDrawCircleTan1_2P(); break;
        case 74: _t->slotDrawCircleTan2_1P(); break;
        case 75: _t->slotDrawCircleByArc(); break;
        case 76: _t->slotDrawArc(); break;
        case 77: _t->slotDrawArc3P(); break;
        case 78: _t->slotDrawArcParallel(); break;
        case 79: _t->slotDrawArcTangential(); break;
        case 80: _t->slotDrawEllipseAxis(); break;
        case 81: _t->slotDrawEllipseArcAxis(); break;
        case 82: _t->slotDrawEllipseFociPoint(); break;
        case 83: _t->slotDrawEllipse4Points(); break;
        case 84: _t->slotDrawEllipseCenter3Points(); break;
        case 85: _t->slotDrawEllipseInscribe(); break;
        case 86: _t->slotDrawParabola4Points(); break;
        case 87: _t->slotDrawParabolaFD(); break;
        case 88: _t->slotDrawSpline(); break;
        case 89: _t->slotDrawSplinePoints(); break;
        case 90: _t->slotDrawMText(); break;
        case 91: _t->slotDrawText(); break;
        case 92: _t->slotDrawHatch(); break;
        case 93: _t->slotDrawImage(); break;
        case 94: _t->slotDrawPolyline(); break;
        case 95: _t->slotPolylineAdd(); break;
        case 96: _t->slotPolylineAppend(); break;
        case 97: _t->slotPolylineDel(); break;
        case 98: _t->slotPolylineDelBetween(); break;
        case 99: _t->slotPolylineTrim(); break;
        case 100: _t->slotPolylineEquidistant(); break;
        case 101: _t->slotPolylineSegment(); break;
        case 102: _t->slotDimAligned(); break;
        case 103: _t->slotDimLinear(); break;
        case 104: _t->slotDimLinearHor(); break;
        case 105: _t->slotDimLinearVer(); break;
        case 106: _t->slotDimRadial(); break;
        case 107: _t->slotDimDiametric(); break;
        case 108: _t->slotDimAngular(); break;
        case 109: _t->slotDimArc(); break;
        case 110: _t->slotDimLeader(); break;
        case 111: _t->slotModifyAttributes(); break;
        case 112: _t->slotModifyDelete(); break;
        case 113: _t->slotModifyDeleteQuick(); break;
        case 114: _t->slotModifyDeleteFree(); break;
        case 115: _t->slotModifyMove(); break;
        case 116: _t->slotModifyScale(); break;
        case 117: _t->slotModifyRevertDirection(); break;
        case 118: _t->slotModifyRotate(); break;
        case 119: _t->slotModifyMirror(); break;
        case 120: _t->slotModifyMoveRotate(); break;
        case 121: _t->slotModifyRotate2(); break;
        case 122: _t->slotModifyEntity(); break;
        case 123: _t->slotModifyTrim(); break;
        case 124: _t->slotModifyTrim2(); break;
        case 125: _t->slotModifyTrimAmount(); break;
        case 126: _t->slotModifyCut(); break;
        case 127: _t->slotModifyStretch(); break;
        case 128: _t->slotModifyBevel(); break;
        case 129: _t->slotModifyRound(); break;
        case 130: _t->slotModifyOffset(); break;
        case 131: _t->slotModifyExplodeText(); break;
        case 132: _t->slotSetSnaps((*reinterpret_cast< const RS_SnapMode(*)>(_a[1]))); break;
        case 133: _t->slotSnapFree(); break;
        case 134: _t->slotSnapGrid(); break;
        case 135: _t->slotSnapEndpoint(); break;
        case 136: _t->slotSnapOnEntity(); break;
        case 137: _t->slotSnapCenter(); break;
        case 138: _t->slotSnapMiddle(); break;
        case 139: _t->slotSnapDist(); break;
        case 140: _t->slotSnapMiddleManual(); break;
        case 141: _t->slotSnapIntersection(); break;
        case 142: _t->slotSnapIntersectionManual(); break;
        case 143: _t->slotRestrictNothing(); break;
        case 144: _t->slotRestrictOrthogonal(); break;
        case 145: _t->slotRestrictHorizontal(); break;
        case 146: _t->slotRestrictVertical(); break;
        case 147: _t->disableSnaps(); break;
        case 148: _t->disableRestrictions(); break;
        case 149: _t->slotSetRelativeZero(); break;
        case 150: _t->slotLockRelativeZero((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 151: _t->slotInfoInside(); break;
        case 152: _t->slotInfoDist(); break;
        case 153: _t->slotInfoDist2(); break;
        case 154: _t->slotInfoAngle(); break;
        case 155: _t->slotInfoTotalLength(); break;
        case 156: _t->slotInfoArea(); break;
        case 157: _t->slotLayersDefreezeAll(); break;
        case 158: _t->slotLayersFreezeAll(); break;
        case 159: _t->slotLayersUnlockAll(); break;
        case 160: _t->slotLayersLockAll(); break;
        case 161: _t->slotLayersAdd(); break;
        case 162: _t->slotLayersRemove(); break;
        case 163: _t->slotLayersEdit(); break;
        case 164: _t->slotLayersToggleView(); break;
        case 165: _t->slotLayersToggleLock(); break;
        case 166: _t->slotLayersTogglePrint(); break;
        case 167: _t->slotLayersToggleConstruction(); break;
        case 168: _t->slotLayersExportSelected(); break;
        case 169: _t->slotLayersExportVisible(); break;
        case 170: _t->slotBlocksDefreezeAll(); break;
        case 171: _t->slotBlocksFreezeAll(); break;
        case 172: _t->slotBlocksAdd(); break;
        case 173: _t->slotBlocksRemove(); break;
        case 174: _t->slotBlocksAttributes(); break;
        case 175: _t->slotBlocksEdit(); break;
        case 176: _t->slotBlocksSave(); break;
        case 177: _t->slotBlocksInsert(); break;
        case 178: _t->slotBlocksToggleView(); break;
        case 179: _t->slotBlocksCreate(); break;
        case 180: _t->slotBlocksExplode(); break;
        case 181: _t->slotModifyLineJoin(); break;
        case 182: _t->slotModifyDuplicate(); break;
        case 183: _t->slotDrawStar(); break;
        case 184: _t->slotModifyBreakDivide(); break;
        case 185: _t->slotModifyLineGap(); break;
        case 186: _t->slotOptionsDrawing(); break;
        case 187: _t->toggleVisibility((*reinterpret_cast< RS_Layer*(*)>(_a[1]))); break;
        case 188: _t->toggleLock((*reinterpret_cast< RS_Layer*(*)>(_a[1]))); break;
        case 189: _t->togglePrint((*reinterpret_cast< RS_Layer*(*)>(_a[1]))); break;
        case 190: _t->toggleConstruction((*reinterpret_cast< RS_Layer*(*)>(_a[1]))); break;
        case 191: _t->slotRedockWidgets(); break;
        case 192: _t->slotPenPick(); break;
        case 193: _t->slotPenPickResolved(); break;
        case 194: _t->slotPenApply(); break;
        case 195: _t->slotPenCopy(); break;
        case 196: _t->slotPenSyncFromLayer(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QG_ActionHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QG_ActionHandler.data,
    qt_meta_data_QG_ActionHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QG_ActionHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QG_ActionHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QG_ActionHandler.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QG_ActionHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 197)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 197;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 197)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 197;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
