/********************************************************************************
** Form generated from reading UI file 'LC_DlgParabola.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_DLGPARABOLA_H
#define UI_LC_DLGPARABOLA_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include "qg_layerbox.h"
#include "qg_widgetpen.h"

QT_BEGIN_NAMESPACE

class Ui_DlgParabola
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout1;
    QLabel *lLayer;
    QG_LayerBox *cbLayer;
    QG_WidgetPen *wPen;
    QGroupBox *buttonGroup8;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_2;
    QTableView *tvPoints;
    QSpacerItem *spacer58;
    QLabel *lPointLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgParabola)
    {
        if (DlgParabola->objectName().isEmpty())
            DlgParabola->setObjectName(QString::fromUtf8("DlgParabola"));
        DlgParabola->resize(406, 320);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgParabola->sizePolicy().hasHeightForWidth());
        DlgParabola->setSizePolicy(sizePolicy);
        DlgParabola->setMinimumSize(QSize(400, 320));
        DlgParabola->setModal(true);
        vboxLayout = new QVBoxLayout(DlgParabola);
        vboxLayout->setSpacing(0);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        lLayer = new QLabel(DlgParabola);
        lLayer->setObjectName(QString::fromUtf8("lLayer"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lLayer->sizePolicy().hasHeightForWidth());
        lLayer->setSizePolicy(sizePolicy1);
        lLayer->setWordWrap(false);

        hboxLayout1->addWidget(lLayer);

        cbLayer = new QG_LayerBox(DlgParabola);
        cbLayer->setObjectName(QString::fromUtf8("cbLayer"));

        hboxLayout1->addWidget(cbLayer);


        vboxLayout1->addLayout(hboxLayout1);

        wPen = new QG_WidgetPen(DlgParabola);
        wPen->setObjectName(QString::fromUtf8("wPen"));
        QSizePolicy sizePolicy2(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(wPen->sizePolicy().hasHeightForWidth());
        wPen->setSizePolicy(sizePolicy2);

        vboxLayout1->addWidget(wPen);


        hboxLayout->addLayout(vboxLayout1);

        buttonGroup8 = new QGroupBox(DlgParabola);
        buttonGroup8->setObjectName(QString::fromUtf8("buttonGroup8"));
        QSizePolicy sizePolicy3(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy3.setHorizontalStretch(1);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(buttonGroup8->sizePolicy().hasHeightForWidth());
        buttonGroup8->setSizePolicy(sizePolicy3);
        gridLayout = new QGridLayout(buttonGroup8);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));

        gridLayout->addLayout(horizontalLayout_2, 0, 0, 1, 2);

        tvPoints = new QTableView(buttonGroup8);
        tvPoints->setObjectName(QString::fromUtf8("tvPoints"));
        tvPoints->setMinimumSize(QSize(320, 240));
        tvPoints->setTextElideMode(Qt::ElideRight);
        tvPoints->horizontalHeader()->setCascadingSectionResizes(false);

        gridLayout->addWidget(tvPoints, 2, 0, 1, 2);

        spacer58 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(spacer58, 3, 0, 1, 1);

        lPointLabel = new QLabel(buttonGroup8);
        lPointLabel->setObjectName(QString::fromUtf8("lPointLabel"));

        gridLayout->addWidget(lPointLabel, 1, 0, 1, 1);


        hboxLayout->addWidget(buttonGroup8);


        vboxLayout->addLayout(hboxLayout);

        buttonBox = new QDialogButtonBox(DlgParabola);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(DlgParabola);
        QObject::connect(buttonBox, SIGNAL(accepted()), DlgParabola, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), DlgParabola, SLOT(reject()));

        QMetaObject::connectSlotsByName(DlgParabola);
    } // setupUi

    void retranslateUi(QDialog *DlgParabola)
    {
        DlgParabola->setWindowTitle(QCoreApplication::translate("DlgParabola", "Parabola", nullptr));
        lLayer->setText(QCoreApplication::translate("DlgParabola", "Layer:", nullptr));
        buttonGroup8->setTitle(QCoreApplication::translate("DlgParabola", "Geometry", nullptr));
#if QT_CONFIG(tooltip)
        tvPoints->setToolTip(QCoreApplication::translate("DlgParabola", "Points on Spline", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        tvPoints->setWhatsThis(QCoreApplication::translate("DlgParabola", "Points on Spline", nullptr));
#endif // QT_CONFIG(whatsthis)
        lPointLabel->setText(QCoreApplication::translate("DlgParabola", "Control Points", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgParabola: public Ui_DlgParabola {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_DLGPARABOLA_H
