/****************************************************************************
** Meta object code from reading C++ file 'qg_selectionwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../librecad/src/ui/forms/qg_selectionwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qg_selectionwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QG_SelectionWidget_t {
    QByteArrayData data[13];
    char stringdata0[117];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QG_SelectionWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QG_SelectionWidget_t qt_meta_stringdata_QG_SelectionWidget = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QG_SelectionWidget"
QT_MOC_LITERAL(1, 19, 9), // "setNumber"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 1), // "n"
QT_MOC_LITERAL(4, 32, 14), // "setTotalLength"
QT_MOC_LITERAL(5, 47, 1), // "l"
QT_MOC_LITERAL(6, 49, 12), // "flashAuxData"
QT_MOC_LITERAL(7, 62, 6), // "header"
QT_MOC_LITERAL(8, 69, 4), // "data"
QT_MOC_LITERAL(9, 74, 7), // "timeout"
QT_MOC_LITERAL(10, 82, 5), // "flash"
QT_MOC_LITERAL(11, 88, 13), // "removeAuxData"
QT_MOC_LITERAL(12, 102, 14) // "languageChange"

    },
    "QG_SelectionWidget\0setNumber\0\0n\0"
    "setTotalLength\0l\0flashAuxData\0header\0"
    "data\0timeout\0flash\0removeAuxData\0"
    "languageChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QG_SelectionWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x0a /* Public */,
       4,    1,   42,    2, 0x0a /* Public */,
       6,    4,   45,    2, 0x0a /* Public */,
      11,    0,   54,    2, 0x0a /* Public */,
      12,    0,   55,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Double,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::UInt, QMetaType::Bool,    7,    8,    9,   10,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QG_SelectionWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QG_SelectionWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setNumber((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setTotalLength((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 2: _t->flashAuxData((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const uint(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4]))); break;
        case 3: _t->removeAuxData(); break;
        case 4: _t->languageChange(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QG_SelectionWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_QG_SelectionWidget.data,
    qt_meta_data_QG_SelectionWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QG_SelectionWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QG_SelectionWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QG_SelectionWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::QG_SelectionWidget"))
        return static_cast< Ui::QG_SelectionWidget*>(this);
    return QWidget::qt_metacast(_clname);
}

int QG_SelectionWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
