/********************************************************************************
** Form generated from reading UI file 'qg_dlgmtext.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QG_DLGMTEXT_H
#define UI_QG_DLGMTEXT_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include "qg_fontbox.h"
#include "qg_layerbox.h"
#include "qg_widgetpen.h"

QT_BEGIN_NAMESPACE

class Ui_QG_DlgMText
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout1;
    QVBoxLayout *vlAttributes;
    QHBoxLayout *hboxLayout2;
    QLabel *lLayer;
    QG_LayerBox *cbLayer;
    QSpacerItem *horizontalSpacer_5;
    QG_WidgetPen *wPen;
    QGroupBox *bgFont;
    QGridLayout *gridLayout;
    QLabel *lLineSpacingFactor;
    QCheckBox *cbDefault;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *spacer2_2;
    QLineEdit *leHeight;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *spacer2;
    QSpacerItem *horizontalSpacer_3;
    QLabel *lHeight;
    QLineEdit *leLineSpacingFactor;
    QLabel *label;
    QG_FontBox *cbFont;
    QSpacerItem *horizontalSpacer_4;
    QHBoxLayout *hboxLayout3;
    QGroupBox *bgAlignment;
    QGridLayout *gridLayout1;
    QToolButton *bTR;
    QToolButton *bTL;
    QToolButton *bML;
    QToolButton *bMC;
    QToolButton *bMR;
    QToolButton *bBL;
    QToolButton *bBR;
    QToolButton *bBC;
    QToolButton *bTC;
    QGroupBox *bgShape;
    QVBoxLayout *vboxLayout2;
    QLineEdit *leAngle;
    QSpacerItem *spacer13;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout_3;
    QRadioButton *rbLeftToRight;
    QRadioButton *rbRightToLeft;
    QVBoxLayout *vboxLayout3;
    QHBoxLayout *hboxLayout4;
    QLabel *lText;
    QSpacerItem *spacer4;
    QToolButton *bClear;
    QToolButton *bLoad;
    QToolButton *bSave;
    QToolButton *bCut;
    QToolButton *bCopy;
    QToolButton *bPaste;
    QTextEdit *teText;
    QHBoxLayout *hboxLayout5;
    QGroupBox *buttonGroup7;
    QVBoxLayout *vboxLayout4;
    QComboBox *cbSymbol;
    QGroupBox *buttonGroup5;
    QGridLayout *gridLayout2;
    QLabel *textLabel1;
    QLabel *textLabel1_2;
    QComboBox *cbUniPage;
    QComboBox *cbUniChar;
    QToolButton *bUnicode;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *QG_DlgMText)
    {
        if (QG_DlgMText->objectName().isEmpty())
            QG_DlgMText->setObjectName(QString::fromUtf8("QG_DlgMText"));
        QG_DlgMText->resize(684, 500);
        QG_DlgMText->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(QG_DlgMText);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        vlAttributes = new QVBoxLayout();
        vlAttributes->setSpacing(6);
        vlAttributes->setObjectName(QString::fromUtf8("vlAttributes"));
        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        lLayer = new QLabel(QG_DlgMText);
        lLayer->setObjectName(QString::fromUtf8("lLayer"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lLayer->sizePolicy().hasHeightForWidth());
        lLayer->setSizePolicy(sizePolicy);
        lLayer->setMinimumSize(QSize(67, 0));
        lLayer->setWordWrap(false);

        hboxLayout2->addWidget(lLayer);

        cbLayer = new QG_LayerBox(QG_DlgMText);
        cbLayer->setObjectName(QString::fromUtf8("cbLayer"));
        cbLayer->setMinimumSize(QSize(70, 25));
        cbLayer->setFocusPolicy(Qt::StrongFocus);

        hboxLayout2->addWidget(cbLayer);

        horizontalSpacer_5 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(horizontalSpacer_5);


        vlAttributes->addLayout(hboxLayout2);

        wPen = new QG_WidgetPen(QG_DlgMText);
        wPen->setObjectName(QString::fromUtf8("wPen"));
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(wPen->sizePolicy().hasHeightForWidth());
        wPen->setSizePolicy(sizePolicy1);
        wPen->setFocusPolicy(Qt::NoFocus);

        vlAttributes->addWidget(wPen);


        hboxLayout1->addLayout(vlAttributes);

        bgFont = new QGroupBox(QG_DlgMText);
        bgFont->setObjectName(QString::fromUtf8("bgFont"));
        sizePolicy.setHeightForWidth(bgFont->sizePolicy().hasHeightForWidth());
        bgFont->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(bgFont);
        gridLayout->setSpacing(2);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        lLineSpacingFactor = new QLabel(bgFont);
        lLineSpacingFactor->setObjectName(QString::fromUtf8("lLineSpacingFactor"));
        lLineSpacingFactor->setWordWrap(false);

        gridLayout->addWidget(lLineSpacingFactor, 5, 0, 1, 1);

        cbDefault = new QCheckBox(bgFont);
        cbDefault->setObjectName(QString::fromUtf8("cbDefault"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(cbDefault->sizePolicy().hasHeightForWidth());
        cbDefault->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(cbDefault, 4, 0, 1, 2);

        horizontalSpacer_2 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 3, 2, 1, 1);

        spacer2_2 = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(spacer2_2, 7, 1, 1, 1);

        leHeight = new QLineEdit(bgFont);
        leHeight->setObjectName(QString::fromUtf8("leHeight"));
        sizePolicy2.setHeightForWidth(leHeight->sizePolicy().hasHeightForWidth());
        leHeight->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(leHeight, 3, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 5, 2, 1, 1);

        spacer2 = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(spacer2, 7, 0, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 4, 2, 1, 1);

        lHeight = new QLabel(bgFont);
        lHeight->setObjectName(QString::fromUtf8("lHeight"));
        lHeight->setWordWrap(false);

        gridLayout->addWidget(lHeight, 3, 0, 1, 1);

        leLineSpacingFactor = new QLineEdit(bgFont);
        leLineSpacingFactor->setObjectName(QString::fromUtf8("leLineSpacingFactor"));
        sizePolicy2.setHeightForWidth(leLineSpacingFactor->sizePolicy().hasHeightForWidth());
        leLineSpacingFactor->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(leLineSpacingFactor, 5, 1, 1, 1);

        label = new QLabel(bgFont);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 2, 0, 1, 1);

        cbFont = new QG_FontBox(bgFont);
        cbFont->setObjectName(QString::fromUtf8("cbFont"));
        sizePolicy2.setHeightForWidth(cbFont->sizePolicy().hasHeightForWidth());
        cbFont->setSizePolicy(sizePolicy2);
        cbFont->setMinimumSize(QSize(100, 25));
        cbFont->setFocusPolicy(Qt::WheelFocus);

        gridLayout->addWidget(cbFont, 2, 1, 1, 1);

        horizontalSpacer_4 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer_4, 2, 2, 1, 1);


        hboxLayout1->addWidget(bgFont);


        vboxLayout1->addLayout(hboxLayout1);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        bgAlignment = new QGroupBox(QG_DlgMText);
        bgAlignment->setObjectName(QString::fromUtf8("bgAlignment"));
        QSizePolicy sizePolicy3(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(bgAlignment->sizePolicy().hasHeightForWidth());
        bgAlignment->setSizePolicy(sizePolicy3);
        gridLayout1 = new QGridLayout(bgAlignment);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        bTR = new QToolButton(bgAlignment);
        bTR->setObjectName(QString::fromUtf8("bTR"));
        QSizePolicy sizePolicy4(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(bTR->sizePolicy().hasHeightForWidth());
        bTR->setSizePolicy(sizePolicy4);
        bTR->setMinimumSize(QSize(25, 25));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/mtext_align_topright.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bTR->setIcon(icon);
        bTR->setCheckable(true);
        bTR->setAutoExclusive(true);

        gridLayout1->addWidget(bTR, 0, 2, 1, 1);

        bTL = new QToolButton(bgAlignment);
        bTL->setObjectName(QString::fromUtf8("bTL"));
        sizePolicy4.setHeightForWidth(bTL->sizePolicy().hasHeightForWidth());
        bTL->setSizePolicy(sizePolicy4);
        bTL->setMinimumSize(QSize(25, 25));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/mtext_align_topleft.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bTL->setIcon(icon1);
        bTL->setCheckable(true);
        bTL->setAutoExclusive(true);

        gridLayout1->addWidget(bTL, 0, 0, 1, 1);

        bML = new QToolButton(bgAlignment);
        bML->setObjectName(QString::fromUtf8("bML"));
        sizePolicy4.setHeightForWidth(bML->sizePolicy().hasHeightForWidth());
        bML->setSizePolicy(sizePolicy4);
        bML->setMinimumSize(QSize(25, 25));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/mtext_align_middleleft.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bML->setIcon(icon2);
        bML->setCheckable(true);
        bML->setAutoExclusive(true);

        gridLayout1->addWidget(bML, 1, 0, 1, 1);

        bMC = new QToolButton(bgAlignment);
        bMC->setObjectName(QString::fromUtf8("bMC"));
        sizePolicy4.setHeightForWidth(bMC->sizePolicy().hasHeightForWidth());
        bMC->setSizePolicy(sizePolicy4);
        bMC->setMinimumSize(QSize(25, 25));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/mtext_align_middlecenter.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bMC->setIcon(icon3);
        bMC->setCheckable(true);
        bMC->setAutoExclusive(true);

        gridLayout1->addWidget(bMC, 1, 1, 1, 1);

        bMR = new QToolButton(bgAlignment);
        bMR->setObjectName(QString::fromUtf8("bMR"));
        sizePolicy4.setHeightForWidth(bMR->sizePolicy().hasHeightForWidth());
        bMR->setSizePolicy(sizePolicy4);
        bMR->setMinimumSize(QSize(25, 25));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/mtext_align_middleright.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bMR->setIcon(icon4);
        bMR->setCheckable(true);
        bMR->setAutoExclusive(true);

        gridLayout1->addWidget(bMR, 1, 2, 1, 1);

        bBL = new QToolButton(bgAlignment);
        bBL->setObjectName(QString::fromUtf8("bBL"));
        sizePolicy4.setHeightForWidth(bBL->sizePolicy().hasHeightForWidth());
        bBL->setSizePolicy(sizePolicy4);
        bBL->setMinimumSize(QSize(25, 25));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/mtext_align_bottomleft.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bBL->setIcon(icon5);
        bBL->setCheckable(true);
        bBL->setAutoExclusive(true);

        gridLayout1->addWidget(bBL, 2, 0, 1, 1);

        bBR = new QToolButton(bgAlignment);
        bBR->setObjectName(QString::fromUtf8("bBR"));
        sizePolicy4.setHeightForWidth(bBR->sizePolicy().hasHeightForWidth());
        bBR->setSizePolicy(sizePolicy4);
        bBR->setMinimumSize(QSize(25, 25));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/mtext_align_bottomright.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bBR->setIcon(icon6);
        bBR->setCheckable(true);
        bBR->setAutoExclusive(true);

        gridLayout1->addWidget(bBR, 2, 2, 1, 1);

        bBC = new QToolButton(bgAlignment);
        bBC->setObjectName(QString::fromUtf8("bBC"));
        sizePolicy4.setHeightForWidth(bBC->sizePolicy().hasHeightForWidth());
        bBC->setSizePolicy(sizePolicy4);
        bBC->setMinimumSize(QSize(25, 25));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/icons/mtext_align_bottomcenter.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bBC->setIcon(icon7);
        bBC->setCheckable(true);
        bBC->setAutoExclusive(true);

        gridLayout1->addWidget(bBC, 2, 1, 1, 1);

        bTC = new QToolButton(bgAlignment);
        bTC->setObjectName(QString::fromUtf8("bTC"));
        sizePolicy4.setHeightForWidth(bTC->sizePolicy().hasHeightForWidth());
        bTC->setSizePolicy(sizePolicy4);
        bTC->setMinimumSize(QSize(25, 25));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/icons/mtext_align_topcenter.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bTC->setIcon(icon8);
        bTC->setCheckable(true);
        bTC->setAutoExclusive(true);

        gridLayout1->addWidget(bTC, 0, 1, 1, 1);


        hboxLayout3->addWidget(bgAlignment);

        bgShape = new QGroupBox(QG_DlgMText);
        bgShape->setObjectName(QString::fromUtf8("bgShape"));
        QSizePolicy sizePolicy5(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(bgShape->sizePolicy().hasHeightForWidth());
        bgShape->setSizePolicy(sizePolicy5);
        bgShape->setMinimumSize(QSize(80, 0));
        vboxLayout2 = new QVBoxLayout(bgShape);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        leAngle = new QLineEdit(bgShape);
        leAngle->setObjectName(QString::fromUtf8("leAngle"));
        sizePolicy2.setHeightForWidth(leAngle->sizePolicy().hasHeightForWidth());
        leAngle->setSizePolicy(sizePolicy2);

        vboxLayout2->addWidget(leAngle);

        spacer13 = new QSpacerItem(20, 1, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout2->addItem(spacer13);

        groupBox = new QGroupBox(bgShape);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        QSizePolicy sizePolicy6(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy6);
        horizontalLayout_3 = new QHBoxLayout(groupBox);
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        rbLeftToRight = new QRadioButton(groupBox);
        rbLeftToRight->setObjectName(QString::fromUtf8("rbLeftToRight"));
        rbLeftToRight->setMinimumSize(QSize(50, 25));

        horizontalLayout_3->addWidget(rbLeftToRight);

        rbRightToLeft = new QRadioButton(groupBox);
        rbRightToLeft->setObjectName(QString::fromUtf8("rbRightToLeft"));
        rbRightToLeft->setMinimumSize(QSize(50, 25));

        horizontalLayout_3->addWidget(rbRightToLeft);


        vboxLayout2->addWidget(groupBox);


        hboxLayout3->addWidget(bgShape);


        vboxLayout1->addLayout(hboxLayout3);


        hboxLayout->addLayout(vboxLayout1);

        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(6);
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setSpacing(6);
        hboxLayout4->setObjectName(QString::fromUtf8("hboxLayout4"));
        lText = new QLabel(QG_DlgMText);
        lText->setObjectName(QString::fromUtf8("lText"));
        lText->setWordWrap(false);

        hboxLayout4->addWidget(lText);

        spacer4 = new QSpacerItem(2, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout4->addItem(spacer4);

        bClear = new QToolButton(QG_DlgMText);
        bClear->setObjectName(QString::fromUtf8("bClear"));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/icons/new.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bClear->setIcon(icon9);

        hboxLayout4->addWidget(bClear);

        bLoad = new QToolButton(QG_DlgMText);
        bLoad->setObjectName(QString::fromUtf8("bLoad"));
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/icons/open.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bLoad->setIcon(icon10);

        hboxLayout4->addWidget(bLoad);

        bSave = new QToolButton(QG_DlgMText);
        bSave->setObjectName(QString::fromUtf8("bSave"));
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/icons/save.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bSave->setIcon(icon11);

        hboxLayout4->addWidget(bSave);

        bCut = new QToolButton(QG_DlgMText);
        bCut->setObjectName(QString::fromUtf8("bCut"));
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/icons/cut.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bCut->setIcon(icon12);

        hboxLayout4->addWidget(bCut);

        bCopy = new QToolButton(QG_DlgMText);
        bCopy->setObjectName(QString::fromUtf8("bCopy"));
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/icons/copy.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bCopy->setIcon(icon13);

        hboxLayout4->addWidget(bCopy);

        bPaste = new QToolButton(QG_DlgMText);
        bPaste->setObjectName(QString::fromUtf8("bPaste"));
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/icons/paste.svg"), QSize(), QIcon::Normal, QIcon::Off);
        bPaste->setIcon(icon14);

        hboxLayout4->addWidget(bPaste);


        vboxLayout3->addLayout(hboxLayout4);

        teText = new QTextEdit(QG_DlgMText);
        teText->setObjectName(QString::fromUtf8("teText"));
        teText->setFocusPolicy(Qt::StrongFocus);
        teText->setTabChangesFocus(true);
        teText->setProperty("linkUnderline", QVariant(false));

        vboxLayout3->addWidget(teText);


        hboxLayout->addLayout(vboxLayout3);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout5 = new QHBoxLayout();
        hboxLayout5->setSpacing(6);
        hboxLayout5->setObjectName(QString::fromUtf8("hboxLayout5"));
        buttonGroup7 = new QGroupBox(QG_DlgMText);
        buttonGroup7->setObjectName(QString::fromUtf8("buttonGroup7"));
        sizePolicy.setHeightForWidth(buttonGroup7->sizePolicy().hasHeightForWidth());
        buttonGroup7->setSizePolicy(sizePolicy);
        buttonGroup7->setMinimumSize(QSize(148, 0));
        vboxLayout4 = new QVBoxLayout(buttonGroup7);
        vboxLayout4->setSpacing(6);
        vboxLayout4->setContentsMargins(11, 11, 11, 11);
        vboxLayout4->setObjectName(QString::fromUtf8("vboxLayout4"));
        cbSymbol = new QComboBox(buttonGroup7);
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->addItem(QString());
        cbSymbol->setObjectName(QString::fromUtf8("cbSymbol"));
        QSizePolicy sizePolicy7(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(cbSymbol->sizePolicy().hasHeightForWidth());
        cbSymbol->setSizePolicy(sizePolicy7);

        vboxLayout4->addWidget(cbSymbol);


        hboxLayout5->addWidget(buttonGroup7);

        buttonGroup5 = new QGroupBox(QG_DlgMText);
        buttonGroup5->setObjectName(QString::fromUtf8("buttonGroup5"));
        gridLayout2 = new QGridLayout(buttonGroup5);
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(11, 11, 11, 11);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        textLabel1 = new QLabel(buttonGroup5);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setWordWrap(false);

        gridLayout2->addWidget(textLabel1, 0, 0, 1, 1);

        textLabel1_2 = new QLabel(buttonGroup5);
        textLabel1_2->setObjectName(QString::fromUtf8("textLabel1_2"));
        textLabel1_2->setWordWrap(false);

        gridLayout2->addWidget(textLabel1_2, 1, 0, 1, 1);

        cbUniPage = new QComboBox(buttonGroup5);
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->addItem(QString());
        cbUniPage->setObjectName(QString::fromUtf8("cbUniPage"));
        sizePolicy7.setHeightForWidth(cbUniPage->sizePolicy().hasHeightForWidth());
        cbUniPage->setSizePolicy(sizePolicy7);
        cbUniPage->setMinimumSize(QSize(200, 0));

        gridLayout2->addWidget(cbUniPage, 0, 1, 1, 1);

        cbUniChar = new QComboBox(buttonGroup5);
        cbUniChar->setObjectName(QString::fromUtf8("cbUniChar"));
        sizePolicy7.setHeightForWidth(cbUniChar->sizePolicy().hasHeightForWidth());
        cbUniChar->setSizePolicy(sizePolicy7);
        cbUniChar->setMinimumSize(QSize(25, 0));

        gridLayout2->addWidget(cbUniChar, 1, 1, 1, 1);

        bUnicode = new QToolButton(buttonGroup5);
        bUnicode->setObjectName(QString::fromUtf8("bUnicode"));
        sizePolicy4.setHeightForWidth(bUnicode->sizePolicy().hasHeightForWidth());
        bUnicode->setSizePolicy(sizePolicy4);
        bUnicode->setMinimumSize(QSize(25, 25));
        bUnicode->setMaximumSize(QSize(25, 25));

        gridLayout2->addWidget(bUnicode, 0, 2, 1, 1);


        hboxLayout5->addWidget(buttonGroup5);


        vboxLayout->addLayout(hboxLayout5);

        buttonBox = new QDialogButtonBox(QG_DlgMText);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setFocusPolicy(Qt::StrongFocus);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        lLineSpacingFactor->setBuddy(leLineSpacingFactor);
        lHeight->setBuddy(leHeight);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(cbLayer, cbFont);
        QWidget::setTabOrder(cbFont, leHeight);
        QWidget::setTabOrder(leHeight, cbDefault);
        QWidget::setTabOrder(cbDefault, leLineSpacingFactor);
        QWidget::setTabOrder(leLineSpacingFactor, teText);
        QWidget::setTabOrder(teText, bTL);
        QWidget::setTabOrder(bTL, bTC);
        QWidget::setTabOrder(bTC, bTR);
        QWidget::setTabOrder(bTR, bML);
        QWidget::setTabOrder(bML, bMC);
        QWidget::setTabOrder(bMC, bMR);
        QWidget::setTabOrder(bMR, bBL);
        QWidget::setTabOrder(bBL, bBC);
        QWidget::setTabOrder(bBC, bBR);
        QWidget::setTabOrder(bBR, leAngle);
        QWidget::setTabOrder(leAngle, cbSymbol);
        QWidget::setTabOrder(cbSymbol, cbUniPage);
        QWidget::setTabOrder(cbUniPage, cbUniChar);
        QWidget::setTabOrder(cbUniChar, bUnicode);
        QWidget::setTabOrder(bUnicode, buttonBox);
        QWidget::setTabOrder(buttonBox, bClear);
        QWidget::setTabOrder(bClear, bLoad);
        QWidget::setTabOrder(bLoad, bSave);
        QWidget::setTabOrder(bSave, bCut);
        QWidget::setTabOrder(bCut, bCopy);
        QWidget::setTabOrder(bCopy, bPaste);

        retranslateUi(QG_DlgMText);
        QObject::connect(buttonBox, SIGNAL(accepted()), QG_DlgMText, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), QG_DlgMText, SLOT(reject()));
        QObject::connect(bTL, SIGNAL(clicked()), QG_DlgMText, SLOT(setAlignmentTL()));
        QObject::connect(bTC, SIGNAL(clicked()), QG_DlgMText, SLOT(setAlignmentTC()));
        QObject::connect(bTR, SIGNAL(clicked()), QG_DlgMText, SLOT(setAlignmentTR()));
        QObject::connect(bML, SIGNAL(clicked()), QG_DlgMText, SLOT(setAlignmentML()));
        QObject::connect(bMC, SIGNAL(clicked()), QG_DlgMText, SLOT(setAlignmentMC()));
        QObject::connect(bMR, SIGNAL(clicked()), QG_DlgMText, SLOT(setAlignmentMR()));
        QObject::connect(bBL, SIGNAL(clicked()), QG_DlgMText, SLOT(setAlignmentBL()));
        QObject::connect(bBC, SIGNAL(clicked()), QG_DlgMText, SLOT(setAlignmentBC()));
        QObject::connect(bBR, SIGNAL(clicked()), QG_DlgMText, SLOT(setAlignmentBR()));
        QObject::connect(cbDefault, SIGNAL(toggled(bool)), leLineSpacingFactor, SLOT(setDisabled(bool)));
        QObject::connect(cbDefault, SIGNAL(toggled(bool)), QG_DlgMText, SLOT(defaultChanged(bool)));
        QObject::connect(bClear, SIGNAL(clicked()), teText, SLOT(clear()));
        QObject::connect(bCut, SIGNAL(clicked()), teText, SLOT(cut()));
        QObject::connect(bCopy, SIGNAL(clicked()), teText, SLOT(copy()));
        QObject::connect(bPaste, SIGNAL(clicked()), teText, SLOT(paste()));
        QObject::connect(bLoad, SIGNAL(clicked()), QG_DlgMText, SLOT(loadText()));
        QObject::connect(bSave, SIGNAL(clicked()), QG_DlgMText, SLOT(saveText()));
        QObject::connect(cbUniPage, SIGNAL(activated(int)), QG_DlgMText, SLOT(updateUniCharComboBox(int)));
        QObject::connect(bUnicode, SIGNAL(clicked()), QG_DlgMText, SLOT(insertChar()));
        QObject::connect(cbUniPage, SIGNAL(activated(int)), QG_DlgMText, SLOT(updateUniCharButton(int)));
        QObject::connect(cbUniChar, SIGNAL(activated(int)), QG_DlgMText, SLOT(updateUniCharButton(int)));
        QObject::connect(cbSymbol, SIGNAL(activated(int)), QG_DlgMText, SLOT(insertSymbol(int)));

        QMetaObject::connectSlotsByName(QG_DlgMText);
    } // setupUi

    void retranslateUi(QDialog *QG_DlgMText)
    {
        QG_DlgMText->setWindowTitle(QCoreApplication::translate("QG_DlgMText", "MText", nullptr));
        lLayer->setText(QCoreApplication::translate("QG_DlgMText", "Layer:", nullptr));
        bgFont->setTitle(QCoreApplication::translate("QG_DlgMText", "Font Settings", nullptr));
        lLineSpacingFactor->setText(QCoreApplication::translate("QG_DlgMText", "&Line spacing:", nullptr));
        cbDefault->setText(QCoreApplication::translate("QG_DlgMText", "&Default line spacing", nullptr));
#if QT_CONFIG(shortcut)
        cbDefault->setShortcut(QCoreApplication::translate("QG_DlgMText", "Alt+D", nullptr));
#endif // QT_CONFIG(shortcut)
        lHeight->setText(QCoreApplication::translate("QG_DlgMText", "&Height:", nullptr));
        label->setText(QCoreApplication::translate("QG_DlgMText", "Font:", nullptr));
        bgAlignment->setTitle(QCoreApplication::translate("QG_DlgMText", "Alignment", nullptr));
#if QT_CONFIG(tooltip)
        bTR->setToolTip(QCoreApplication::translate("QG_DlgMText", "Top Right", nullptr));
#endif // QT_CONFIG(tooltip)
        bTR->setText(QString());
#if QT_CONFIG(tooltip)
        bTL->setToolTip(QCoreApplication::translate("QG_DlgMText", "Top Left", nullptr));
#endif // QT_CONFIG(tooltip)
        bTL->setText(QString());
#if QT_CONFIG(tooltip)
        bML->setToolTip(QCoreApplication::translate("QG_DlgMText", "Middle Left", nullptr));
#endif // QT_CONFIG(tooltip)
        bML->setText(QString());
#if QT_CONFIG(tooltip)
        bMC->setToolTip(QCoreApplication::translate("QG_DlgMText", "Middle Center", nullptr));
#endif // QT_CONFIG(tooltip)
        bMC->setText(QString());
#if QT_CONFIG(tooltip)
        bMR->setToolTip(QCoreApplication::translate("QG_DlgMText", "Middle Right", nullptr));
#endif // QT_CONFIG(tooltip)
        bMR->setText(QString());
#if QT_CONFIG(tooltip)
        bBL->setToolTip(QCoreApplication::translate("QG_DlgMText", "Bottom Left", nullptr));
#endif // QT_CONFIG(tooltip)
        bBL->setText(QString());
#if QT_CONFIG(tooltip)
        bBR->setToolTip(QCoreApplication::translate("QG_DlgMText", "Bottom Right", nullptr));
#endif // QT_CONFIG(tooltip)
        bBR->setText(QString());
#if QT_CONFIG(tooltip)
        bBC->setToolTip(QCoreApplication::translate("QG_DlgMText", "Bottom Center", nullptr));
#endif // QT_CONFIG(tooltip)
        bBC->setText(QString());
#if QT_CONFIG(tooltip)
        bTC->setToolTip(QCoreApplication::translate("QG_DlgMText", "Top Center", nullptr));
#endif // QT_CONFIG(tooltip)
        bTC->setText(QString());
        bgShape->setTitle(QCoreApplication::translate("QG_DlgMText", "Angle", nullptr));
        groupBox->setTitle(QCoreApplication::translate("QG_DlgMText", "Text Direction", nullptr));
        rbLeftToRight->setText(QCoreApplication::translate("QG_DlgMText", "Left to right", nullptr));
        rbRightToLeft->setText(QCoreApplication::translate("QG_DlgMText", "Right to left", nullptr));
        lText->setText(QCoreApplication::translate("QG_DlgMText", "Text:", nullptr));
#if QT_CONFIG(tooltip)
        bClear->setToolTip(QCoreApplication::translate("QG_DlgMText", "Clear Text", nullptr));
#endif // QT_CONFIG(tooltip)
        bClear->setText(QString());
#if QT_CONFIG(tooltip)
        bLoad->setToolTip(QCoreApplication::translate("QG_DlgMText", "Load Text From File", nullptr));
#endif // QT_CONFIG(tooltip)
        bLoad->setText(QString());
#if QT_CONFIG(tooltip)
        bSave->setToolTip(QCoreApplication::translate("QG_DlgMText", "Save Text To File", nullptr));
#endif // QT_CONFIG(tooltip)
        bSave->setText(QString());
#if QT_CONFIG(tooltip)
        bCut->setToolTip(QCoreApplication::translate("QG_DlgMText", "Cut", nullptr));
#endif // QT_CONFIG(tooltip)
        bCut->setText(QString());
#if QT_CONFIG(tooltip)
        bCopy->setToolTip(QCoreApplication::translate("QG_DlgMText", "Copy", nullptr));
#endif // QT_CONFIG(tooltip)
        bCopy->setText(QString());
#if QT_CONFIG(tooltip)
        bPaste->setToolTip(QCoreApplication::translate("QG_DlgMText", "Paste", nullptr));
#endif // QT_CONFIG(tooltip)
        bPaste->setText(QString());
        buttonGroup7->setTitle(QCoreApplication::translate("QG_DlgMText", "Insert Symbol", nullptr));
        cbSymbol->setItemText(0, QCoreApplication::translate("QG_DlgMText", "Diameter (\342\214\200)", nullptr));
        cbSymbol->setItemText(1, QCoreApplication::translate("QG_DlgMText", "Degree (\302\260)", nullptr));
        cbSymbol->setItemText(2, QCoreApplication::translate("QG_DlgMText", "Plus / Minus (\302\261)", nullptr));
        cbSymbol->setItemText(3, QCoreApplication::translate("QG_DlgMText", "At (@)", nullptr));
        cbSymbol->setItemText(4, QCoreApplication::translate("QG_DlgMText", "Hash (#)", nullptr));
        cbSymbol->setItemText(5, QCoreApplication::translate("QG_DlgMText", "Dollar ($)", nullptr));
        cbSymbol->setItemText(6, QCoreApplication::translate("QG_DlgMText", "Copyright (\302\251)", nullptr));
        cbSymbol->setItemText(7, QCoreApplication::translate("QG_DlgMText", "Registered (\302\256)", nullptr));
        cbSymbol->setItemText(8, QCoreApplication::translate("QG_DlgMText", "Paragraph (\302\247)", nullptr));
        cbSymbol->setItemText(9, QCoreApplication::translate("QG_DlgMText", "Pi (\317\200)", nullptr));
        cbSymbol->setItemText(10, QCoreApplication::translate("QG_DlgMText", "Pound (\302\243)", nullptr));
        cbSymbol->setItemText(11, QCoreApplication::translate("QG_DlgMText", "Yen (\302\245)", nullptr));
        cbSymbol->setItemText(12, QCoreApplication::translate("QG_DlgMText", "Times (\303\227)", nullptr));
        cbSymbol->setItemText(13, QCoreApplication::translate("QG_DlgMText", "Division (\303\267)", nullptr));

        buttonGroup5->setTitle(QCoreApplication::translate("QG_DlgMText", "Insert Unicode", nullptr));
        textLabel1->setText(QCoreApplication::translate("QG_DlgMText", "Page:", nullptr));
        textLabel1_2->setText(QCoreApplication::translate("QG_DlgMText", "Char:", nullptr));
        cbUniPage->setItemText(0, QCoreApplication::translate("QG_DlgMText", "[0000-007F] Basic Latin", nullptr));
        cbUniPage->setItemText(1, QCoreApplication::translate("QG_DlgMText", "[0080-00FF] Latin-1 Supplementary", nullptr));
        cbUniPage->setItemText(2, QCoreApplication::translate("QG_DlgMText", "[0100-017F] Latin Extended-A", nullptr));
        cbUniPage->setItemText(3, QCoreApplication::translate("QG_DlgMText", "[0180-024F] Latin Extended-B", nullptr));
        cbUniPage->setItemText(4, QCoreApplication::translate("QG_DlgMText", "[0250-02AF] IPA Extensions", nullptr));
        cbUniPage->setItemText(5, QCoreApplication::translate("QG_DlgMText", "[02B0-02FF] Spacing Modifier Letters", nullptr));
        cbUniPage->setItemText(6, QCoreApplication::translate("QG_DlgMText", "[0300-036F] Combining Diacritical Marks", nullptr));
        cbUniPage->setItemText(7, QCoreApplication::translate("QG_DlgMText", "[0370-03FF] Greek and Coptic", nullptr));
        cbUniPage->setItemText(8, QCoreApplication::translate("QG_DlgMText", "[0400-04FF] Cyrillic", nullptr));
        cbUniPage->setItemText(9, QCoreApplication::translate("QG_DlgMText", "[0500-052F] Cyrillic Supplementary", nullptr));
        cbUniPage->setItemText(10, QCoreApplication::translate("QG_DlgMText", "[0530-058F] Armenian", nullptr));
        cbUniPage->setItemText(11, QCoreApplication::translate("QG_DlgMText", "[0590-05FF] Hebrew", nullptr));
        cbUniPage->setItemText(12, QCoreApplication::translate("QG_DlgMText", "[0600-06FF] Arabic", nullptr));
        cbUniPage->setItemText(13, QCoreApplication::translate("QG_DlgMText", "[0700-074F] Syriac", nullptr));
        cbUniPage->setItemText(14, QCoreApplication::translate("QG_DlgMText", "[0780-07BF] Thaana", nullptr));
        cbUniPage->setItemText(15, QCoreApplication::translate("QG_DlgMText", "[0900-097F] Devanagari", nullptr));
        cbUniPage->setItemText(16, QCoreApplication::translate("QG_DlgMText", "[0980-09FF] Bengali", nullptr));
        cbUniPage->setItemText(17, QCoreApplication::translate("QG_DlgMText", "[0A00-0A7F] Gurmukhi", nullptr));
        cbUniPage->setItemText(18, QCoreApplication::translate("QG_DlgMText", "[0A80-0AFF] Gujarati", nullptr));
        cbUniPage->setItemText(19, QCoreApplication::translate("QG_DlgMText", "[0B00-0B7F] Oriya", nullptr));
        cbUniPage->setItemText(20, QCoreApplication::translate("QG_DlgMText", "[0B80-0BFF] Tamil", nullptr));
        cbUniPage->setItemText(21, QCoreApplication::translate("QG_DlgMText", "[0C00-0C7F] Telugu", nullptr));
        cbUniPage->setItemText(22, QCoreApplication::translate("QG_DlgMText", "[0C80-0CFF] Kannada", nullptr));
        cbUniPage->setItemText(23, QCoreApplication::translate("QG_DlgMText", "[0D00-0D7F] Malayalam", nullptr));
        cbUniPage->setItemText(24, QCoreApplication::translate("QG_DlgMText", "[0D80-0DFF] Sinhala", nullptr));
        cbUniPage->setItemText(25, QCoreApplication::translate("QG_DlgMText", "[0E00-0E7F] Thai", nullptr));
        cbUniPage->setItemText(26, QCoreApplication::translate("QG_DlgMText", "[0E80-0EFF] Lao", nullptr));
        cbUniPage->setItemText(27, QCoreApplication::translate("QG_DlgMText", "[0F00-0FFF] Tibetan", nullptr));
        cbUniPage->setItemText(28, QCoreApplication::translate("QG_DlgMText", "[1000-109F] Myanmar", nullptr));
        cbUniPage->setItemText(29, QCoreApplication::translate("QG_DlgMText", "[10A0-10FF] Georgian", nullptr));
        cbUniPage->setItemText(30, QCoreApplication::translate("QG_DlgMText", "[1100-11FF] Hangul Jamo", nullptr));
        cbUniPage->setItemText(31, QCoreApplication::translate("QG_DlgMText", "[1200-137F] Ethiopic", nullptr));
        cbUniPage->setItemText(32, QCoreApplication::translate("QG_DlgMText", "[13A0-13FF] Cherokee", nullptr));
        cbUniPage->setItemText(33, QCoreApplication::translate("QG_DlgMText", "[1400-167F] Unified Canadian Aboriginal Syllabic", nullptr));
        cbUniPage->setItemText(34, QCoreApplication::translate("QG_DlgMText", "[1680-169F] Ogham", nullptr));
        cbUniPage->setItemText(35, QCoreApplication::translate("QG_DlgMText", "[16A0-16FF] Runic", nullptr));
        cbUniPage->setItemText(36, QCoreApplication::translate("QG_DlgMText", "[1700-171F] Tagalog", nullptr));
        cbUniPage->setItemText(37, QCoreApplication::translate("QG_DlgMText", "[1720-173F] Hanunoo", nullptr));
        cbUniPage->setItemText(38, QCoreApplication::translate("QG_DlgMText", "[1740-175F] Buhid", nullptr));
        cbUniPage->setItemText(39, QCoreApplication::translate("QG_DlgMText", "[1760-177F] Tagbanwa", nullptr));
        cbUniPage->setItemText(40, QCoreApplication::translate("QG_DlgMText", "[1780-17FF] Khmer", nullptr));
        cbUniPage->setItemText(41, QCoreApplication::translate("QG_DlgMText", "[1800-18AF] Mongolian", nullptr));
        cbUniPage->setItemText(42, QCoreApplication::translate("QG_DlgMText", "[1E00-1EFF] Latin Extended Additional", nullptr));
        cbUniPage->setItemText(43, QCoreApplication::translate("QG_DlgMText", "[1F00-1FFF] Greek Extended", nullptr));
        cbUniPage->setItemText(44, QCoreApplication::translate("QG_DlgMText", "[2000-206F] General Punctuation", nullptr));
        cbUniPage->setItemText(45, QCoreApplication::translate("QG_DlgMText", "[2070-209F] Superscripts and Subscripts", nullptr));
        cbUniPage->setItemText(46, QCoreApplication::translate("QG_DlgMText", "[20A0-20CF] Currency Symbols", nullptr));
        cbUniPage->setItemText(47, QCoreApplication::translate("QG_DlgMText", "[20D0-20FF] Combining Marks for Symbols", nullptr));
        cbUniPage->setItemText(48, QCoreApplication::translate("QG_DlgMText", "[2100-214F] Letterlike Symbols", nullptr));
        cbUniPage->setItemText(49, QCoreApplication::translate("QG_DlgMText", "[2150-218F] Number Forms", nullptr));
        cbUniPage->setItemText(50, QCoreApplication::translate("QG_DlgMText", "[2190-21FF] Arrows", nullptr));
        cbUniPage->setItemText(51, QCoreApplication::translate("QG_DlgMText", "[2200-22FF] Mathematical Operators", nullptr));
        cbUniPage->setItemText(52, QCoreApplication::translate("QG_DlgMText", "[2300-23FF] Miscellaneous Technical", nullptr));
        cbUniPage->setItemText(53, QCoreApplication::translate("QG_DlgMText", "[2400-243F] Control Pictures", nullptr));
        cbUniPage->setItemText(54, QCoreApplication::translate("QG_DlgMText", "[2440-245F] Optical Character Recognition", nullptr));
        cbUniPage->setItemText(55, QCoreApplication::translate("QG_DlgMText", "[2460-24FF] Enclosed Alphanumerics", nullptr));
        cbUniPage->setItemText(56, QCoreApplication::translate("QG_DlgMText", "[2500-257F] Box Drawing", nullptr));
        cbUniPage->setItemText(57, QCoreApplication::translate("QG_DlgMText", "[2580-259F] Block Elements", nullptr));
        cbUniPage->setItemText(58, QCoreApplication::translate("QG_DlgMText", "[25A0-25FF] Geometric Shapes", nullptr));
        cbUniPage->setItemText(59, QCoreApplication::translate("QG_DlgMText", "[2600-26FF] Miscellaneous Symbols", nullptr));
        cbUniPage->setItemText(60, QCoreApplication::translate("QG_DlgMText", "[2700-27BF] Dingbats", nullptr));
        cbUniPage->setItemText(61, QCoreApplication::translate("QG_DlgMText", "[27C0-27EF] Miscellaneous Mathematical Symbols-A", nullptr));
        cbUniPage->setItemText(62, QCoreApplication::translate("QG_DlgMText", "[27F0-27FF] Supplemental Arrows-A", nullptr));
        cbUniPage->setItemText(63, QCoreApplication::translate("QG_DlgMText", "[2800-28FF] Braille Patterns", nullptr));
        cbUniPage->setItemText(64, QCoreApplication::translate("QG_DlgMText", "[2900-297F] Supplemental Arrows-B", nullptr));
        cbUniPage->setItemText(65, QCoreApplication::translate("QG_DlgMText", "[2980-29FF] Miscellaneous Mathematical Symbols-B", nullptr));
        cbUniPage->setItemText(66, QCoreApplication::translate("QG_DlgMText", "[2A00-2AFF] Supplemental Mathematical Operators", nullptr));
        cbUniPage->setItemText(67, QCoreApplication::translate("QG_DlgMText", "[2E80-2EFF] CJK Radicals Supplement", nullptr));
        cbUniPage->setItemText(68, QCoreApplication::translate("QG_DlgMText", "[2F00-2FDF] Kangxi Radicals", nullptr));
        cbUniPage->setItemText(69, QCoreApplication::translate("QG_DlgMText", "[2FF0-2FFF] Ideographic Description Characters", nullptr));
        cbUniPage->setItemText(70, QCoreApplication::translate("QG_DlgMText", "[3000-303F] CJK Symbols and Punctuation", nullptr));
        cbUniPage->setItemText(71, QCoreApplication::translate("QG_DlgMText", "[3040-309F] Hiragana", nullptr));
        cbUniPage->setItemText(72, QCoreApplication::translate("QG_DlgMText", "[30A0-30FF] Katakana", nullptr));
        cbUniPage->setItemText(73, QCoreApplication::translate("QG_DlgMText", "[3100-312F] Bopomofo", nullptr));
        cbUniPage->setItemText(74, QCoreApplication::translate("QG_DlgMText", "[3130-318F] Hangul Compatibility Jamo", nullptr));
        cbUniPage->setItemText(75, QCoreApplication::translate("QG_DlgMText", "[3190-319F] Kanbun", nullptr));
        cbUniPage->setItemText(76, QCoreApplication::translate("QG_DlgMText", "[31A0-31BF] Bopomofo Extended", nullptr));
        cbUniPage->setItemText(77, QCoreApplication::translate("QG_DlgMText", "[3200-32FF] Enclosed CJK Letters and Months", nullptr));
        cbUniPage->setItemText(78, QCoreApplication::translate("QG_DlgMText", "[3300-33FF] CJK Compatibility", nullptr));
        cbUniPage->setItemText(79, QCoreApplication::translate("QG_DlgMText", "[3400-4DBF] CJK Unified Ideographs Extension A", nullptr));
        cbUniPage->setItemText(80, QCoreApplication::translate("QG_DlgMText", "[4E00-9FAF] CJK Unified Ideographs", nullptr));
        cbUniPage->setItemText(81, QCoreApplication::translate("QG_DlgMText", "[A000-A48F] Yi Syllables", nullptr));
        cbUniPage->setItemText(82, QCoreApplication::translate("QG_DlgMText", "[A490-A4CF] Yi Radicals", nullptr));
        cbUniPage->setItemText(83, QCoreApplication::translate("QG_DlgMText", "[AC00-D7AF] Hangul Syllables", nullptr));
        cbUniPage->setItemText(84, QCoreApplication::translate("QG_DlgMText", "[D800-DBFF] High Surrogates", nullptr));
        cbUniPage->setItemText(85, QCoreApplication::translate("QG_DlgMText", "[DC00-DFFF] Low Surrogate Area", nullptr));
        cbUniPage->setItemText(86, QCoreApplication::translate("QG_DlgMText", "[E000-F8FF] Private Use Area", nullptr));
        cbUniPage->setItemText(87, QCoreApplication::translate("QG_DlgMText", "[F900-FAFF] CJK Compatibility Ideographs", nullptr));
        cbUniPage->setItemText(88, QCoreApplication::translate("QG_DlgMText", "[FB00-FB4F] Alphabetic Presentation Forms", nullptr));
        cbUniPage->setItemText(89, QCoreApplication::translate("QG_DlgMText", "[FB50-FDFF] Arabic Presentation Forms-A", nullptr));
        cbUniPage->setItemText(90, QCoreApplication::translate("QG_DlgMText", "[FE00-FE0F] Variation Selectors", nullptr));
        cbUniPage->setItemText(91, QCoreApplication::translate("QG_DlgMText", "[FE20-FE2F] Combining Half Marks", nullptr));
        cbUniPage->setItemText(92, QCoreApplication::translate("QG_DlgMText", "[FE30-FE4F] CJK Compatibility Forms", nullptr));
        cbUniPage->setItemText(93, QCoreApplication::translate("QG_DlgMText", "[FE50-FE6F] Small Form Variants", nullptr));
        cbUniPage->setItemText(94, QCoreApplication::translate("QG_DlgMText", "[FE70-FEFF] Arabic Presentation Forms-B", nullptr));
        cbUniPage->setItemText(95, QCoreApplication::translate("QG_DlgMText", "[FF00-FFEF] Halfwidth and Fullwidth Forms", nullptr));
        cbUniPage->setItemText(96, QCoreApplication::translate("QG_DlgMText", "[FFF0-FFFF] Specials", nullptr));
        cbUniPage->setItemText(97, QCoreApplication::translate("QG_DlgMText", "[10300-1032F] Old Italic", nullptr));
        cbUniPage->setItemText(98, QCoreApplication::translate("QG_DlgMText", "[10330-1034F] Gothic", nullptr));
        cbUniPage->setItemText(99, QCoreApplication::translate("QG_DlgMText", "[10400-1044F] Deseret", nullptr));
        cbUniPage->setItemText(100, QCoreApplication::translate("QG_DlgMText", "[1D000-1D0FF] Byzantine Musical Symbols", nullptr));
        cbUniPage->setItemText(101, QCoreApplication::translate("QG_DlgMText", "[1D100-1D1FF] Musical Symbols", nullptr));
        cbUniPage->setItemText(102, QCoreApplication::translate("QG_DlgMText", "[1D400-1D7FF] Mathematical Alphanumeric Symbols", nullptr));
        cbUniPage->setItemText(103, QCoreApplication::translate("QG_DlgMText", "[20000-2A6DF] CJK Unified Ideographs Extension B", nullptr));
        cbUniPage->setItemText(104, QCoreApplication::translate("QG_DlgMText", "[2F800-2FA1F] CJK Compatibility Ideographs Supplement", nullptr));
        cbUniPage->setItemText(105, QCoreApplication::translate("QG_DlgMText", "[E0000-E007F] Tags", nullptr));
        cbUniPage->setItemText(106, QCoreApplication::translate("QG_DlgMText", "[F0000-FFFFD] Supplementary Private Use Area-A", nullptr));
        cbUniPage->setItemText(107, QCoreApplication::translate("QG_DlgMText", "[100000-10FFFD] Supplementary Private Use Area-B", nullptr));

        bUnicode->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class QG_DlgMText: public Ui_QG_DlgMText {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QG_DLGMTEXT_H
