/********************************************************************************
** Form generated from reading UI file 'lc_staroptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_STAROPTIONS_H
#define UI_LC_STAROPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_StarOptions
{
public:
    QGridLayout *gridLayout;
    QCheckBox *cbRadiusInner;
    QLineEdit *leRadusOuter;
    QCheckBox *cbPolyline;
    QCheckBox *cbRadiusOuter;
    QSpinBox *sbNumber;
    QLineEdit *leRadiusInner;
    QCheckBox *cbSymmertix;
    QLabel *label;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *LC_StarOptions)
    {
        if (LC_StarOptions->objectName().isEmpty())
            LC_StarOptions->setObjectName(QString::fromUtf8("LC_StarOptions"));
        LC_StarOptions->resize(560, 25);
        gridLayout = new QGridLayout(LC_StarOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        cbRadiusInner = new QCheckBox(LC_StarOptions);
        cbRadiusInner->setObjectName(QString::fromUtf8("cbRadiusInner"));

        gridLayout->addWidget(cbRadiusInner, 0, 6, 1, 1);

        leRadusOuter = new QLineEdit(LC_StarOptions);
        leRadusOuter->setObjectName(QString::fromUtf8("leRadusOuter"));
        leRadusOuter->setMinimumSize(QSize(30, 0));
        leRadusOuter->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leRadusOuter, 0, 5, 1, 1);

        cbPolyline = new QCheckBox(LC_StarOptions);
        cbPolyline->setObjectName(QString::fromUtf8("cbPolyline"));

        gridLayout->addWidget(cbPolyline, 0, 8, 1, 1);

        cbRadiusOuter = new QCheckBox(LC_StarOptions);
        cbRadiusOuter->setObjectName(QString::fromUtf8("cbRadiusOuter"));

        gridLayout->addWidget(cbRadiusOuter, 0, 4, 1, 1);

        sbNumber = new QSpinBox(LC_StarOptions);
        sbNumber->setObjectName(QString::fromUtf8("sbNumber"));
        sbNumber->setMinimum(3);

        gridLayout->addWidget(sbNumber, 0, 2, 1, 1);

        leRadiusInner = new QLineEdit(LC_StarOptions);
        leRadiusInner->setObjectName(QString::fromUtf8("leRadiusInner"));
        leRadiusInner->setMinimumSize(QSize(30, 0));
        leRadiusInner->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leRadiusInner, 0, 7, 1, 1);

        cbSymmertix = new QCheckBox(LC_StarOptions);
        cbSymmertix->setObjectName(QString::fromUtf8("cbSymmertix"));

        gridLayout->addWidget(cbSymmertix, 0, 0, 1, 1);

        label = new QLabel(LC_StarOptions);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 9, 1, 1);


        retranslateUi(LC_StarOptions);

        QMetaObject::connectSlotsByName(LC_StarOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_StarOptions)
    {
        LC_StarOptions->setWindowTitle(QCoreApplication::translate("LC_StarOptions", "Form", nullptr));
        cbRadiusInner->setText(QCoreApplication::translate("LC_StarOptions", "Radius Inner", nullptr));
        cbPolyline->setText(QCoreApplication::translate("LC_StarOptions", "Polyline", nullptr));
        cbRadiusOuter->setText(QCoreApplication::translate("LC_StarOptions", "Radius Outer", nullptr));
        cbSymmertix->setText(QCoreApplication::translate("LC_StarOptions", "Symmetric", nullptr));
        label->setText(QCoreApplication::translate("LC_StarOptions", "Number:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_StarOptions: public Ui_LC_StarOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_STAROPTIONS_H
