/********************************************************************************
** Form generated from reading UI file 'lc_rectangle2pointsoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_RECTANGLE2POINTSOPTIONS_H
#define UI_LC_RECTANGLE2POINTSOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_Rectangle2PointsOptions
{
public:
    QGridLayout *gridLayout;
    QCheckBox *cbSnapRadiusCenter;
    QLabel *lblRadius;
    QLineEdit *leX;
    QLabel *lblEdges;
    QCheckBox *cbPolyline;
    QLabel *label_4;
    QLabel *label_3;
    QLineEdit *leLenY;
    QLabel *lblX;
    QLabel *label;
    QLineEdit *leAngle;
    QComboBox *cbSnapStart;
    QLineEdit *leRadius;
    QCheckBox *chkFixedBaseAngle;
    QComboBox *cbSnapEnd;
    QComboBox *cbCorners;
    QLabel *lblLenY;
    QComboBox *cbEdges;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *LC_Rectangle2PointsOptions)
    {
        if (LC_Rectangle2PointsOptions->objectName().isEmpty())
            LC_Rectangle2PointsOptions->setObjectName(QString::fromUtf8("LC_Rectangle2PointsOptions"));
        LC_Rectangle2PointsOptions->resize(1240, 24);
        gridLayout = new QGridLayout(LC_Rectangle2PointsOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        cbSnapRadiusCenter = new QCheckBox(LC_Rectangle2PointsOptions);
        cbSnapRadiusCenter->setObjectName(QString::fromUtf8("cbSnapRadiusCenter"));

        gridLayout->addWidget(cbSnapRadiusCenter, 0, 11, 1, 1);

        lblRadius = new QLabel(LC_Rectangle2PointsOptions);
        lblRadius->setObjectName(QString::fromUtf8("lblRadius"));

        gridLayout->addWidget(lblRadius, 0, 9, 1, 1);

        leX = new QLineEdit(LC_Rectangle2PointsOptions);
        leX->setObjectName(QString::fromUtf8("leX"));
        leX->setMinimumSize(QSize(30, 0));
        leX->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leX, 0, 13, 1, 1);

        lblEdges = new QLabel(LC_Rectangle2PointsOptions);
        lblEdges->setObjectName(QString::fromUtf8("lblEdges"));

        gridLayout->addWidget(lblEdges, 0, 16, 1, 1);

        cbPolyline = new QCheckBox(LC_Rectangle2PointsOptions);
        cbPolyline->setObjectName(QString::fromUtf8("cbPolyline"));

        gridLayout->addWidget(cbPolyline, 0, 6, 1, 1);

        label_4 = new QLabel(LC_Rectangle2PointsOptions);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout->addWidget(label_4, 0, 2, 1, 1);

        label_3 = new QLabel(LC_Rectangle2PointsOptions);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 0, 7, 1, 1);

        leLenY = new QLineEdit(LC_Rectangle2PointsOptions);
        leLenY->setObjectName(QString::fromUtf8("leLenY"));
        leLenY->setMinimumSize(QSize(30, 0));
        leLenY->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leLenY, 0, 15, 1, 1);

        lblX = new QLabel(LC_Rectangle2PointsOptions);
        lblX->setObjectName(QString::fromUtf8("lblX"));

        gridLayout->addWidget(lblX, 0, 12, 1, 1);

        label = new QLabel(LC_Rectangle2PointsOptions);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        leAngle = new QLineEdit(LC_Rectangle2PointsOptions);
        leAngle->setObjectName(QString::fromUtf8("leAngle"));
        leAngle->setMinimumSize(QSize(30, 0));
        leAngle->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leAngle, 0, 5, 1, 1);

        cbSnapStart = new QComboBox(LC_Rectangle2PointsOptions);
        cbSnapStart->addItem(QString());
        cbSnapStart->addItem(QString());
        cbSnapStart->addItem(QString());
        cbSnapStart->addItem(QString());
        cbSnapStart->setObjectName(QString::fromUtf8("cbSnapStart"));

        gridLayout->addWidget(cbSnapStart, 0, 1, 1, 1);

        leRadius = new QLineEdit(LC_Rectangle2PointsOptions);
        leRadius->setObjectName(QString::fromUtf8("leRadius"));
        leRadius->setMinimumSize(QSize(30, 0));
        leRadius->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leRadius, 0, 10, 1, 1);

        chkFixedBaseAngle = new QCheckBox(LC_Rectangle2PointsOptions);
        chkFixedBaseAngle->setObjectName(QString::fromUtf8("chkFixedBaseAngle"));

        gridLayout->addWidget(chkFixedBaseAngle, 0, 4, 1, 1);

        cbSnapEnd = new QComboBox(LC_Rectangle2PointsOptions);
        cbSnapEnd->addItem(QString());
        cbSnapEnd->addItem(QString());
        cbSnapEnd->addItem(QString());
        cbSnapEnd->addItem(QString());
        cbSnapEnd->setObjectName(QString::fromUtf8("cbSnapEnd"));

        gridLayout->addWidget(cbSnapEnd, 0, 3, 1, 1);

        cbCorners = new QComboBox(LC_Rectangle2PointsOptions);
        cbCorners->addItem(QString());
        cbCorners->addItem(QString());
        cbCorners->addItem(QString());
        cbCorners->setObjectName(QString::fromUtf8("cbCorners"));

        gridLayout->addWidget(cbCorners, 0, 8, 1, 1);

        lblLenY = new QLabel(LC_Rectangle2PointsOptions);
        lblLenY->setObjectName(QString::fromUtf8("lblLenY"));

        gridLayout->addWidget(lblLenY, 0, 14, 1, 1);

        cbEdges = new QComboBox(LC_Rectangle2PointsOptions);
        cbEdges->addItem(QString());
        cbEdges->addItem(QString());
        cbEdges->addItem(QString());
        cbEdges->setObjectName(QString::fromUtf8("cbEdges"));

        gridLayout->addWidget(cbEdges, 0, 17, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 18, 1, 1);


        retranslateUi(LC_Rectangle2PointsOptions);

        QMetaObject::connectSlotsByName(LC_Rectangle2PointsOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_Rectangle2PointsOptions)
    {
        LC_Rectangle2PointsOptions->setWindowTitle(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Form", nullptr));
        cbSnapRadiusCenter->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Snap shift", nullptr));
        lblRadius->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Radius:", nullptr));
        lblEdges->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Edges:", nullptr));
        cbPolyline->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Polyline", nullptr));
        label_4->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "End Snap:", nullptr));
        label_3->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Corners:", nullptr));
        lblX->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Length X:", nullptr));
        label->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Start Snap:", nullptr));
        cbSnapStart->setItemText(0, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Corner", nullptr));
        cbSnapStart->setItemText(1, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Mid-Vertical", nullptr));
        cbSnapStart->setItemText(2, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Mid-Horizontal", nullptr));
        cbSnapStart->setItemText(3, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Middle", nullptr));

        chkFixedBaseAngle->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Base Angle:", nullptr));
        cbSnapEnd->setItemText(0, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Corner", nullptr));
        cbSnapEnd->setItemText(1, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Mid-Vertical", nullptr));
        cbSnapEnd->setItemText(2, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Mid-Horizontal", nullptr));
        cbSnapEnd->setItemText(3, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Middle", nullptr));

        cbCorners->setItemText(0, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Straight", nullptr));
        cbCorners->setItemText(1, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Round", nullptr));
        cbCorners->setItemText(2, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Bevel", nullptr));

        lblLenY->setText(QCoreApplication::translate("LC_Rectangle2PointsOptions", "Length Y:", nullptr));
        cbEdges->setItemText(0, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Both", nullptr));
        cbEdges->setItemText(1, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Vertical", nullptr));
        cbEdges->setItemText(2, QCoreApplication::translate("LC_Rectangle2PointsOptions", "Horizontal", nullptr));

    } // retranslateUi

};

namespace Ui {
    class LC_Rectangle2PointsOptions: public Ui_LC_Rectangle2PointsOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_RECTANGLE2POINTSOPTIONS_H
