/********************************************************************************
** Form generated from reading UI file 'lc_slicedivideoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_SLICEDIVIDEOPTIONS_H
#define UI_LC_SLICEDIVIDEOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_SliceDivideOptions
{
public:
    QGridLayout *gridLayout;
    QFrame *frmEdge;
    QGridLayout *gridLayout_3;
    QComboBox *cbEdgeTick;
    QLabel *lblEdgetTick;
    QLineEdit *leTickAngle;
    QLineEdit *leTickLengh;
    QCheckBox *cbRelAngle;
    QLabel *label_4;
    QFrame *frmCount;
    QGridLayout *gridLayout_4;
    QSpinBox *sbCount;
    QLabel *label;
    QFrame *frmCircle;
    QGridLayout *gridLayout_2;
    QLabel *lblCircleStartAngle;
    QLineEdit *leCircleStartAngle;
    QLabel *label_2;
    QLabel *label_5;
    QLabel *label_3;
    QCheckBox *cbDivide;
    QLineEdit *leTickOffset;
    QComboBox *cbTickSnap;
    QCheckBox *cbMode;
    QFrame *frmDistance;
    QGridLayout *gridLayout_5;
    QLabel *label_6;
    QLineEdit *leDistance;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *LC_SliceDivideOptions)
    {
        if (LC_SliceDivideOptions->objectName().isEmpty())
            LC_SliceDivideOptions->setObjectName(QString::fromUtf8("LC_SliceDivideOptions"));
        LC_SliceDivideOptions->resize(1131, 25);
        gridLayout = new QGridLayout(LC_SliceDivideOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        frmEdge = new QFrame(LC_SliceDivideOptions);
        frmEdge->setObjectName(QString::fromUtf8("frmEdge"));
        frmEdge->setFrameShape(QFrame::NoFrame);
        frmEdge->setFrameShadow(QFrame::Raised);
        gridLayout_3 = new QGridLayout(frmEdge);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        cbEdgeTick = new QComboBox(frmEdge);
        cbEdgeTick->addItem(QString());
        cbEdgeTick->addItem(QString());
        cbEdgeTick->addItem(QString());
        cbEdgeTick->addItem(QString());
        cbEdgeTick->setObjectName(QString::fromUtf8("cbEdgeTick"));

        gridLayout_3->addWidget(cbEdgeTick, 0, 1, 1, 1);

        lblEdgetTick = new QLabel(frmEdge);
        lblEdgetTick->setObjectName(QString::fromUtf8("lblEdgetTick"));

        gridLayout_3->addWidget(lblEdgetTick, 0, 0, 1, 1);


        gridLayout->addWidget(frmEdge, 1, 13, 1, 1);

        leTickAngle = new QLineEdit(LC_SliceDivideOptions);
        leTickAngle->setObjectName(QString::fromUtf8("leTickAngle"));
        leTickAngle->setMinimumSize(QSize(30, 0));
        leTickAngle->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leTickAngle, 1, 7, 1, 1);

        leTickLengh = new QLineEdit(LC_SliceDivideOptions);
        leTickLengh->setObjectName(QString::fromUtf8("leTickLengh"));
        leTickLengh->setMinimumSize(QSize(30, 0));
        leTickLengh->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leTickLengh, 1, 5, 1, 1);

        cbRelAngle = new QCheckBox(LC_SliceDivideOptions);
        cbRelAngle->setObjectName(QString::fromUtf8("cbRelAngle"));

        gridLayout->addWidget(cbRelAngle, 1, 8, 1, 1);

        label_4 = new QLabel(LC_SliceDivideOptions);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout->addWidget(label_4, 1, 9, 1, 1);

        frmCount = new QFrame(LC_SliceDivideOptions);
        frmCount->setObjectName(QString::fromUtf8("frmCount"));
        frmCount->setFrameShape(QFrame::NoFrame);
        frmCount->setFrameShadow(QFrame::Raised);
        gridLayout_4 = new QGridLayout(frmCount);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        gridLayout_4->setHorizontalSpacing(6);
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        sbCount = new QSpinBox(frmCount);
        sbCount->setObjectName(QString::fromUtf8("sbCount"));
        sbCount->setMaximum(5000);

        gridLayout_4->addWidget(sbCount, 0, 1, 1, 1);

        label = new QLabel(frmCount);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout_4->addWidget(label, 0, 0, 1, 1);


        gridLayout->addWidget(frmCount, 1, 1, 1, 1);

        frmCircle = new QFrame(LC_SliceDivideOptions);
        frmCircle->setObjectName(QString::fromUtf8("frmCircle"));
        frmCircle->setFrameShape(QFrame::NoFrame);
        frmCircle->setFrameShadow(QFrame::Raised);
        gridLayout_2 = new QGridLayout(frmCircle);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        lblCircleStartAngle = new QLabel(frmCircle);
        lblCircleStartAngle->setObjectName(QString::fromUtf8("lblCircleStartAngle"));

        gridLayout_2->addWidget(lblCircleStartAngle, 0, 0, 1, 1);

        leCircleStartAngle = new QLineEdit(frmCircle);
        leCircleStartAngle->setObjectName(QString::fromUtf8("leCircleStartAngle"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(leCircleStartAngle->sizePolicy().hasHeightForWidth());
        leCircleStartAngle->setSizePolicy(sizePolicy);
        leCircleStartAngle->setMinimumSize(QSize(30, 0));
        leCircleStartAngle->setMaximumSize(QSize(60, 16777215));

        gridLayout_2->addWidget(leCircleStartAngle, 0, 1, 1, 1);


        gridLayout->addWidget(frmCircle, 1, 14, 1, 1);

        label_2 = new QLabel(LC_SliceDivideOptions);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 1, 4, 1, 1);

        label_5 = new QLabel(LC_SliceDivideOptions);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        gridLayout->addWidget(label_5, 1, 11, 1, 1);

        label_3 = new QLabel(LC_SliceDivideOptions);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 1, 6, 1, 1);

        cbDivide = new QCheckBox(LC_SliceDivideOptions);
        cbDivide->setObjectName(QString::fromUtf8("cbDivide"));

        gridLayout->addWidget(cbDivide, 1, 15, 1, 1);

        leTickOffset = new QLineEdit(LC_SliceDivideOptions);
        leTickOffset->setObjectName(QString::fromUtf8("leTickOffset"));
        leTickOffset->setMinimumSize(QSize(30, 0));
        leTickOffset->setMaximumSize(QSize(60, 16777215));

        gridLayout->addWidget(leTickOffset, 1, 10, 1, 1);

        cbTickSnap = new QComboBox(LC_SliceDivideOptions);
        cbTickSnap->addItem(QString());
        cbTickSnap->addItem(QString());
        cbTickSnap->addItem(QString());
        cbTickSnap->setObjectName(QString::fromUtf8("cbTickSnap"));

        gridLayout->addWidget(cbTickSnap, 1, 12, 1, 1);

        cbMode = new QCheckBox(LC_SliceDivideOptions);
        cbMode->setObjectName(QString::fromUtf8("cbMode"));

        gridLayout->addWidget(cbMode, 1, 0, 1, 1);

        frmDistance = new QFrame(LC_SliceDivideOptions);
        frmDistance->setObjectName(QString::fromUtf8("frmDistance"));
        frmDistance->setFrameShape(QFrame::NoFrame);
        frmDistance->setFrameShadow(QFrame::Raised);
        gridLayout_5 = new QGridLayout(frmDistance);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        label_6 = new QLabel(frmDistance);
        label_6->setObjectName(QString::fromUtf8("label_6"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);

        gridLayout_5->addWidget(label_6, 0, 0, 1, 1);

        leDistance = new QLineEdit(frmDistance);
        leDistance->setObjectName(QString::fromUtf8("leDistance"));
        sizePolicy.setHeightForWidth(leDistance->sizePolicy().hasHeightForWidth());
        leDistance->setSizePolicy(sizePolicy);
        leDistance->setMinimumSize(QSize(30, 0));
        leDistance->setMaximumSize(QSize(50, 16777215));

        gridLayout_5->addWidget(leDistance, 0, 1, 1, 1);


        gridLayout->addWidget(frmDistance, 1, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 16, 1, 1);


        retranslateUi(LC_SliceDivideOptions);

        QMetaObject::connectSlotsByName(LC_SliceDivideOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_SliceDivideOptions)
    {
        LC_SliceDivideOptions->setWindowTitle(QCoreApplication::translate("LC_SliceDivideOptions", "Form", nullptr));
        cbEdgeTick->setItemText(0, QCoreApplication::translate("LC_SliceDivideOptions", "None", nullptr));
        cbEdgeTick->setItemText(1, QCoreApplication::translate("LC_SliceDivideOptions", "Both", nullptr));
        cbEdgeTick->setItemText(2, QCoreApplication::translate("LC_SliceDivideOptions", "Start", nullptr));
        cbEdgeTick->setItemText(3, QCoreApplication::translate("LC_SliceDivideOptions", "End", nullptr));

        lblEdgetTick->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Edge Tick:", nullptr));
        cbRelAngle->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Rel", nullptr));
        label_4->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Offset:", nullptr));
        label->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Count:", nullptr));
        lblCircleStartAngle->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Start Circle Angle:", nullptr));
        label_2->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Tick Length:", nullptr));
        label_5->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Tick snap:", nullptr));
        label_3->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Angle:", nullptr));
        cbDivide->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Divide", nullptr));
        cbTickSnap->setItemText(0, QCoreApplication::translate("LC_SliceDivideOptions", "Start", nullptr));
        cbTickSnap->setItemText(1, QCoreApplication::translate("LC_SliceDivideOptions", "Middle", nullptr));
        cbTickSnap->setItemText(2, QCoreApplication::translate("LC_SliceDivideOptions", "End", nullptr));

        cbMode->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Fixed ", nullptr));
        label_6->setText(QCoreApplication::translate("LC_SliceDivideOptions", "Distance:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_SliceDivideOptions: public Ui_LC_SliceDivideOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_SLICEDIVIDEOPTIONS_H
