/********************************************************************************
** Form generated from reading UI file 'lc_penpaletteoptionsdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_PENPALETTEOPTIONSDIALOG_H
#define UI_LC_PENPALETTEOPTIONSDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>

QT_BEGIN_NAMESPACE

class Ui_LC_PenPaletteOptionsDialog
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *verticalSpacer;
    QGroupBox *gbColumns;
    QGridLayout *gridLayout_2;
    QCheckBox *chkShowLineTypeName;
    QCheckBox *chkShowColorName;
    QCheckBox *chkShowLineTypeIcon;
    QCheckBox *chkShowColorIcon;
    QCheckBox *chkShowWidthIcon;
    QCheckBox *chkShowWidthName;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_5;
    QComboBox *cbColorActiveBg;
    QSpacerItem *horizontalSpacer;
    QComboBox *cbColorGrid;
    QToolButton *tbGridColorSelect;
    QLabel *label_2;
    QToolButton *tbActiveColorSelect;
    QCheckBox *cbAllRowBold;
    QLabel *label_3;
    QLabel *label_4;
    QComboBox *cbColorMatchedItem;
    QToolButton *tbMatchedItemColorSelect;
    QCheckBox *cbFilterCaseInsensitive;
    QDialogButtonBox *buttonBox;
    QCheckBox *cbShowTooltip;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLineEdit *lePensFile;
    QToolButton *tbBrowse;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_3;
    QRadioButton *rbHEX;
    QRadioButton *rbRGB;
    QRadioButton *rbNatural;
    QCheckBox *cbShowMessageForNoSelection;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_5;
    QComboBox *cbDoubleClickMode;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QDialog *LC_PenPaletteOptionsDialog)
    {
        if (LC_PenPaletteOptionsDialog->objectName().isEmpty())
            LC_PenPaletteOptionsDialog->setObjectName(QString::fromUtf8("LC_PenPaletteOptionsDialog"));
        LC_PenPaletteOptionsDialog->resize(349, 469);
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(LC_PenPaletteOptionsDialog->sizePolicy().hasHeightForWidth());
        LC_PenPaletteOptionsDialog->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(LC_PenPaletteOptionsDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(verticalSpacer, 10, 0, 1, 1);

        gbColumns = new QGroupBox(LC_PenPaletteOptionsDialog);
        gbColumns->setObjectName(QString::fromUtf8("gbColumns"));
        gridLayout_2 = new QGridLayout(gbColumns);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        chkShowLineTypeName = new QCheckBox(gbColumns);
        chkShowLineTypeName->setObjectName(QString::fromUtf8("chkShowLineTypeName"));

        gridLayout_2->addWidget(chkShowLineTypeName, 1, 2, 1, 1);

        chkShowColorName = new QCheckBox(gbColumns);
        chkShowColorName->setObjectName(QString::fromUtf8("chkShowColorName"));

        gridLayout_2->addWidget(chkShowColorName, 1, 0, 1, 1);

        chkShowLineTypeIcon = new QCheckBox(gbColumns);
        chkShowLineTypeIcon->setObjectName(QString::fromUtf8("chkShowLineTypeIcon"));

        gridLayout_2->addWidget(chkShowLineTypeIcon, 0, 2, 1, 1);

        chkShowColorIcon = new QCheckBox(gbColumns);
        chkShowColorIcon->setObjectName(QString::fromUtf8("chkShowColorIcon"));

        gridLayout_2->addWidget(chkShowColorIcon, 0, 0, 1, 1);

        chkShowWidthIcon = new QCheckBox(gbColumns);
        chkShowWidthIcon->setObjectName(QString::fromUtf8("chkShowWidthIcon"));

        gridLayout_2->addWidget(chkShowWidthIcon, 0, 1, 1, 1);

        chkShowWidthName = new QCheckBox(gbColumns);
        chkShowWidthName->setObjectName(QString::fromUtf8("chkShowWidthName"));

        gridLayout_2->addWidget(chkShowWidthName, 1, 1, 1, 1);


        gridLayout->addWidget(gbColumns, 6, 0, 1, 1);

        groupBox_2 = new QGroupBox(LC_PenPaletteOptionsDialog);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        gridLayout_5 = new QGridLayout(groupBox_2);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        cbColorActiveBg = new QComboBox(groupBox_2);
        cbColorActiveBg->setObjectName(QString::fromUtf8("cbColorActiveBg"));
        cbColorActiveBg->setEditable(true);

        gridLayout_5->addWidget(cbColorActiveBg, 0, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_5->addItem(horizontalSpacer, 0, 3, 1, 1);

        cbColorGrid = new QComboBox(groupBox_2);
        cbColorGrid->setObjectName(QString::fromUtf8("cbColorGrid"));
        cbColorGrid->setEditable(true);

        gridLayout_5->addWidget(cbColorGrid, 1, 1, 1, 1);

        tbGridColorSelect = new QToolButton(groupBox_2);
        tbGridColorSelect->setObjectName(QString::fromUtf8("tbGridColorSelect"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/ui/colorxx.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbGridColorSelect->setIcon(icon);

        gridLayout_5->addWidget(tbGridColorSelect, 1, 2, 1, 1);

        label_2 = new QLabel(groupBox_2);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout_5->addWidget(label_2, 0, 0, 1, 1);

        tbActiveColorSelect = new QToolButton(groupBox_2);
        tbActiveColorSelect->setObjectName(QString::fromUtf8("tbActiveColorSelect"));
        tbActiveColorSelect->setIcon(icon);

        gridLayout_5->addWidget(tbActiveColorSelect, 0, 2, 1, 1);

        cbAllRowBold = new QCheckBox(groupBox_2);
        cbAllRowBold->setObjectName(QString::fromUtf8("cbAllRowBold"));

        gridLayout_5->addWidget(cbAllRowBold, 3, 0, 1, 2);

        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout_5->addWidget(label_3, 1, 0, 1, 1);

        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout_5->addWidget(label_4, 2, 0, 1, 1);

        cbColorMatchedItem = new QComboBox(groupBox_2);
        cbColorMatchedItem->setObjectName(QString::fromUtf8("cbColorMatchedItem"));
        cbColorMatchedItem->setEditable(true);

        gridLayout_5->addWidget(cbColorMatchedItem, 2, 1, 1, 1);

        tbMatchedItemColorSelect = new QToolButton(groupBox_2);
        tbMatchedItemColorSelect->setObjectName(QString::fromUtf8("tbMatchedItemColorSelect"));
        tbMatchedItemColorSelect->setIcon(icon);

        gridLayout_5->addWidget(tbMatchedItemColorSelect, 2, 2, 1, 1);


        gridLayout->addWidget(groupBox_2, 7, 0, 1, 1);

        cbFilterCaseInsensitive = new QCheckBox(LC_PenPaletteOptionsDialog);
        cbFilterCaseInsensitive->setObjectName(QString::fromUtf8("cbFilterCaseInsensitive"));

        gridLayout->addWidget(cbFilterCaseInsensitive, 0, 0, 1, 1);

        buttonBox = new QDialogButtonBox(LC_PenPaletteOptionsDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 11, 0, 1, 1);

        cbShowTooltip = new QCheckBox(LC_PenPaletteOptionsDialog);
        cbShowTooltip->setObjectName(QString::fromUtf8("cbShowTooltip"));

        gridLayout->addWidget(cbShowTooltip, 1, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label = new QLabel(LC_PenPaletteOptionsDialog);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        lePensFile = new QLineEdit(LC_PenPaletteOptionsDialog);
        lePensFile->setObjectName(QString::fromUtf8("lePensFile"));

        horizontalLayout->addWidget(lePensFile);

        tbBrowse = new QToolButton(LC_PenPaletteOptionsDialog);
        tbBrowse->setObjectName(QString::fromUtf8("tbBrowse"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/actions/fileopen.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbBrowse->setIcon(icon1);

        horizontalLayout->addWidget(tbBrowse);


        gridLayout->addLayout(horizontalLayout, 9, 0, 1, 1);

        groupBox = new QGroupBox(LC_PenPaletteOptionsDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout_3 = new QGridLayout(groupBox);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        rbHEX = new QRadioButton(groupBox);
        rbHEX->setObjectName(QString::fromUtf8("rbHEX"));

        gridLayout_3->addWidget(rbHEX, 0, 1, 1, 1);

        rbRGB = new QRadioButton(groupBox);
        rbRGB->setObjectName(QString::fromUtf8("rbRGB"));

        gridLayout_3->addWidget(rbRGB, 0, 0, 1, 1);

        rbNatural = new QRadioButton(groupBox);
        rbNatural->setObjectName(QString::fromUtf8("rbNatural"));

        gridLayout_3->addWidget(rbNatural, 0, 2, 1, 1);


        gridLayout->addWidget(groupBox, 4, 0, 2, 1);

        cbShowMessageForNoSelection = new QCheckBox(LC_PenPaletteOptionsDialog);
        cbShowMessageForNoSelection->setObjectName(QString::fromUtf8("cbShowMessageForNoSelection"));

        gridLayout->addWidget(cbShowMessageForNoSelection, 2, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label_5 = new QLabel(LC_PenPaletteOptionsDialog);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        horizontalLayout_2->addWidget(label_5);

        cbDoubleClickMode = new QComboBox(LC_PenPaletteOptionsDialog);
        cbDoubleClickMode->setObjectName(QString::fromUtf8("cbDoubleClickMode"));

        horizontalLayout_2->addWidget(cbDoubleClickMode);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        gridLayout->addLayout(horizontalLayout_2, 3, 0, 1, 1);


        retranslateUi(LC_PenPaletteOptionsDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), LC_PenPaletteOptionsDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), LC_PenPaletteOptionsDialog, SLOT(reject()));

        cbDoubleClickMode->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(LC_PenPaletteOptionsDialog);
    } // setupUi

    void retranslateUi(QDialog *LC_PenPaletteOptionsDialog)
    {
        LC_PenPaletteOptionsDialog->setWindowTitle(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Pen Palette Settings", nullptr));
        gbColumns->setTitle(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Columns to display", nullptr));
        chkShowLineTypeName->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Line Type Name", nullptr));
        chkShowColorName->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Color Name", nullptr));
        chkShowLineTypeIcon->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Line Type Icon", nullptr));
        chkShowColorIcon->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Color Icon", nullptr));
        chkShowWidthIcon->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Line Width Icon", nullptr));
        chkShowWidthName->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Line Width Name", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Colors", nullptr));
        tbGridColorSelect->setText(QString());
        label_2->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Active Pen Background:", nullptr));
        tbActiveColorSelect->setText(QString());
        cbAllRowBold->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Entire table row is bold for active pen", nullptr));
        label_3->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Grid Color:", nullptr));
        label_4->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Matched Item Color:", nullptr));
        tbMatchedItemColorSelect->setText(QString());
        cbFilterCaseInsensitive->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Case-insensitive pens filtering", nullptr));
#if QT_CONFIG(tooltip)
        cbShowTooltip->setToolTip(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Controls whether tooltip will be shown for table item or not", nullptr));
#endif // QT_CONFIG(tooltip)
        cbShowTooltip->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Show tooltip for table items", nullptr));
        label->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Pens file:", nullptr));
        tbBrowse->setText(QString());
        groupBox->setTitle(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Show colorn name as", nullptr));
        rbHEX->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "HEX", nullptr));
        rbRGB->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "RGB", nullptr));
        rbNatural->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Natural name where possible", nullptr));
        cbShowMessageForNoSelection->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "Show info message if no entities may be selected by pen", nullptr));
        label_5->setText(QCoreApplication::translate("LC_PenPaletteOptionsDialog", "On double click on table item", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_PenPaletteOptionsDialog: public Ui_LC_PenPaletteOptionsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_PENPALETTEOPTIONSDIALOG_H
