/********************************************************************************
** Form generated from reading UI file 'lc_crossoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_CROSSOPTIONS_H
#define UI_LC_CROSSOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LC_CrossOptions
{
public:
    QGridLayout *gridLayout;
    QComboBox *cbMode;
    QLineEdit *leAngle;
    QLineEdit *leY;
    QLabel *label;
    QLabel *label_2;
    QLineEdit *leX;
    QLabel *label_3;
    QLabel *label_4;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *LC_CrossOptions)
    {
        if (LC_CrossOptions->objectName().isEmpty())
            LC_CrossOptions->setObjectName(QString::fromUtf8("LC_CrossOptions"));
        LC_CrossOptions->resize(423, 24);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(2);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(LC_CrossOptions->sizePolicy().hasHeightForWidth());
        LC_CrossOptions->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(LC_CrossOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(-1, 1, -1, 1);
        cbMode = new QComboBox(LC_CrossOptions);
        cbMode->addItem(QString());
        cbMode->addItem(QString());
        cbMode->addItem(QString());
        cbMode->setObjectName(QString::fromUtf8("cbMode"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(3);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(cbMode->sizePolicy().hasHeightForWidth());
        cbMode->setSizePolicy(sizePolicy1);
        cbMode->setMinimumSize(QSize(110, 0));
        cbMode->setMaximumSize(QSize(130, 16777215));
        cbMode->setMaxVisibleItems(3);
        cbMode->setInsertPolicy(QComboBox::InsertAlphabetically);
        cbMode->setSizeAdjustPolicy(QComboBox::AdjustToContents);
        cbMode->setMinimumContentsLength(20);

        gridLayout->addWidget(cbMode, 0, 5, 1, 1);

        leAngle = new QLineEdit(LC_CrossOptions);
        leAngle->setObjectName(QString::fromUtf8("leAngle"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(leAngle->sizePolicy().hasHeightForWidth());
        leAngle->setSizePolicy(sizePolicy2);
        leAngle->setMinimumSize(QSize(30, 0));
        leAngle->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leAngle, 0, 7, 1, 1);

        leY = new QLineEdit(LC_CrossOptions);
        leY->setObjectName(QString::fromUtf8("leY"));
        sizePolicy2.setHeightForWidth(leY->sizePolicy().hasHeightForWidth());
        leY->setSizePolicy(sizePolicy2);
        leY->setMinimumSize(QSize(30, 0));
        leY->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leY, 0, 3, 1, 1);

        label = new QLabel(LC_CrossOptions);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(LC_CrossOptions);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 2, 1, 1);

        leX = new QLineEdit(LC_CrossOptions);
        leX->setObjectName(QString::fromUtf8("leX"));
        sizePolicy2.setHeightForWidth(leX->sizePolicy().hasHeightForWidth());
        leX->setSizePolicy(sizePolicy2);
        leX->setMinimumSize(QSize(30, 0));
        leX->setMaximumSize(QSize(50, 16777215));

        gridLayout->addWidget(leX, 0, 1, 1, 1);

        label_3 = new QLabel(LC_CrossOptions);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 0, 4, 1, 1);

        label_4 = new QLabel(LC_CrossOptions);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout->addWidget(label_4, 0, 6, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 8, 1, 1);


        retranslateUi(LC_CrossOptions);

        QMetaObject::connectSlotsByName(LC_CrossOptions);
    } // setupUi

    void retranslateUi(QWidget *LC_CrossOptions)
    {
        LC_CrossOptions->setWindowTitle(QCoreApplication::translate("LC_CrossOptions", "Form", nullptr));
        cbMode->setItemText(0, QCoreApplication::translate("LC_CrossOptions", "Extension", nullptr));
        cbMode->setItemText(1, QCoreApplication::translate("LC_CrossOptions", "Total Length", nullptr));
        cbMode->setItemText(2, QCoreApplication::translate("LC_CrossOptions", "Percent", nullptr));

        label->setText(QCoreApplication::translate("LC_CrossOptions", "X:", nullptr));
        label_2->setText(QCoreApplication::translate("LC_CrossOptions", "Y:", nullptr));
        label_3->setText(QCoreApplication::translate("LC_CrossOptions", "Type", nullptr));
        label_4->setText(QCoreApplication::translate("LC_CrossOptions", "Angle:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_CrossOptions: public Ui_LC_CrossOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_CROSSOPTIONS_H
