/********************************************************************************
** Form generated from reading UI file 'lc_layerdialog_ex.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_LAYERDIALOG_EX_H
#define UI_LC_LAYERDIALOG_EX_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include "qg_widgetpen.h"

QT_BEGIN_NAMESPACE

class Ui_LC_LayerDialogEx
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *spacer87;
    QLineEdit *leName;
    QLabel *lParentPathName;
    QDialogButtonBox *buttonBox;
    QGroupBox *gbLayerType;
    QGridLayout *gridLayout_2;
    QRadioButton *rbInformational;
    QRadioButton *rbNormal;
    QRadioButton *rbDimensions;
    QRadioButton *rbAlternativePosition;
    QCheckBox *cbConstructionLayer;
    QG_WidgetPen *wPen;
    QSpacerItem *spacer187;
    QLineEdit *leParentPath;
    QLabel *lName;
    QButtonGroup *bgLayerType;

    void setupUi(QDialog *LC_LayerDialogEx)
    {
        if (LC_LayerDialogEx->objectName().isEmpty())
            LC_LayerDialogEx->setObjectName(QString::fromUtf8("LC_LayerDialogEx"));
        LC_LayerDialogEx->resize(323, 221);
        LC_LayerDialogEx->setBaseSize(QSize(0, 0));
        LC_LayerDialogEx->setSizeGripEnabled(false);
        LC_LayerDialogEx->setModal(true);
        gridLayout = new QGridLayout(LC_LayerDialogEx);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        spacer87 = new QSpacerItem(5, 16, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacer87, 1, 1, 1, 1);

        leName = new QLineEdit(LC_LayerDialogEx);
        leName->setObjectName(QString::fromUtf8("leName"));

        gridLayout->addWidget(leName, 1, 2, 1, 1);

        lParentPathName = new QLabel(LC_LayerDialogEx);
        lParentPathName->setObjectName(QString::fromUtf8("lParentPathName"));
        lParentPathName->setFrameShape(QFrame::NoFrame);
        lParentPathName->setFrameShadow(QFrame::Plain);
        lParentPathName->setWordWrap(false);

        gridLayout->addWidget(lParentPathName, 0, 0, 1, 1);

        buttonBox = new QDialogButtonBox(LC_LayerDialogEx);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 6, 0, 1, 3);

        gbLayerType = new QGroupBox(LC_LayerDialogEx);
        gbLayerType->setObjectName(QString::fromUtf8("gbLayerType"));
        gridLayout_2 = new QGridLayout(gbLayerType);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        rbInformational = new QRadioButton(gbLayerType);
        bgLayerType = new QButtonGroup(LC_LayerDialogEx);
        bgLayerType->setObjectName(QString::fromUtf8("bgLayerType"));
        bgLayerType->addButton(rbInformational);
        rbInformational->setObjectName(QString::fromUtf8("rbInformational"));
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(rbInformational->sizePolicy().hasHeightForWidth());
        rbInformational->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(rbInformational, 0, 1, 1, 1);

        rbNormal = new QRadioButton(gbLayerType);
        bgLayerType->addButton(rbNormal);
        rbNormal->setObjectName(QString::fromUtf8("rbNormal"));

        gridLayout_2->addWidget(rbNormal, 0, 0, 1, 1);

        rbDimensions = new QRadioButton(gbLayerType);
        bgLayerType->addButton(rbDimensions);
        rbDimensions->setObjectName(QString::fromUtf8("rbDimensions"));

        gridLayout_2->addWidget(rbDimensions, 1, 0, 1, 1);

        rbAlternativePosition = new QRadioButton(gbLayerType);
        bgLayerType->addButton(rbAlternativePosition);
        rbAlternativePosition->setObjectName(QString::fromUtf8("rbAlternativePosition"));

        gridLayout_2->addWidget(rbAlternativePosition, 1, 1, 1, 1);


        gridLayout->addWidget(gbLayerType, 3, 0, 1, 3);

        cbConstructionLayer = new QCheckBox(LC_LayerDialogEx);
        cbConstructionLayer->setObjectName(QString::fromUtf8("cbConstructionLayer"));

        gridLayout->addWidget(cbConstructionLayer, 2, 2, 1, 1);

        wPen = new QG_WidgetPen(LC_LayerDialogEx);
        wPen->setObjectName(QString::fromUtf8("wPen"));

        gridLayout->addWidget(wPen, 5, 0, 1, 3);

        spacer187 = new QSpacerItem(5, 16, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacer187, 0, 1, 1, 1);

        leParentPath = new QLineEdit(LC_LayerDialogEx);
        leParentPath->setObjectName(QString::fromUtf8("leParentPath"));
        leParentPath->setEnabled(false);

        gridLayout->addWidget(leParentPath, 0, 2, 1, 1);

        lName = new QLabel(LC_LayerDialogEx);
        lName->setObjectName(QString::fromUtf8("lName"));
        lName->setFrameShape(QFrame::NoFrame);
        lName->setFrameShadow(QFrame::Plain);
        lName->setWordWrap(false);

        gridLayout->addWidget(lName, 1, 0, 1, 1);


        retranslateUi(LC_LayerDialogEx);
        QObject::connect(buttonBox, SIGNAL(rejected()), LC_LayerDialogEx, SLOT(reject()));
        QObject::connect(buttonBox, SIGNAL(accepted()), LC_LayerDialogEx, SLOT(validate()));

        QMetaObject::connectSlotsByName(LC_LayerDialogEx);
    } // setupUi

    void retranslateUi(QDialog *LC_LayerDialogEx)
    {
        LC_LayerDialogEx->setWindowTitle(QCoreApplication::translate("LC_LayerDialogEx", "Layer Settings", nullptr));
        leName->setText(QString());
        lParentPathName->setText(QCoreApplication::translate("LC_LayerDialogEx", "Parent Layers Path:", nullptr));
        gbLayerType->setTitle(QCoreApplication::translate("LC_LayerDialogEx", "Layer Type", nullptr));
        rbInformational->setText(QCoreApplication::translate("LC_LayerDialogEx", "Informational", nullptr));
        rbNormal->setText(QCoreApplication::translate("LC_LayerDialogEx", "Normal", nullptr));
        rbDimensions->setText(QCoreApplication::translate("LC_LayerDialogEx", "Dimensions", nullptr));
        rbAlternativePosition->setText(QCoreApplication::translate("LC_LayerDialogEx", "Alternative Position", nullptr));
#if QT_CONFIG(tooltip)
        cbConstructionLayer->setToolTip(QCoreApplication::translate("LC_LayerDialogEx", "A Construction Layer has entities of infinite straight lines intended to be used for geometric construction.\n"
"       The contents of a Construction Layer should not appear in printout.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbConstructionLayer->setText(QCoreApplication::translate("LC_LayerDialogEx", "Construction Layer", nullptr));
        leParentPath->setText(QString());
        lName->setText(QCoreApplication::translate("LC_LayerDialogEx", "Layer Name:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LC_LayerDialogEx: public Ui_LC_LayerDialogEx {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_LAYERDIALOG_EX_H
